@@ -1,4 +1,4 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 
 import argparse
 import base64
@@ -36,6 +36,11 @@ def die(msg):
     sys.exit(msgfmt(msg, "error: "))
 
 
+def check_dir(dir):
+    if not os.path.isdir(dir):
+        die("invalid directory: {0}".format(dir))
+
+
 def die_atnode(msg, node):
     if debug:
         msg += '\n\n'
@@ -44,46 +49,92 @@ def die_atnode(msg, node):
     die(msg)
 
 
+def validate_sync_filter():
+    valid_sync_filter_values = {'event', 'apt', 'recur-event', 'recur-apt', 'todo', 'recur', 'cal'}
+    return set(sync_filter.split(',')) - valid_sync_filter_values
+
+
 def calcurse_wipe():
     if verbose:
         print('Removing all local calcurse objects...')
     if dry_run:
         return
-    subprocess.call([calcurse, '-F', '--filter-hash=XXX'])
+
+    command = calcurse + ['-F', '--filter-hash=XXX']
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    subprocess.call(command)
 
 
 def calcurse_import(icaldata):
-    p = subprocess.Popen([calcurse, '-i', '-', '--dump-imported', '-q',
-                          '--format-apt=%(hash)\\n',
-                          '--format-recur-apt=%(hash)\\n',
-                          '--format-event=%(hash)\\n',
-                          '--format-recur-event=%(hash)\\n',
-                          '--format-todo=%(hash)\\n'],
-                         stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+    command = calcurse + [
+        '-i', '-',
+        '--dump-imported',
+        '-q',
+        '--format-apt=%(hash)\\n',
+        '--format-recur-apt=%(hash)\\n',
+        '--format-event=%(hash)\\n',
+        '--format-recur-event=%(hash)\\n',
+        '--format-todo=%(hash)\\n'
+    ]
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    p = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
     return p.communicate(icaldata.encode('utf-8'))[0].decode('utf-8').rstrip()
 
 
 def calcurse_export(objhash):
-    p = subprocess.Popen([calcurse, '-x', 'ical', '--export-uid',
-                          '--filter-hash=' + objhash], stdout=subprocess.PIPE)
+    command = calcurse + [
+        '-xical',
+        '--export-uid',
+        '--filter-hash=' + objhash
+    ]
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    p = subprocess.Popen(command, stdout=subprocess.PIPE)
     return p.communicate()[0].decode('utf-8').rstrip()
 
 
 def calcurse_hashset():
-    p = subprocess.Popen([calcurse, '-G', '--format-apt=%(hash)\\n',
-                          '--format-recur-apt=%(hash)\\n',
-                          '--format-event=%(hash)\\n',
-                          '--format-recur-event=%(hash)\\n',
-                          '--format-todo=%(hash)\\n'], stdout=subprocess.PIPE)
+    command = calcurse + [
+        '-G',
+        '--filter-type', sync_filter,
+        '--format-apt=%(hash)\\n',
+        '--format-recur-apt=%(hash)\\n',
+        '--format-event=%(hash)\\n',
+        '--format-recur-event=%(hash)\\n',
+        '--format-todo=%(hash)\\n'
+    ]
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    p = subprocess.Popen(command, stdout=subprocess.PIPE)
     return set(p.communicate()[0].decode('utf-8').rstrip().splitlines())
 
 
 def calcurse_remove(objhash):
-    subprocess.call([calcurse, '-F', '--filter-hash=!' + objhash])
+    command = calcurse + ['-F', '--filter-hash=!' + objhash]
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    subprocess.call(command)
 
 
 def calcurse_version():
-    p = subprocess.Popen([calcurse, '--version'], stdout=subprocess.PIPE)
+    command = calcurse + ['--version']
+
+    if debug:
+        print('Running command: {}'.format(command))
+
+    p = subprocess.Popen(command, stdout=subprocess.PIPE)
     m = re.match(r'calcurse ([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9]+)-)?',
                  p.communicate()[0].decode('utf-8'))
     if not m:
@@ -170,7 +221,10 @@ def remote_query(conn, cmd, path, additional_headers, body):
 
     if debug:
         print("> {} {}".format(cmd, path))
-        print("> Headers: " + repr(headers))
+        headers_sanitized = headers.copy()
+        if not debug_raw:
+            headers_sanitized.pop('Authorization', None)
+        print("> Headers: " + repr(headers_sanitized))
         if body:
             for line in body.splitlines():
                 print("> " + line)
@@ -186,6 +240,7 @@ def remote_query(conn, cmd, path, additional_headers, body):
         return (None, None)
 
     if debug:
+        print("< Status: {} ({})".format(resp.status, resp.reason))
         print("< Headers: " + repr(resp))
         for line in body.splitlines():
             print("< " + line)
@@ -412,8 +467,15 @@ def pull_objects(hrefs_missing, hrefs_modified, conn, syncdb, etagdict):
                 continue
 
         objhash = calcurse_import(cdata)
-        syncdb_add(syncdb, href, etag, objhash)
-        added += 1
+
+        # TODO: Add support for importing multiple events at once, see GitHub
+        # issue #20 for details.
+        if re.match(r'[0-ga-f]+$', objhash):
+            syncdb_add(syncdb, href, etag, objhash)
+            added += 1
+        else:
+            print("Failed to import object: {} ({})".format(etag, href),
+                  file=sys.stderr)
 
     return added
 
@@ -461,6 +523,9 @@ parser.add_argument('--init', action='store', dest='init', default=None,
 parser.add_argument('--config', action='store', dest='configfn',
                     default=configfn,
                     help='path to the calcurse-caldav configuration')
+parser.add_argument('--datadir', action='store', dest='datadir',
+                    default=None,
+                    help='path to the calcurse data directory')
 parser.add_argument('--lockfile', action='store', dest='lockfn',
                     default=lockfn,
                     help='path to the calcurse-caldav lock file')
@@ -470,9 +535,6 @@ parser.add_argument('--syncdb', action='store', dest='syncdbfn',
 parser.add_argument('--hookdir', action='store', dest='hookdir',
                     default=hookdir,
                     help='path to the calcurse-caldav hooks directory')
-parser.add_argument('--password', action='store', dest='password',
-                    default=None,
-                    help='password for basic authentication')
 parser.add_argument('--authcode', action='store', dest='authcode',
                     default=None,
                     help='auth code for OAuth2 authentication')
@@ -481,41 +543,52 @@ parser.add_argument('-v', '--verbose', action='store_true', dest='verbose',
                     help='print status messages to stdout')
 parser.add_argument('--debug', action='store_true', dest='debug',
                     default=False, help='print debug messages to stdout')
+parser.add_argument('--debug-raw', action='store_true', dest='debug_raw',
+                    default=False, help='do not sanitize debug messages')
 args = parser.parse_args()
 
 init = args.init is not None
 configfn = args.configfn
 lockfn = args.lockfn
 syncdbfn = args.syncdbfn
+datadir = args.datadir
 hookdir = args.hookdir
-password = args.password
 authcode = args.authcode
 verbose = args.verbose
 debug = args.debug
+debug_raw = args.debug_raw
+
+# Read environment variables
+password = os.getenv('CALCURSE_CALDAV_PASSWORD')
 
 # Read configuration.
 config = configparser.RawConfigParser()
 if verbose:
     print('Loading configuration from ' + configfn + '...')
 try:
-    config.readfp(open(configfn))
+    config.read_file(open(configfn))
 except FileNotFoundError as e:
     die('Configuration file not found: {}'.format(configfn))
 
-hostname = config.get('General', 'HostName')
-path = '/' + config.get('General', 'Path').strip('/') + '/'
-hostname_uri = 'https://' + hostname
-absolute_uri = hostname_uri + path
-
 if config.has_option('General', 'InsecureSSL'):
     insecure_ssl = config.getboolean('General', 'InsecureSSL')
 else:
     insecure_ssl = False
 
-if config.has_option('General', 'Binary'):
-    calcurse = config.get('General', 'Binary')
+# Read config for "HTTPS" option (default=True)
+if config.has_option('General', 'HTTPS'):
+    https = config.getboolean('General', 'HTTPS')
 else:
-    calcurse = 'calcurse'
+    https = True
+
+if config.has_option('General', 'Binary'):
+    calcurse = [config.get('General', 'Binary')]
+else:
+    calcurse = ['calcurse']
+
+if datadir:
+    check_dir(datadir)
+    calcurse += ['-D', datadir]
 
 if config.has_option('General', 'DryRun'):
     dry_run = config.getboolean('General', 'DryRun')
@@ -533,6 +606,16 @@ if config.has_option('General', 'AuthMethod'):
 else:
     authmethod = 'basic'
 
+if config.has_option('General', 'SyncFilter'):
+    sync_filter = config.get('General', 'SyncFilter')
+
+    invalid_filter_values = validate_sync_filter()
+
+    if len(invalid_filter_values):
+        die('Invalid value(s) in SyncFilter option: ' + ', '.join(invalid_filter_values))
+else:
+    sync_filter = 'cal,todo'
+
 if config.has_option('Auth', 'UserName'):
     username = config.get('Auth', 'UserName')
 else:
@@ -566,6 +649,17 @@ if config.has_option('OAuth2', 'RedirectURI'):
 else:
     redirect_uri = 'http://127.0.0.1'
 
+# Change URl prefix according to HTTP/HTTPS
+if https:
+    urlprefix = "https://"
+else:
+    urlprefix = "http://"
+
+hostname = config.get('General', 'HostName')
+path = '/' + config.get('General', 'Path').strip('/') + '/'
+hostname_uri = urlprefix + hostname
+absolute_uri = hostname_uri + path
+
 # Show disclaimer when performing a dry run.
 if dry_run:
     warn(('Dry run; nothing is imported/exported. Add "DryRun = No" to the '
@@ -592,7 +686,7 @@ if os.path.exists(lockfn):
 open(lockfn, 'w')
 
 try:
-    # Connect to the server via HTTPs.
+    # Connect to the server.
     if verbose:
         print('Connecting to ' + hostname + '...')
     conn = httplib2.Http()
@@ -603,7 +697,10 @@ try:
         # Authenticate with OAuth2 and authorize HTTP object
         cred = run_auth(authcode)
         conn = cred.authorize(conn)
-    elif authmethod != 'basic':
+    elif authmethod == 'basic':
+        # Add credentials to httplib2
+        conn.add_credentials(username, password)
+    else:
         die('Invalid option for AuthMethod in config file. Use "basic" or "oauth2"')
 
     if init:
