@@ -98,7 +98,7 @@ static int apoint_cmp(struct apoint *a, struct apoint *b)
 	return strcmp(a->mesg, b->mesg);
 }
 
-struct apoint *apoint_new(char *mesg, char *note, long start, long dur,
+struct apoint *apoint_new(char *mesg, char *note, time_t start, long dur,
 			  char state)
 {
 	struct apoint *apt;
@@ -117,14 +117,14 @@ struct apoint *apoint_new(char *mesg, char *note, long start, long dur,
 	return apt;
 }
 
-unsigned apoint_inday(struct apoint *i, long *start)
+unsigned apoint_inday(struct apoint *i, time_t *start)
 {
 	return (date_cmp_day(i->start, *start) == 0 ||
 		(date_cmp_day(i->start, *start) < 0 &&
 		 date_cmp_day(i->start + i->dur - 1, *start) >= 0));
 }
 
-void apoint_sec2str(struct apoint *o, long day, char *start, char *end)
+void apoint_sec2str(struct apoint *o, time_t day, char *start, char *end)
 {
 	struct tm lt;
 	time_t t;
@@ -137,7 +137,7 @@ void apoint_sec2str(struct apoint *o, long day, char *start, char *end)
 		snprintf(start, HRMIN_SIZE, "%02u:%02u", lt.tm_hour,
 			 lt.tm_min);
 	}
-	if (o->start + o->dur > day + DAYINSEC) {
+	if (o->start + o->dur > day + DAYLEN(day)) {
 		strncpy(end, "..:..", 6);
 	} else {
 		t = o->start + o->dur;
@@ -200,7 +200,8 @@ struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
 {
 	char buf[BUFSIZ], *newline;
 	time_t tstart, tend;
-	struct apoint *apt;
+	struct apoint *apt = NULL;
+	int cond;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_date(end.tm_year, end.tm_mon, end.tm_mday) ||
@@ -230,31 +231,30 @@ struct apoint *apoint_scan(FILE * f, struct tm start, struct tm end,
 
 	/* Filter item. */
 	if (filter) {
-		if (!(filter->type_mask & TYPE_MASK_APPT))
-			return NULL;
-		if (filter->regex && regexec(filter->regex, buf, 0, 0, 0))
-			return NULL;
-		if (filter->start_from >= 0 && tstart < filter->start_from)
-			return NULL;
-		if (filter->start_to >= 0 && tstart > filter->start_to)
+		cond = (
+		    !(filter->type_mask & TYPE_MASK_APPT) ||
+		    (filter->regex && regexec(filter->regex, buf, 0, 0, 0)) ||
+		    (filter->start_from != -1 && tstart < filter->start_from) ||
+		    (filter->start_to != -1 && tstart > filter->start_to) ||
+		    (filter->end_from != -1 && tend < filter->end_from) ||
+		    (filter->end_to != -1 && tend > filter->end_to)
+		);
+		if (filter->hash) {
+			apt = apoint_new(
+				buf, note, tstart, tend - tstart, state);
+			char *hash = apoint_hash(apt);
+			cond = cond || !hash_matches(filter->hash, hash);
+			mem_free(hash);
+		}
+
+		if ((!filter->invert && cond) || (filter->invert && !cond)) {
+			if (filter->hash)
+				apoint_delete(apt);
 			return NULL;
-		if (filter->end_from >= 0 && tend < filter->end_from)
-			return NULL;
-		if (filter->end_to >= 0 && tend > filter->end_to)
-			return NULL;
-	}
-
-	apt = apoint_new(buf, note, tstart, tend - tstart, state);
-
-	/* Filter by hash. */
-	if (filter && filter->hash) {
-		char *hash = apoint_hash(apt);
-		if (!hash_matches(filter->hash, hash)) {
-			apoint_delete(apt);
-			apt = NULL;
 		}
-		mem_free(hash);
 	}
+	if (!apt)
+		apt = apoint_new(buf, note, tstart, tend - tstart, state);
 
 	return apt;
 }
@@ -278,7 +278,7 @@ void apoint_delete(struct apoint *apt)
 	LLIST_TS_UNLOCK(&alist_p);
 }
 
-static int apoint_starts_after(struct apoint *apt, long *time)
+static int apoint_starts_after(struct apoint *apt, time_t *time)
 {
 	return apt->start > *time;
 }
@@ -287,7 +287,7 @@ static int apoint_starts_after(struct apoint *apt, long *time)
  * Look in the appointment list if we have an item which starts before the item
  * stored in the notify_app structure (which is the next item to be notified).
  */
-struct notify_app *apoint_check_next(struct notify_app *app, long start)
+struct notify_app *apoint_check_next(struct notify_app *app, time_t start)
 {
 	llist_item_t *i;
 
@@ -324,9 +324,12 @@ void apoint_switch_notify(struct apoint *apt)
 	LLIST_TS_UNLOCK(&alist_p);
 }
 
-void apoint_paste_item(struct apoint *apt, long date)
+void apoint_paste_item(struct apoint *apt, time_t date)
 {
-	apt->start = date + get_item_time(apt->start);
+	struct tm t;
+
+	localtime_r((time_t *)&apt->start, &t);
+	apt->start = update_time_in_date(date, t.tm_hour, t.tm_min);
 
 	LLIST_TS_LOCK(&alist_p);
 	LLIST_TS_ADD_SORTED(&alist_p, apt, apoint_cmp);
