@@ -44,9 +44,18 @@
 
 #include "calcurse.h"
 
+/* Input types for parse_datetimearg() */
+enum {
+	ARG_DATE,
+	ARG_DATE_TIME,
+	ARG_TIME,
+	ARG_ERR
+};
+
 /* Long options */
 enum {
-	OPT_FILTER_TYPE = 1000,
+	OPT_FILTER_INVERT = 1000,
+	OPT_FILTER_TYPE,
 	OPT_FILTER_HASH,
 	OPT_FILTER_PATTERN,
 	OPT_FILTER_START_FROM,
@@ -74,7 +83,9 @@ enum {
 	OPT_EXPORT_UID,
 	OPT_READ_ONLY,
 	OPT_STATUS,
-	OPT_DAEMON
+	OPT_DAEMON,
+	OPT_INPUT_DATEFMT,
+	OPT_OUTPUT_DATEFMT
 };
 
 /*
@@ -82,9 +93,11 @@ enum {
  */
 static void usage(void)
 {
-	printf("%s\n", _("usage: calcurse [--daemon|-F|-G|-g|-i<file>|-Q|--status|-x[<format>]]\n"
-			 "                [-c<file>] [-D<path>] [-h] [-q] [--read-only] [-v]\n"
-			 "                [--filter-*] [--format-*]"));
+	printf("%s\n", _("Usage:\n"
+			 "calcurse [-D <directory>] [-C <directory>] [-c <calendar file>]\n"
+			 "calcurse -Q [--from <date>] [--to <date>] [--days <number>]\n"
+			 "calcurse -a | -d <date> | -d <number> | -n | -r[<number>] | -s[<date>] | -t[<number>]\n"
+			 "calcurse -h | -v | --status | -G | -P | -g | -i <file> | -x[<file>] | --daemon"));
 }
 
 static void usage_try(void)
@@ -110,26 +123,37 @@ static void help_arg(void)
 {
 	usage();
 	putchar('\n');
-	printf("%s\n", _("Operations in non-interactive mode:"));
-	printf("%s\n", _("  -F, --filter  Read items, filter them, and write them back"));
-	printf("%s\n", _("  -G, --grep    Grep items from the data files"));
+	printf("%s\n", _("Operations in command line mode:"));
 	printf("%s\n", _("  -Q, --query   Print items in a given query range"));
+	printf("%s\n", _("  -G, --grep    Grep items from the data files"));
+	printf("%s\n", _("  -P, --purge   Read items and write them back"));
+	printf("%s\n", _("Query short forms:\n"
+			 "-a, -d <date>|<number>, -n, -r[<number>], -s[<date>], -t<number>"));
 	putchar('\n');
-	printf("%s\n", _("Note that -F, -G and -Q can be combined with filter options --filter-*\n"
-			 "and formatting options --format-*. Consult the man page for details."));
+	printf("%s\n", _("Note that filter, format and day-range options affect input or output:"));
+	printf("%s\n", _("  --filter-*              Filter items loaded by -Q, -G, -P and -x"));
+	printf("%s\n", _("  --format-*              Rewrite output from -Q, -G and --dump-imported"));
+	printf("%s\n", _("  --from <date>           Limit day range of -Q."));
+	printf("%s\n", _("  --to <date>             Limit day range of -Q."));
+	printf("%s\n", _("  --days <number>         Limit day range of -Q."));
+	putchar('\n');
+	printf("%s\n", _("  --limit, -l <number>    Limit number of query results"));
+	printf("%s\n", _("  --search, -S <regexp>   Match regular expression in queries"));
+	printf("%s\n", _("Consult the man page for details."));
 	putchar('\n');
 	printf("%s\n", _("Miscellaneous:"));
-	printf("%s\n", _("  -c, --calendar <file>   Specify the calendar data file to use"));
+	printf("%s\n", _("  -c, --calendar <file>   The calendar data file to use"));
+	printf("%s\n", _("  -C, --confdir <dir>     The configuration directory to use"));
 	printf("%s\n", _("  --daemon                Run notification daemon in the background"));
-	printf("%s\n", _("  -D, --directory <path>  Specify the data directory to use"));
-	printf("%s\n", _("  -g, --gc                Run the garbage collector and exit"));
-	printf("%s\n", _("  -h, --help              Show this help text and exit"));
-	printf("%s\n", _("  -i, --import <file>     Import iCal data from a file"));
-	printf("%s\n", _("  -q, --quiet             Do not show system dialogs"));
+	printf("%s\n", _("  -D, --datadir <dir>     The data directory to use"));
+	printf("%s\n", _("  -g, --gc                Run the garbage collector"));
+	printf("%s\n", _("  -h, --help              Show this help text"));
+	printf("%s\n", _("  -i, --import <file>     Import iCal data from file"));
+	printf("%s\n", _("  -q, --quiet             Suppress system dialogs"));
 	printf("%s\n", _("  --read-only             Do not save configuration or data files"));
-	printf("%s\n", _("  --status                Display the status of running instances"));
-	printf("%s\n", _("  -v, --version           Show version information and exit"));
-	printf("%s\n", _("  -x, --export [<format>] Export items, valid formats: ical, pcal"));
+	printf("%s\n", _("  --status                Display status of running instances"));
+	printf("%s\n", _("  -v, --version           Show version information"));
+	printf("%s\n", _("  -x, --export[<format>]  Export to stdout in ical (default) or pcal format"));
 	putchar('\n');
 	printf("%s\n", _("For more information, type '?' from within calcurse, or read the manpage."));
 	printf("%s\n", _("Submit feature requests and suggestions to <misc@calcurse.org>."));
@@ -167,7 +191,7 @@ static void status_arg(void)
 			_("calcurse is running in background (pid %d)\n"),
 			dpid);
 	else
-		puts(_("calcurse is not running\n"));
+		puts(_("calcurse is not running"));
 }
 
 /* Print TODO list and return the number of printed items. */
@@ -207,14 +231,16 @@ static void next_arg(void)
 	next_app.got_app = 0;
 	next_app.txt = NULL;
 
-	next_app = *recur_apoint_check_next(&next_app, current_time,
-					    get_today());
+	recur_apoint_check_next(&next_app, current_time, get_today());
 	next_app = *apoint_check_next(&next_app, current_time);
 
 	if (next_app.got_app) {
 		time_left = next_app.time - current_time;
-		hours_left = (time_left / HOURINSEC);
-		min_left = (time_left - hours_left * HOURINSEC) / MININSEC;
+		/* In minutes rounded up. */
+		min_left = time_left / MININSEC +
+			   (time_left % MININSEC ? 1 : 0);
+		hours_left = min_left / HOURINMIN;
+		min_left = min_left % HOURINMIN;
 		fputs(_("next appointment:\n"), stdout);
 		fprintf(stdout, "   [%02d:%02d] %s\n", hours_left,
 			min_left, next_app.txt);
@@ -248,8 +274,8 @@ date_arg_from_to(long from, long to, int add_line, const char *fmt_apt,
 {
 	long date;
 
-	for (date = from; date < to; date = date_sec_change(date, 0, 1)) {
-		day_store_items(date, 0);
+	for (date = from; date <= to; date = date_sec_change(date, 0, 1)) {
+		day_store_items(date, 0, 1);
 		if (day_item_count(0) == 0)
 			continue;
 		if (add_line)
@@ -261,62 +287,54 @@ date_arg_from_to(long from, long to, int add_line, const char *fmt_apt,
 	}
 }
 
-static time_t parse_datearg(const char *str)
-{
-	struct date day;
+/*
+ * Convert a string with a (local time) date, date-time or time into
+ * the Unix time for that point in time as follows:
+ * - a date only is converted to midnight (beginning) of that day
+ * - a date-time is converted to that day and time
+ * - a time only is converted to that time of the current day
+ * The date format is taken from the user configuration.
+ * The type of the input string is returned in the type argument.
+ */
 
-	if (parse_date(str, DATEFMT_YYYYMMDD, (int *)&day.yyyy,
-			(int *)&day.mm, (int *)&day.dd, NULL))
-		return date2sec(day, 0, 0);
-
-	if (parse_date(str, DATEFMT_MMDDYYYY, (int *)&day.yyyy,
-			(int *)&day.mm, (int *)&day.dd, NULL))
-		return date2sec(day, 0, 0);
-
-	if (parse_date(str, DATEFMT_ISO, (int *)&day.yyyy,
-			(int *)&day.mm, (int *)&day.dd, NULL))
-		return date2sec(day, 0, 0);
-
-	return LONG_MAX;
-}
-
-static time_t parse_datetimearg(const char *str)
+static time_t parse_datetimearg(const char *str, int *type)
 {
 	char *date = mem_strdup(str);
+	*type = ARG_ERR;
 	char *time;
+	struct date day;
 	unsigned hour, min;
 	time_t ret;
 
 	time = strchr(date, ' ');
-	if (time) {
-		/* Date and time. */
-		*time = '\0';
-		time++;
-
+	if (time) { /* Date and time? */
+		*time++ = '\0';
 		if (!parse_time(time, &hour, &min))
 			return -1;
-		ret = parse_datearg(date);
-		if (ret == LONG_MAX)
+		if (!parse_date(date, conf.input_datefmt,
+		    (int *)&day.yyyy, (int *)&day.mm, (int *)&day.dd, NULL))
 			return -1;
-		ret += hour * HOURINSEC + min * MININSEC;
-
-		return ret;
-	}
-
-	ret = parse_datearg(date);
-	if (ret == LONG_MAX) {
-		/* No date specified, use time only. */
-		if (!parse_time(date, &hour, &min))
-			return -1;
-		return get_today() + hour * HOURINSEC + min * MININSEC;
-	}
-
+		ret = date2sec(day, hour, min);
+		*type = ARG_DATE_TIME;
+	} else /* Date?*/ if (parse_date(date, conf.input_datefmt,
+		    (int *)&day.yyyy, (int *)&day.mm, (int *)&day.dd, NULL)) {
+		ret = date2sec(day, 0, 0);
+		*type = ARG_DATE;
+	} else /* Time? */ if (parse_time(date, &hour, &min)) {
+		ret = date2sec(*ui_calendar_get_today(), hour, min);
+		*type = ARG_TIME;
+	} else
+		return -1;
 	return ret;
 }
 
+/*
+ * Parse a "from,to" date range.
+ * For an open-end range no change occurs at the open end.
+ */
 static int parse_daterange(const char *str, time_t *date_from, time_t *date_to)
 {
-	int ret = 0;
+	int type, ret = 0;
 	char *s = mem_strdup(str);
 	char *p = strchr(s, ',');
 
@@ -327,19 +345,17 @@ static int parse_daterange(const char *str, time_t *date_from, time_t *date_to)
 	p++;
 
 	if (*s != '\0') {
-		*date_from = parse_datetimearg(s);
+		*date_from = parse_datetimearg(s, &type);
 		if (*date_from == -1)
 			goto cleanup;
-	} else {
-		*date_from = -1;
 	}
 
 	if (*p != '\0') {
-		*date_to = parse_datetimearg(p);
+		*date_to = parse_datetimearg(p, &type);
 		if (*date_to == -1)
 			goto cleanup;
-	} else {
-		*date_to = -1;
+		if (type == ARG_DATE)
+			*date_to = ENDOFDAY(*date_to);
 	}
 
 	ret = 1;
@@ -382,18 +398,22 @@ cleanup:
 /*
  * Parse the command-line arguments and call the appropriate
  * routines to handle those arguments. Also initialize the data paths.
+ * Returns the non-interactive value.
  */
 int parse_args(int argc, char **argv)
 {
-	/* Command-line flags */
-	int grep = 0, grep_filter = 0, query = 0, next = 0;
+	/* Command-line flags - NOTE that read_only is global */
+	int grep = 0, grep_filter = 0, purge = 0, query = 0, next = 0;
 	int status = 0, gc = 0, import = 0, export = 0, daemon = 0;
+	/* Command line invocation */
+	int filter_opt = 0, format_opt = 0, query_range = 0, cmd_line = 0;
+	int start_from = 0, start_to = 0, end_from = 0, end_to = 0;
 	/* Query ranges */
 	time_t from = -1, to = -1;
-	int range = -1;
+	int range = 0;
 	int limit = INT_MAX;
 	/* Filters */
-	struct item_filter filter = { 0, NULL, NULL, -1, -1, -1, -1, 0, 0, 0 };
+	struct item_filter filter = { 0, 0, NULL, NULL, -1, -1, -1, -1, 0, 0, 0 };
 	/* Format strings */
 	const char *fmt_apt = NULL;
 	const char *fmt_rapt = NULL;
@@ -404,18 +424,27 @@ int parse_args(int argc, char **argv)
 	int xfmt = IO_EXPORT_ICAL;
 	int dump_imported = 0, export_uid = 0;
 	/* Data file locations */
-	const char *cfile = NULL, *datadir = NULL, *ifile = NULL;
+	const char *datadir = NULL;
+	const char *cfile = NULL, *ifile = NULL, *confdir = NULL;
 
 	int non_interactive = 1;
-	int ch;
+	int ch, cpid, type;
 	regex_t reg;
+	char buf[BUFSIZ];
+	struct tm tm;
+	time_t t;
 
-	static const char *optstr = "FgGhvnNax::t::d:c:r::s::S:D:i:l:qQ";
+	static const char *optstr = "PFgGhvnNax::t::C:d:c:r::s::S:D:i:l:qQ";
 
 	struct option longopts[] = {
 		{"appointment", no_argument, NULL, 'a'},
 		{"calendar", required_argument, NULL, 'c'},
+		/* Deprecated */
+		{"conf", required_argument, NULL, 'C'},
+		{"confdir", required_argument, NULL, 'C'},
 		{"day", required_argument, NULL, 'd'},
+		{"datadir", required_argument, NULL, 'D'},
+		/* Deprecated */
 		{"directory", required_argument, NULL, 'D'},
 		{"filter", no_argument, NULL, 'F'},
 		{"gc", no_argument, NULL, 'g'},
@@ -425,6 +454,7 @@ int parse_args(int argc, char **argv)
 		{"limit", required_argument, NULL, 'l'},
 		{"next", no_argument, NULL, 'n'},
 		{"note", no_argument, NULL, 'N'},
+		{"purge", no_argument, NULL, 'P'},
 		{"range", optional_argument, NULL, 'r'},
 		{"startday", optional_argument, NULL, 's'},
 		{"search", required_argument, NULL, 'S'},
@@ -434,6 +464,7 @@ int parse_args(int argc, char **argv)
 		{"quiet", no_argument, NULL, 'q'},
 		{"query", optional_argument, NULL, 'Q'},
 
+		{"filter-invert", no_argument, NULL, OPT_FILTER_INVERT},
 		{"filter-type", required_argument, NULL, OPT_FILTER_TYPE},
 		{"filter-hash", required_argument, NULL, OPT_FILTER_HASH},
 		{"filter-pattern", required_argument, NULL, OPT_FILTER_PATTERN},
@@ -463,9 +494,36 @@ int parse_args(int argc, char **argv)
 		{"read-only", no_argument, NULL, OPT_READ_ONLY},
 		{"status", no_argument, NULL, OPT_STATUS},
 		{"daemon", no_argument, NULL, OPT_DAEMON},
+		{"input-datefmt", required_argument, NULL, OPT_INPUT_DATEFMT},
+		{"output-datefmt", required_argument, NULL, OPT_OUTPUT_DATEFMT},
 		{NULL, no_argument, NULL, 0}
 	};
 
+	/*
+	 * Load the configuration file first to get the input date format for
+	 * parsing the remaining options.
+	 */
+	while ((ch = getopt_long(argc, argv, optstr, longopts, NULL)) != -1) {
+		switch (ch) {
+		case 'C':
+			confdir = optarg;
+			break;
+		case 'D':
+			datadir = optarg;
+			break;
+		case 'c':
+			cfile = optarg;
+			break;
+		}
+	}
+	io_init(cfile, datadir, confdir);
+	vars_init();
+	notify_init_vars();
+	if (io_file_exists(path_conf))
+		config_load();
+
+	/* Parse the remaining options. */
+	optind = 1;
 	while ((ch = getopt_long(argc, argv, optstr, longopts, NULL)) != -1) {
 		switch (ch) {
 		case 'a':
@@ -473,22 +531,25 @@ int parse_args(int argc, char **argv)
 			query = 1;
 			break;
 		case 'c':
-			cfile = optarg;
+			break;
+		case 'C':
 			break;
 		case 'd':
-			if (is_all_digit(optarg)) {
+			if (is_all_digit(optarg) ||
+			    (*optarg == '-' && is_all_digit(optarg + 1))) {
 				range = atoi(optarg);
-			} else {
-				from = parse_datetimearg(optarg);
-				EXIT_IF(from == -1, _("invalid date: %s"),
+				EXIT_IF(range == 0, _("invalid range: %s"),
 					optarg);
+			} else {
+				from = parse_datetimearg(optarg, &type);
+				EXIT_IF(from == -1 || type != ARG_DATE,
+					_("invalid date: %s"), optarg);
 			}
 
 			filter.type_mask |= TYPE_MASK_CAL;
 			query = 1;
 			break;
 		case 'D':
-			datadir = optarg;
 			break;
 		case 'F':
 			grep_filter = grep = 1;
@@ -512,6 +573,10 @@ int parse_args(int argc, char **argv)
 		case 'n':
 			next = 1;
 			break;
+		case 'P':
+			filter.invert = 1;
+			purge = grep = 1;
+			break;
 		case 'r':
 			if (optarg)
 				range = atoi(optarg);
@@ -524,8 +589,9 @@ int parse_args(int argc, char **argv)
 		case 's':
 			if (!optarg)
 				optarg = "today";
-			from = parse_datetimearg(optarg);
-			EXIT_IF(from == -1, _("invalid date: %s"), optarg);
+			from = parse_datetimearg(optarg, &type);
+			EXIT_IF(from == -1 || type != ARG_DATE,
+				_("invalid date: %s"), optarg);
 			filter.type_mask |= TYPE_MASK_CAL;
 			query = 1;
 			break;
@@ -567,13 +633,19 @@ int parse_args(int argc, char **argv)
 		case 'Q':
 			query = 1;
 			break;
+		case OPT_FILTER_INVERT:
+			filter.invert = !filter.invert;
+			filter_opt = 1;
+			break;
 		case OPT_FILTER_TYPE:
 			filter.type_mask = parse_type_mask(optarg);
 			EXIT_IF(filter.type_mask == 0,
 				_("invalid filter mask"));
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_HASH:
 			filter.hash = mem_strdup(optarg);
+			filter_opt = 1;
 			break;
 		case 'S':
 		case OPT_FILTER_PATTERN:
@@ -582,94 +654,175 @@ int parse_args(int argc, char **argv)
 			if (regcomp(&reg, optarg, REG_EXTENDED))
 				EXIT(_("could not compile regular expression: %s"), optarg);
 			filter.regex = &reg;
+			filter_opt = 1;
 			break;
+		/*
+		 * A date only is a time span (a day) and interpreted thus:
+		 * "to" means "to end of day"
+		 * "after" means "from start of next day"
+		 */
 		case OPT_FILTER_START_FROM:
-			filter.start_from = parse_datetimearg(optarg);
+			EXIT_IF(start_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.start_from = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.start_from == -1,
 				_("invalid date: %s"), optarg);
+			start_from = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_START_TO:
-			filter.start_to = parse_datetimearg(optarg);
+			EXIT_IF(start_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.start_to = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.start_to == -1,
 				_("invalid date: %s"), optarg);
+			if (type == ARG_DATE)
+				filter.start_to = ENDOFDAY(filter.start_to);
+			start_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_START_AFTER:
-			filter.start_from = parse_datetimearg(optarg) + 1;
+			EXIT_IF(start_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.start_from = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.start_from == -1,
 				_("invalid date: %s"), optarg);
+			if (type == ARG_DATE)
+				filter.start_from = NEXTDAY(filter.start_from);
+			else
+				filter.start_from++;
+			start_from = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_START_BEFORE:
-			filter.start_to = parse_datetimearg(optarg) - 1;
+			EXIT_IF(start_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.start_to = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.start_to == -1,
 				_("invalid date: %s"), optarg);
+			filter.start_to--;
+			start_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_START_RANGE:
-			EXIT_IF(!parse_daterange(optarg, &filter.start_from,
-						 &filter.start_to),
+			EXIT_IF(start_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			EXIT_IF(start_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			/* Set initialization values in case of open-end range. */
+			filter.start_from = filter.start_to = -1;
+			EXIT_IF(!parse_daterange(optarg, &filter.start_from, &filter.start_to),
 				_("invalid date range: %s"), optarg);
+			start_from = 1;
+			start_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_END_FROM:
-			filter.end_from = parse_datetimearg(optarg);
+			EXIT_IF(end_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.end_from = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.end_from == -1,
 				_("invalid date: %s"), optarg);
+			end_from = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_END_TO:
-			filter.end_to = parse_datetimearg(optarg);
+			EXIT_IF(end_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.end_to = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.end_to == -1,
 				_("invalid date: %s"), optarg);
+			if (type == ARG_DATE)
+				filter.end_to = ENDOFDAY(filter.end_to);
+			end_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_END_AFTER:
-			filter.end_from = parse_datetimearg(optarg) + 1;
+			EXIT_IF(end_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.end_from = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.end_from == -1,
 				_("invalid date: %s"), optarg);
+			if (type == ARG_DATE)
+				filter.end_from = NEXTDAY(filter.end_from);
+			else
+				filter.end_from++;
+			end_from = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_END_BEFORE:
-			filter.end_to = parse_datetimearg(optarg) - 1;
+			EXIT_IF(end_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			filter.end_to = parse_datetimearg(optarg, &type);
 			EXIT_IF(filter.end_to == -1,
 				_("invalid date: %s"), optarg);
+			filter.end_to--;
+			end_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_END_RANGE:
-			EXIT_IF(!parse_daterange(optarg, &filter.end_from,
-						 &filter.end_to),
+			EXIT_IF(end_from,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			EXIT_IF(end_to,
+			    _("filter criterion already in use: %s"), argv[optind - 2]);
+			/* Set default values in case of open-ended range. */
+			filter.start_from = filter.start_to = -1;
+			EXIT_IF(!parse_daterange(optarg, &filter.end_from, &filter.end_to),
 				_("invalid date range: %s"), optarg);
+			end_from = 1;
+			end_to = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_PRIORITY:
 			filter.priority = atoi(optarg);
 			EXIT_IF(filter.priority < 1 || filter.priority > 9,
 				_("invalid priority: %s"), optarg);
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_COMPLETED:
 			filter.completed = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FILTER_UNCOMPLETED:
 			filter.uncompleted = 1;
+			filter_opt = 1;
 			break;
 		case OPT_FROM:
-			from = parse_datetimearg(optarg);
-			EXIT_IF(from == -1, _("invalid date: %s"), optarg);
+			from = parse_datetimearg(optarg, &type);
+			EXIT_IF(from == -1 || type != ARG_DATE,
+				_("invalid date: %s"), optarg);
+			query_range = 1;
 			break;
 		case OPT_TO:
-			to = parse_datetimearg(optarg);
-			EXIT_IF(to == -1, _("invalid date: %s"), optarg);
+			to = parse_datetimearg(optarg, &type);
+			EXIT_IF(to == -1 || type != ARG_DATE,
+				_("invalid date: %s"), optarg);
+			query_range = 1;
 			break;
 		case OPT_DAYS:
 			range = atoi(optarg);
 			EXIT_IF(range == 0, _("invalid range: %s"), optarg);
+			query_range = 1;
 			break;
 		case OPT_FMT_APT:
 			fmt_apt = optarg;
+			format_opt = 1;
 			break;
 		case OPT_FMT_RAPT:
 			fmt_rapt = optarg;
+			format_opt = 1;
 			break;
 		case OPT_FMT_EV:
 			fmt_ev = optarg;
+			format_opt = 1;
 			break;
 		case OPT_FMT_REV:
 			fmt_rev = optarg;
+			format_opt = 1;
 			break;
 		case OPT_FMT_TODO:
 			fmt_todo = optarg;
+			format_opt = 1;
 			break;
 		case OPT_DUMP_IMPORTED:
 			dump_imported = 1;
@@ -684,7 +837,27 @@ int parse_args(int argc, char **argv)
 			status = 1;
 			break;
 		case OPT_DAEMON:
+			EXIT_IF(cpid = io_get_pid(path_cpid),
+				_("calcurse is running (pid = %d)"), cpid);
 			daemon = 1;
+			filter.type_mask = TYPE_MASK_APPT | TYPE_MASK_RECUR_APPT;
+			break;
+		case OPT_INPUT_DATEFMT:
+			conf.input_datefmt = atoi(optarg);
+			EXIT_IF(conf.input_datefmt < 1 || conf.input_datefmt > 4,
+				_("invalid input date format: %s"), optarg);
+			cmd_line = 1;
+			break;
+		case OPT_OUTPUT_DATEFMT:
+			time(&t);
+			localtime_r(&t, &tm);
+			EXIT_IF(!strftime(buf, sizeof(buf), optarg, &tm),
+			       _("invalid output date format: %s"), optarg);
+			strncpy(conf.output_datefmt, optarg,
+				sizeof(conf.output_datefmt) - 1);
+			conf.output_datefmt[sizeof(conf.output_datefmt) - 1] =
+				'\0';
+			cmd_line = 1;
 			break;
 		default:
 			usage();
@@ -692,33 +865,38 @@ int parse_args(int argc, char **argv)
 			goto cleanup;
 		}
 	}
-	argc -= optind;
 
 	if (filter.type_mask == 0)
 		filter.type_mask = TYPE_MASK_ALL;
 
-	if (status + grep + query + next + gc + import + export + daemon > 1) {
+	if (status + grep + query + next + gc + import + export + daemon > 1 ||
+	    optind < argc ||
+	    (filter_opt && !(grep + query + export)) ||
+	    (format_opt && !(grep + query + dump_imported)) ||
+	    (query_range && !query) ||
+	    (purge && !filter.invert)
+	   ) {
 		ERROR_MSG(_("invalid argument combination"));
 		usage();
 		usage_try();
 		goto cleanup;
 	}
 
-	if (from == -1) {
-		struct date day = { 0, 0, 0 };
-		from = get_sec_date(day);
-	}
+	EXIT_IF(to >= 0 && range, _("cannot specify a range and an end date"));
+	if (from == -1)
+		from = get_today();
+	if (to == -1)
+		to = ENDOFDAY(from);
+	EXIT_IF(to < from, _("end date cannot come before start date"));
+	if (range > 0)
+		to = date_sec_change(from, 0, range - 1);
+	else if (range < 0)
+		from = date_sec_change(to, 0, range + 1);
 
-	if (to == -1 && range == -1)
-		to = date_sec_change(from, 0, 1);
-	else if (to == -1 && range >= 0)
-		to = date_sec_change(from, 0, range);
-	else if (to >= 0 && range >= 0)
-		EXIT_IF(to >= 0, _("cannot specify a range and an end date"));
-
-	io_init(cfile, datadir);
-	io_check_dir(path_dir);
+	io_check_dir(path_ddir);
 	io_check_dir(path_notes);
+	io_check_dir(path_cdir);
+	io_check_dir(path_hooks);
 
 	if (status) {
 		status_arg();
@@ -726,10 +904,8 @@ int parse_args(int argc, char **argv)
 		io_check_file(path_apts);
 		io_check_file(path_todo);
 		io_check_file(path_conf);
-		vars_init();
-		config_load();	/* To get output date format. */
-		io_load_data(&filter);
-		if (grep_filter) {
+		io_load_data(&filter, FORCE);
+		if (purge || grep_filter) {
 			io_save_todo(path_todo);
 			io_save_apts(path_apts);
 		} else {
@@ -749,9 +925,7 @@ int parse_args(int argc, char **argv)
 		io_check_file(path_apts);
 		io_check_file(path_todo);
 		io_check_file(path_conf);
-		vars_init();
-		config_load();	/* To get output date format. */
-		io_load_data(&filter);
+		io_load_data(&filter, FORCE);
 
 		/* Use default values for non-specified format strings. */
 		fmt_apt = fmt_apt ? fmt_apt : " - %S -> %E\n\t%m\n";
@@ -770,14 +944,12 @@ int parse_args(int argc, char **argv)
 	} else if (gc) {
 		io_check_file(path_apts);
 		io_check_file(path_todo);
-		io_load_data(NULL);
+		io_load_data(NULL, FORCE);
 		note_gc();
 	} else if (import) {
 		io_check_file(path_apts);
 		io_check_file(path_todo);
-		/* Get default pager in case we need to show a log file. */
-		vars_init();
-		io_load_data(NULL);
+		io_load_data(NULL, FORCE);
 		if (dump_imported) {
 			/*
 			 * Use default values for non-specified format strings.
@@ -801,13 +973,12 @@ int parse_args(int argc, char **argv)
 	} else if (export) {
 		io_check_file(path_apts);
 		io_check_file(path_todo);
-		io_load_data(&filter);
+		io_load_data(&filter, FORCE);
 		io_export_data(xfmt, export_uid);
 	} else if (daemon) {
-		notify_init_vars();
 		dmon_stop();
 		dmon_start(0);
-	} else {
+	} else if (!cmd_line) {
 		/* interactive mode */
 		non_interactive = 0;
 	}
