@@ -95,16 +95,6 @@
 #define NOTES_DIR_NAME   "notes/"
 #define HOOKS_DIR_NAME   "hooks/"
 
-#define TODO_PATH        DIR_NAME TODO_PATH_NAME
-#define APTS_PATH        DIR_NAME APTS_PATH_NAME
-#define CONF_PATH        DIR_NAME CONF_PATH_NAME
-#define KEYS_PATH        DIR_NAME KEYS_PATH_NAME
-#define CPID_PATH        DIR_NAME CPID_PATH_NAME
-#define DLOG_PATH        DIR_NAME DLOG_PATH_NAME
-#define DPID_PATH        DIR_NAME DPID_PATH_NAME
-#define NOTES_DIR        DIR_NAME NOTES_DIR_NAME
-#define HOOKS_DIR        DIR_NAME HOOKS_DIR_NAME
-
 #define DEFAULT_EDITOR     "vi"
 #define DEFAULT_PAGER      "less"
 #define DEFAULT_MERGETOOL  "vimdiff"
@@ -127,7 +117,7 @@
 #define NBUSERCOLORS  6
 
 /* Side bar width acceptable boundaries. */
-#define SBARMINWIDTH     32
+#define SBARMINWIDTH     34
 #define SBARMAXWIDTHPERC 50
 
 /* Related to date manipulation. */
@@ -144,11 +134,17 @@
 #define WEEKINMIN       (WEEKINHOURS * HOURINMIN)
 #define WEEKINSEC       (WEEKINMIN * MININSEC)
 #define DAYINMIN        (DAYINHOURS * HOURINMIN)
+/*
+ * Note the difference between the number of seconds in a day and daylength
+ * in seconds. The two may differ when DST is in effect (daylength is either
+ * 23, 24 or 25 hours. The argument "date" is assumed to be of type time_t.
+ */
 #define DAYINSEC        (DAYINMIN * MININSEC)
+#define NEXTDAY(date)	date_sec_change((date), 0, 1)
+#define DAYLEN(date)	(NEXTDAY(date) - (date))
+#define ENDOFDAY(date)	(NEXTDAY(date) - 1)
 #define HOURINSEC       (HOURINMIN * MININSEC)
 
-#define MAXDAYSPERMONTH 31
-
 /* Calendar window. */
 #define CALHEIGHT       8
 
@@ -170,6 +166,12 @@
 /* Size of the hash table the note garbage collector uses. */
 #define NOTE_GC_HSIZE 1024
 
+/* Mnemonics */
+#define NOHILT		0 	/* 'No highlight' argument */
+#define NOFORCE		0
+#define FORCE		1
+#define DUMMY		-1	/* The dummy event */
+
 #define ERROR_MSG(...) do {                                                   \
   char msg[BUFSIZ];                                                           \
   int len;                                                                    \
@@ -225,13 +227,10 @@
 #define TOSTRING(x)   STRINGIFY(x)
 #define __FILE_POS__   __FILE__ ":" TOSTRING(__LINE__)
 
-#define UTF8_MAXLEN 6
-#define UTF8_LENGTH(ch) ((unsigned char)ch >= 0xFC ? 6 : \
-    ((unsigned char)ch >= 0xF8 ? 5 : \
-    ((unsigned char)ch >= 0xF0 ? 4 : \
+#define UTF8_MAXLEN 4
+#define UTF8_LENGTH(ch) ((unsigned char)ch >= 0xF0 ? 4 : \
     ((unsigned char)ch >= 0xE0 ? 3 : \
-    ((unsigned char)ch >= 0xC0 ? 2 : 1)))))
-#define UTF8_ISMULTI(ch) ((unsigned char)ch >= 0x80)
+    ((unsigned char)ch >= 0xC0 ? 2 : 1)))
 #define UTF8_ISCONT(ch) ((unsigned char)ch >= 0x80 && \
     (unsigned char)ch <= 0xBF)
 
@@ -256,17 +255,38 @@ enum pos {
 	RIGHT
 };
 
+/* Available views for the calendar panel. */
+enum cal_view {
+	CAL_MONTH_VIEW,
+	CAL_WEEK_VIEW,
+	CAL_VIEWS
+};
+
+/* Available views for the todo panel. */
+enum todo_view {
+	TODO_SHOW_COMPLETED_VIEW,
+	TODO_HIDE_COMPLETED_VIEW,
+	TODO_VIEWS
+};
+
 /* General configuration variables. */
 struct conf {
+	enum cal_view cal_view;
+	enum todo_view todo_view;
 	unsigned auto_save;
 	unsigned auto_gc;
 	unsigned periodic_save;
+	unsigned systemevents;
 	unsigned confirm_quit;
 	unsigned confirm_delete;
 	enum win default_panel;
 	unsigned compact_panels;
 	unsigned system_dialogs;
-	unsigned progress_bar;
+	unsigned multiple_days;
+	unsigned header_line;
+	unsigned event_separator;
+	unsigned day_separator;
+	unsigned empty_appt_line;
 	const char *editor;
 	const char *pager;
 	const char *mergetool;
@@ -302,19 +322,24 @@ enum datefmt {
     (datefmt == DATEFMT_YYYYMMDD ? _("yyyy/mm/dd") : _("yyyy-mm-dd"))))
 
 /* Day heading default format. */
-#define DAY_HEADING_DEFAULT "%B %-d, %Y"
+#define DAY_HEADING_DEFAULT "%B %e, %Y"
 
+/*
+ * Calcurse representation of the date of a day in the calendar.
+ * When time_t is a 32-bit signed integer, the year range is 1902 - 2037.
+ */
+#define YEAR1902_2037 (sizeof(time_t) == 4)
 struct date {
-	unsigned dd;
-	unsigned mm;
-	unsigned yyyy;
+	unsigned dd;		/* day: 1 - 31 */
+	unsigned mm;		/* month: 1 - 12 */
+	unsigned yyyy;		/* year AD */
 };
 
 #define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
 
 /* Appointment definition. */
 struct apoint {
-	long start;		/* seconds since 1 jan 1970 */
+	time_t start;		/* seconds since 1 jan 1970 */
 	long dur;		/* duration of the appointment in seconds */
 
 #define APOINT_NULL      0x0
@@ -329,7 +354,7 @@ struct apoint {
 /* Event definition. */
 struct event {
 	int id;			/* event identifier */
-	long day;		/* seconds since 1 jan 1970 */
+	time_t day;		/* seconds since 1 jan 1970 */
 	char *mesg;
 	char *note;
 };
@@ -343,7 +368,7 @@ struct todo {
 };
 
 struct excp {
-	long st;		/* beggining of the considered day, in seconds */
+	time_t st;		/* beggining of the considered day, in seconds */
 };
 
 enum recur_type {
@@ -359,14 +384,14 @@ enum recur_type {
 struct rpt {
 	enum recur_type type;	/* repetition type */
 	int freq;		/* repetition frequency */
-	long until;		/* ending date for repeated event */
+	time_t until;		/* ending date for repeated event */
 };
 
 /* Recurrent appointment definition. */
 struct recur_apoint {
 	struct rpt *rpt;	/* information about repetition */
 	llist_t exc;		/* days when the item should not be repeated */
-	long start;		/* beggining of the appointment */
+	time_t start;		/* beggining of the appointment */
 	long dur;		/* duration of the appointment */
 	char state;		/* 8 bits to store item state */
 	char *mesg;		/* appointment description */
@@ -378,7 +403,7 @@ struct recur_event {
 	struct rpt *rpt;	/* information about repetition */
 	llist_t exc;		/* days when the item should not be repeated */
 	int id;			/* event type */
-	long day;		/* day at which event occurs */
+	time_t day;		/* day at which event occurs */
 	char *mesg;		/* event description */
 	char *note;		/* note attached to event */
 };
@@ -396,9 +421,11 @@ enum day_item_type {
 	DAY_HEADING = 1,
 	RECUR_EVNT,
 	EVNT,
-	DAY_SEPARATOR,
+	EVNT_SEPARATOR,
 	RECUR_APPT,
-	APPT
+	APPT,
+	EMPTY_SEPARATOR,
+	END_SEPARATOR
 };
 
 /* Available item types. */
@@ -422,6 +449,7 @@ enum item_type {
 
 /* Filter settings. */
 struct item_filter {
+	int invert;
 	int type_mask;
 	char *hash;
 	regex_t *regex;
@@ -437,26 +465,14 @@ struct item_filter {
 /* Generic item description (to hold appointments, events...). */
 struct day_item {
 	enum day_item_type type;
-	long start;
+	time_t start;
+	time_t order;
 	union aptev_ptr item;
 };
 
-/* Available views for the calendar panel. */
-enum {
-	CAL_MONTH_VIEW,
-	CAL_WEEK_VIEW,
-	CAL_VIEWS
-};
-
-/* Available views for the todo panel. */
-enum {
-	TODO_SHOW_COMPLETED_VIEW,
-	TODO_HIDE_COMPLETED_VIEW,
-	TODO_VIEWS
-};
-
+/* Shared variables for the notification threads. */
 struct notify_app {
-	long time;
+	time_t time;
 	int got_app;
 	char *txt;
 	char state;
@@ -571,8 +587,8 @@ struct window {
 
 /* Generic scrolling window structure. */
 struct scrollwin {
-	WINDOW *win;
-	WINDOW *inner;
+	WINDOW *win;		/* viewport */
+	WINDOW *inner;		/* pad */
 	int y;
 	int x;
 	int h;
@@ -647,6 +663,29 @@ enum getstr {
 #define PARSE_DATETIME_HAS_DATE (1 << 0)
 #define PARSE_DATETIME_HAS_TIME (1 << 1)
 
+/* Save types and return codes for the io_save_cal() function. */
+enum save_type {
+	interactive,
+	periodic
+};
+enum {
+	IO_SAVE_CTINUE,
+	IO_SAVE_RELOAD,
+	IO_SAVE_CANCEL,
+	IO_SAVE_NOOP,
+	IO_SAVE_ERROR
+};
+
+/* Return codes for the io_reload_data() function. */
+enum {
+	IO_RELOAD_LOAD,
+	IO_RELOAD_CTINUE,
+	IO_RELOAD_MERGE,
+	IO_RELOAD_CANCEL,
+	IO_RELOAD_NOOP,
+	IO_RELOAD_ERROR
+};
+
 /* Week days. */
 enum wday {
 	SUNDAY,
@@ -673,7 +712,11 @@ enum move {
 	YEAR_NEXT
 };
 
-/* Available color pairs. */
+/*
+ * Available color pairs.
+ * The numbering must agree with
+ * the colour numbers 1-6, see custom.c.
+ */
 enum {
 	COLR_RED = 1,
 	COLR_GREEN,
@@ -699,12 +742,6 @@ enum export_type {
 	IO_EXPORT_NBTYPES
 };
 
-/* To customize the display when saving data. */
-enum save_display {
-	IO_SAVE_DISPLAY_BAR,
-	IO_SAVE_DISPLAY_NONE
-};
-
 /* apoint.c */
 extern llist_ts_t alist_p;
 void apoint_free_bkp(void);
@@ -712,23 +749,26 @@ struct apoint *apoint_dup(struct apoint *);
 void apoint_free(struct apoint *);
 void apoint_llist_init(void);
 void apoint_llist_free(void);
-struct apoint *apoint_new(char *, char *, long, long, char);
-unsigned apoint_inday(struct apoint *, long *);
-void apoint_sec2str(struct apoint *, long, char *, char *);
+struct apoint *apoint_new(char *, char *, time_t, long, char);
+unsigned apoint_inday(struct apoint *, time_t *);
+void apoint_sec2str(struct apoint *, time_t, char *, char *);
 char *apoint_tostr(struct apoint *);
 char *apoint_hash(struct apoint *);
 void apoint_write(struct apoint *, FILE *);
 struct apoint *apoint_scan(FILE *, struct tm, struct tm, char, char *,
 			   struct item_filter *);
 void apoint_delete(struct apoint *);
-struct notify_app *apoint_check_next(struct notify_app *, long);
+struct notify_app *apoint_check_next(struct notify_app *, time_t);
 void apoint_switch_notify(struct apoint *);
-void apoint_paste_item(struct apoint *, long);
+void apoint_paste_item(struct apoint *, time_t);
 
 /* args.c */
 int parse_args(int, char **);
 
 /* calendar.c */
+extern struct day_item empty_day;
+
+/* ui-calendar.c */
 void ui_calendar_view_next(void);
 void ui_calendar_view_prev(void);
 void ui_calendar_set_view(int);
@@ -736,20 +776,21 @@ int ui_calendar_get_view(void);
 void ui_calendar_start_date_thread(void);
 void ui_calendar_stop_date_thread(void);
 void ui_calendar_set_current_date(void);
+struct date *ui_calendar_get_today(void);
 void ui_calendar_set_first_day_of_week(enum wday);
 void ui_calendar_change_first_day_of_week(void);
 unsigned ui_calendar_week_begins_on_monday(void);
 void ui_calendar_store_current_date(struct date *);
 void ui_calendar_init_slctd_day(void);
 struct date *ui_calendar_get_slctd_day(void);
-time_t ui_calendar_get_slctd_day_sec(void);
+void ui_calendar_set_slctd_day(struct date);
 void ui_calendar_monthly_view_cache_set_invalid(void);
 void ui_calendar_update_panel(void);
 void ui_calendar_goto_today(void);
 void ui_calendar_change_day(int);
 void ui_calendar_move(enum move, int);
-long ui_calendar_start_of_year(void);
-long ui_calendar_end_of_year(void);
+time_t ui_calendar_start_of_year(void);
+time_t ui_calendar_end_of_year(void);
 
 /* config.c */
 void config_load(void);
@@ -762,35 +803,33 @@ void custom_remove_attr(WINDOW *, int);
 void custom_layout_config(void);
 void custom_sidebar_config(void);
 void custom_color_config(void);
-void custom_color_theme_name(char *);
-void custom_confwin_init(struct window *, const char *);
-void custom_set_swsiz(struct scrollwin *);
 void custom_general_config(void);
 void custom_keys_config(void);
 void custom_config_main(void);
 
 /* day.c */
+int day_set_sel_data(struct day_item *);
+int day_check_sel_data(void);
+int day_sel_index(void);
+int day_get_days(void);
 void day_free_vector(void);
 char *day_item_get_mesg(struct day_item *);
 char *day_item_get_note(struct day_item *);
 void day_item_erase_note(struct day_item *);
 long day_item_get_duration(struct day_item *);
 int day_item_get_state(struct day_item *);
-void day_item_add_exc(struct day_item *, long);
+void day_item_add_exc(struct day_item *, time_t);
 void day_item_fork(struct day_item *, struct day_item *);
-void day_store_items(long, int);
-void day_process_storage(struct date *, unsigned);
-void day_display_item_date(struct day_item *, WINDOW *, int, long, int, int);
+void day_store_items(time_t, int, int);
+void day_display_item_date(struct day_item *, WINDOW *, int, time_t, int, int);
 void day_display_item(struct day_item *, WINDOW *, int, int, int, int);
-void day_write_stdout(long, const char *, const char *, const char *,
+void day_write_stdout(time_t, const char *, const char *, const char *,
 		      const char *, int *);
 void day_popup_item(struct day_item *);
 int day_check_if_item(struct date);
 unsigned day_chk_busy_slices(struct date, int, int *);
-struct day_item *day_cut_item(long, int);
-int day_paste_item(struct day_item *, long);
-int day_get_position_by_aptev_ptr(union aptev_ptr);
-int day_get_position(struct day_item *);
+struct day_item *day_cut_item(int);
+int day_paste_item(struct day_item *, time_t);
 struct day_item *day_get_item(int);
 unsigned day_item_count(int);
 void day_edit_note(struct day_item *, const char *);
@@ -803,19 +842,21 @@ void dmon_stop(void);
 
 /* event.c */
 extern llist_t eventlist;
+extern struct event dummy;
 void event_free_bkp(void);
 struct event *event_dup(struct event *);
 void event_free(struct event *);
 void event_llist_init(void);
 void event_llist_free(void);
-struct event *event_new(char *, char *, long, int);
-unsigned event_inday(struct event *, long *);
+struct event *event_new(char *, char *, time_t, int);
+unsigned event_inday(struct event *, time_t *);
 char *event_tostr(struct event *);
 char *event_hash(struct event *);
 void event_write(struct event *, FILE *);
 struct event *event_scan(FILE *, struct tm, int, char *, struct item_filter *);
 void event_delete(struct event *);
-void event_paste_item(struct event *, long);
+void event_paste_item(struct event *, time_t);
+int event_dummy(struct day_item *);
 
 /* getstring.c */
 enum getstr getstring(WINDOW *, char *, int, int, int);
@@ -835,17 +876,17 @@ void ical_export_data(FILE *, int);
 
 /* io.c */
 unsigned io_fprintln(const char *, const char *, ...);
-void io_init(const char *, const char *);
+void io_init(const char *, const char *, const char *);
 void io_extract_data(char *, const char *, int);
 void io_dump_apts(const char *, const char *, const char *, const char *);
 unsigned io_save_apts(const char *);
 void io_dump_todo(const char *);
 unsigned io_save_todo(const char *);
 unsigned io_save_keys(void);
-void io_save_cal(enum save_display);
+int io_save_cal(enum save_type);
 void io_load_app(struct item_filter *);
 void io_load_todo(struct item_filter *);
-void io_load_data(struct item_filter *);
+int io_load_data(struct item_filter *, int);
 int io_reload_data(void);
 void io_load_keys(const char *);
 int io_check_dir(const char *);
@@ -905,10 +946,11 @@ void listbox_resize(struct listbox *, int, int, int, int);
 void listbox_set_cb_data(struct listbox *, void *);
 void listbox_load_items(struct listbox *, int);
 void listbox_draw_deco(struct listbox *, int);
-void listbox_display(struct listbox *);
+void listbox_display(struct listbox *, int);
 int listbox_get_sel(struct listbox *);
 void listbox_set_sel(struct listbox *, unsigned);
-void listbox_sel_move(struct listbox *, int);
+void listbox_item_in_view(struct listbox *, int);
+int listbox_sel_move(struct listbox *, int);
 
 /* mem.c */
 void *xmalloc(size_t);
@@ -954,7 +996,7 @@ void note_gc(void);
 /* notify.c */
 int notify_time_left(void);
 unsigned notify_needs_reminder(void);
-void notify_update_app(long, char, char *);
+void notify_update_app(time_t, char, char *);
 int notify_bar(void);
 void notify_init_vars(void);
 void notify_init_bar(void);
@@ -968,9 +1010,9 @@ unsigned notify_get_next(struct notify_app *);
 unsigned notify_get_next_bkgd(void);
 char *notify_app_txt(void);
 void notify_check_next_app(int);
-void notify_check_added(char *, long, char);
+void notify_check_added(char *, time_t, char);
 void notify_check_repeated(struct recur_apoint *);
-int notify_same_item(long);
+int notify_same_item(time_t);
 int notify_same_recur_item(struct recur_apoint *);
 void notify_config_bar(void);
 
@@ -980,6 +1022,8 @@ void pcal_export_data(FILE *);
 /* recur.c */
 extern llist_ts_t recur_alist_p;
 extern llist_t recur_elist;
+int recur_update_exc(llist_t *, char *);
+char *recur_exc2str(llist_t *);
 struct recur_event *recur_event_dup(struct recur_event *);
 struct recur_apoint *recur_apoint_dup(struct recur_apoint *);
 void recur_event_free_bkp(void);
@@ -990,10 +1034,10 @@ void recur_apoint_llist_init(void);
 void recur_event_llist_init(void);
 void recur_apoint_llist_free(void);
 void recur_event_llist_free(void);
-struct recur_apoint *recur_apoint_new(char *, char *, long, long, char,
-				      int, int, long, llist_t *);
-struct recur_event *recur_event_new(char *, char *, long, int, int, int,
-				    long, llist_t *);
+struct recur_apoint *recur_apoint_new(char *, char *, time_t, long, char,
+				      int, int, time_t, llist_t *);
+struct recur_event *recur_event_new(char *, char *, time_t, int, int, int,
+				    time_t, llist_t *);
 char recur_def2char(enum recur_type);
 int recur_char2def(char);
 struct recur_apoint *recur_apoint_scan(FILE *, struct tm, struct tm,
@@ -1009,23 +1053,22 @@ char *recur_event_tostr(struct recur_event *);
 char *recur_event_hash(struct recur_event *);
 void recur_event_write(struct recur_event *, FILE *);
 void recur_save_data(FILE *);
-unsigned recur_item_find_occurrence(long, long, llist_t *, int,
-				    int, long, long, time_t *);
-unsigned recur_apoint_find_occurrence(struct recur_apoint *, long, time_t *);
-unsigned recur_event_find_occurrence(struct recur_event *, long, time_t *);
-unsigned recur_item_inday(long, long, llist_t *, int, int, long, long);
-unsigned recur_apoint_inday(struct recur_apoint *, long *);
-unsigned recur_event_inday(struct recur_event *, long *);
-void recur_event_add_exc(struct recur_event *, long);
-void recur_apoint_add_exc(struct recur_apoint *, long);
+unsigned recur_item_find_occurrence(time_t, long, llist_t *, int,
+				    int, time_t, time_t, time_t *);
+unsigned recur_apoint_find_occurrence(struct recur_apoint *, time_t, time_t *);
+unsigned recur_event_find_occurrence(struct recur_event *, time_t, time_t *);
+unsigned recur_item_inday(time_t, long, llist_t *, int, int, time_t, time_t);
+unsigned recur_apoint_inday(struct recur_apoint *, time_t *);
+unsigned recur_event_inday(struct recur_event *, time_t *);
+void recur_event_add_exc(struct recur_event *, time_t);
+void recur_apoint_add_exc(struct recur_apoint *, time_t);
 void recur_event_erase(struct recur_event *);
 void recur_apoint_erase(struct recur_apoint *);
 void recur_exc_scan(llist_t *, FILE *);
-struct notify_app *recur_apoint_check_next(struct notify_app *, long,
-					   long);
+void recur_apoint_check_next(struct notify_app *, time_t, time_t);
 void recur_apoint_switch_notify(struct recur_apoint *);
-void recur_event_paste_item(struct recur_event *, long);
-void recur_apoint_paste_item(struct recur_apoint *, long);
+void recur_event_paste_item(struct recur_event *, time_t);
+void recur_apoint_paste_item(struct recur_apoint *, time_t);
 
 /* sigs.c */
 void sigs_init(void);
@@ -1063,9 +1106,6 @@ void todo_init_list(void);
 void todo_free_list(void);
 
 /* ui-day.c */
-struct day_item *ui_day_selitem(void);
-void ui_day_set_selitem_by_aptev_ptr(union aptev_ptr);
-void ui_day_set_selitem(struct day_item *);
 void ui_day_item_add(void);
 void ui_day_item_delete(unsigned);
 void ui_day_item_edit(void);
@@ -1075,8 +1115,14 @@ void ui_day_item_cut_free(unsigned);
 void ui_day_item_copy(unsigned);
 void ui_day_item_paste(unsigned);
 void ui_day_load_items(void);
+void ui_day_find_sel(void);
+struct day_item *ui_day_get_sel(void);
+time_t ui_day_sel_date(void);
 void ui_day_sel_reset(void);
-void ui_day_sel_move(int);
+void ui_day_set_sel(struct day_item *);
+int ui_day_sel_move(int);
+void ui_day_sel_daybegin(int);
+void ui_day_sel_dayend(void);
 void ui_day_draw(int, WINDOW *, int, int, void *);
 enum listbox_row_type ui_day_row_type(int, void *);
 int ui_day_height(int, void *);
@@ -1109,10 +1155,11 @@ int ui_todo_get_view(void);
 void ui_todo_set_view(int);
 
 /* utf8.c */
-int utf8_ord(const char *);
+int utf8_decode(const char *);
 int utf8_width(char *);
 int utf8_strwidth(char *);
 int utf8_chop(char *, int);
+char *utf8_encode(int);
 
 /* utils.c */
 void exit_calcurse(int) __attribute__ ((__noreturn__));
@@ -1127,24 +1174,27 @@ void erase_window_part(WINDOW *, int, int, int, int);
 WINDOW *popup(int, int, int, int, const char *, const char *, int);
 void print_in_middle(WINDOW *, int, int, int, const char *);
 int is_all_digit(const char *);
-long get_item_time(long);
-int get_item_hour(long);
-int get_item_min(long);
+long get_item_time(time_t);
+int get_item_hour(time_t);
+int get_item_min(time_t);
 struct tm date2tm(struct date, unsigned, unsigned);
 time_t date2sec(struct date, unsigned, unsigned);
+struct date sec2date(time_t);
 time_t utcdate2sec(struct date, unsigned, unsigned);
+int date_cmp(struct date *, struct date *);
 int date_cmp_day(time_t, time_t);
-char *date_sec2date_str(long, const char *);
-void date_sec2date_fmt(long, const char *, char *);
+char *date_sec2date_str(time_t, const char *);
+void date_sec2date_fmt(time_t, const char *, char *);
 int date_change(struct tm *, int, int);
-long date_sec_change(long, int, int);
-long update_time_in_date(long, unsigned, unsigned);
+time_t date_sec_change(time_t, int, int);
+time_t update_time_in_date(time_t, unsigned, unsigned);
 time_t get_sec_date(struct date);
 long min2sec(unsigned);
-void draw_scrollbar(WINDOW *, int, int, int, int, int, unsigned);
+void draw_scrollbar(struct scrollwin *, int);
 void item_in_popup(const char *, const char *, const char *, const char *);
 time_t get_today(void);
-long now(void);
+time_t get_slctd_day(void);
+time_t now(void);
 char *nowstr(void);
 void print_bool_option_incolor(WINDOW *, unsigned, int, int);
 const char *get_tempdir(void);
@@ -1152,21 +1202,22 @@ char *new_tempfile(const char *);
 int check_date(unsigned, unsigned, unsigned);
 int parse_date(const char *, enum datefmt, int *, int *, int *, struct date *);
 int parse_date_interactive(const char *, int *, int *, int *);
+int check_sec(time_t *);
 int check_time(unsigned, unsigned);
 int parse_time(const char *, unsigned *, unsigned *);
-int parse_duration(const char *, unsigned *);
-int parse_date_duration(const char *, unsigned *);
-int parse_datetime(const char *, long *);
+int parse_duration(const char *, unsigned *, time_t);
+int parse_date_duration(const char *, unsigned *, time_t);
+int parse_datetime(const char *, time_t *, time_t);
 void file_close(FILE *, const char *);
 void psleep(unsigned);
 int fork_exec(int *, int *, const char *, const char *const *);
 int shell_exec(int *, int *, const char *, const char *const *);
 int child_wait(int *, int *, int);
 void press_any_key(void);
-void print_apoint(const char *, long, struct apoint *);
-void print_event(const char *, long, struct event *);
-void print_recur_apoint(const char *, long, time_t, struct recur_apoint *);
-void print_recur_event(const char *, long, struct recur_event *);
+void print_apoint(const char *, time_t, struct apoint *);
+void print_event(const char *, time_t, struct event *);
+void print_recur_apoint(const char *, time_t, time_t, struct recur_apoint *);
+void print_recur_event(const char *, time_t, struct recur_event *);
 void print_todo(const char *, struct todo *);
 int vasprintf(char **, const char *, va_list);
 int asprintf(char **, const char *, ...);
@@ -1174,6 +1225,8 @@ int starts_with(const char *, const char *);
 int starts_with_ci(const char *, const char *);
 int hash_matches(const char *, const char *);
 int show_dialogs(void);
+long overflow_add(long, long, long *);
+long overflow_mul(long, long, long *);
 
 /* vars.c */
 extern int col, row;
@@ -1184,23 +1237,25 @@ extern enum ui_mode ui_mode;
 extern int read_only;
 extern int quiet;
 extern int want_reload;
-extern const char *datefmt_str[DATE_FORMATS];
-extern int days[12];
-extern char path_dir[BUFSIZ];
-extern char path_todo[BUFSIZ];
-extern char path_apts[BUFSIZ];
-extern char path_conf[BUFSIZ];
-extern char path_keys[BUFSIZ];
-extern char path_notes[BUFSIZ];
-extern char path_cpid[BUFSIZ];
-extern char path_dpid[BUFSIZ];
-extern char path_dmon_log[BUFSIZ];
-extern char path_hooks[BUFSIZ];
+extern const char *datefmt_str[];
+extern int days[];
+extern char *path_ddir;
+extern char *path_cdir;
+extern char *path_todo;
+extern char *path_apts;
+extern char *path_conf;
+extern char *path_keys;
+extern char *path_notes;
+extern char *path_cpid;
+extern char *path_dpid;
+extern char *path_dmon_log;
+extern char *path_hooks;
 extern struct conf conf;
 extern struct pad apad;
 extern struct nbar nbar;
 extern struct dmon_conf dmon;
 void vars_init(void);
+extern pthread_t notify_t_main, io_t_psave, ui_calendar_t_date;
 
 /* wins.c */
 extern struct window win[NBWINS];
@@ -1230,15 +1285,13 @@ void wins_slctd_next(void);
 void wins_init(void);
 void wins_scrollwin_init(struct scrollwin *, int, int, int, int, const char *);
 void wins_scrollwin_resize(struct scrollwin *, int, int, int, int);
-void wins_scrollwin_set_linecount(struct scrollwin *, unsigned);
+void wins_scrollwin_set_pad(struct scrollwin *, unsigned);
 void wins_scrollwin_delete(struct scrollwin *);
 void wins_scrollwin_draw_deco(struct scrollwin *, int);
-void wins_scrollwin_display(struct scrollwin *);
+void wins_scrollwin_display(struct scrollwin *, int);
 void wins_scrollwin_up(struct scrollwin *, int);
 void wins_scrollwin_down(struct scrollwin *, int);
-int wins_scrollwin_is_visible(struct scrollwin *, unsigned);
-void wins_scrollwin_ensure_visible(struct scrollwin *, unsigned);
-void wins_scrollwin_set_lower(struct scrollwin *, unsigned);
+void wins_scrollwin_in_view(struct scrollwin *, unsigned);
 void wins_resize(void);
 void wins_resize_panels(void);
 void wins_show(WINDOW *, const char *);
@@ -1255,7 +1308,14 @@ void wins_set_bindings(int *, int);
 void wins_update_bindings(void);
 void wins_status_bar(void);
 void wins_erase_status_bar(void);
-void wins_other_status_page(int);
+void wins_other_status_page();
 void wins_reset_status_page(void);
 
+/* queue.c */
+void que_init(void);
+int que_ued(void);
+struct event *que_ins(char *, time_t, int);
+void que_rem(void);
+void que_show(void);
+void que_save(void);
 #endif /* CALCURSE_H */
