@@ -94,6 +94,10 @@ static const struct confvar confmap[] = {
 	{"appearance.compactpanels", CONFIG_HANDLER_BOOL(conf.compact_panels)},
 	{"appearance.defaultpanel", config_parse_default_panel, config_serialize_default_panel, NULL},
 	{"appearance.layout", config_parse_layout, config_serialize_layout, NULL},
+	{"appearance.headerline", CONFIG_HANDLER_BOOL(conf.header_line)},
+	{"appearance.eventseparator", CONFIG_HANDLER_BOOL(conf.event_separator)},
+	{"appearance.dayseparator", CONFIG_HANDLER_BOOL(conf.day_separator)},
+	{"appearance.emptyline", CONFIG_HANDLER_BOOL(conf.empty_appt_line)},
 	{"appearance.notifybar", CONFIG_HANDLER_BOOL(nbar.show)},
 	{"appearance.sidebarwidth", config_parse_sidebar_width, config_serialize_sidebar_width, NULL},
 	{"appearance.theme", config_parse_color_theme, config_serialize_color_theme, NULL},
@@ -111,8 +115,9 @@ static const struct confvar confmap[] = {
 	{"general.confirmdelete", CONFIG_HANDLER_BOOL(conf.confirm_delete)},
 	{"general.confirmquit", CONFIG_HANDLER_BOOL(conf.confirm_quit)},
 	{"general.firstdayofweek", config_parse_first_day_of_week, config_serialize_first_day_of_week, NULL},
+	{"general.multipledays", CONFIG_HANDLER_BOOL(conf.multiple_days)},
 	{"general.periodicsave", CONFIG_HANDLER_UNSIGNED(conf.periodic_save)},
-	{"general.progressbar", CONFIG_HANDLER_BOOL(conf.progress_bar)},
+	{"general.systemevents", CONFIG_HANDLER_BOOL(conf.systemevents)},
 	{"general.systemdialogs", CONFIG_HANDLER_BOOL(conf.system_dialogs)},
 	{"notification.command", CONFIG_HANDLER_STR(nbar.cmd)},
 	{"notification.notifyall", config_parse_notifyall, config_serialize_notifyall, NULL},
@@ -151,18 +156,26 @@ static int config_parse_unsigned(unsigned *dest, const char *val)
 
 static int config_parse_int(int *dest, const char *val)
 {
-	if ((*val == '+' || *val == '-' || isdigit(*val))
-	    && is_all_digit(val + 1))
+	char *cp = (char *)val;
+
+	if (*val == '+' || *val == '-')
+		cp++;
+	/* Test for empty string before checking for digits only. */
+	if (*cp && is_all_digit(cp))
 		*dest = atoi(val);
 	else
 		return 0;
-
 	return 1;
 }
 
 static int config_parse_str(char *dest, const char *val)
 {
-	strncpy(dest, val, BUFSIZ);
+	int len = strlen(val);
+
+	if (len >= BUFSIZ)
+		return 0;
+
+	memcpy(dest, val, len + 1);
 	return 1;
 }
 
@@ -205,11 +218,13 @@ static int config_parse_color_pair(int *dest1, int *dest2, const char *val)
 
 static int config_parse_calendar_view(void *dummy, const char *val)
 {
-	if (!strcmp(val, "monthly"))
+	if (!strcmp(val, "monthly")) {
 		ui_calendar_set_view(CAL_MONTH_VIEW);
-	else if (!strcmp(val, "weekly"))
+		conf.cal_view = CAL_MONTH_VIEW;
+	} else if (!strcmp(val, "weekly")) {
 		ui_calendar_set_view(CAL_WEEK_VIEW);
-	else
+		conf.cal_view = CAL_WEEK_VIEW;
+	} else
 		return 0;
 
 	return 1;
@@ -217,11 +232,13 @@ static int config_parse_calendar_view(void *dummy, const char *val)
 
 static int config_parse_todo_view(void *dummy, const char *val)
 {
-	if (!strcmp(val, "show-completed"))
+	if (!strcmp(val, "show-completed")) {
 		ui_todo_set_view(TODO_SHOW_COMPLETED_VIEW);
-	else if (!strcmp(val, "hide-completed"))
+		conf.todo_view = TODO_SHOW_COMPLETED_VIEW;
+	} else if (!strcmp(val, "hide-completed")) {
 		ui_todo_set_view(TODO_HIDE_COMPLETED_VIEW);
-	else
+		conf.todo_view = TODO_HIDE_COMPLETED_VIEW;
+	} else
 		return 0;
 
 	return 1;
@@ -263,6 +280,7 @@ static int config_parse_color_theme(void *dummy, const char *val)
 	if (!config_parse_color_pair(&color1, &color2, val))
 		return 0;
 	init_pair(COLR_CUSTOM, color1, color2);
+	custom_init_attr();
 	return 1;
 }
 
@@ -417,7 +435,7 @@ static char *config_color_theme_name(void)
 
 static int config_serialize_calendar_view(char **buf, void *dummy)
 {
-	if (ui_calendar_get_view() == CAL_WEEK_VIEW)
+	if (conf.cal_view == CAL_WEEK_VIEW)
 		*buf = mem_strdup("weekly");
 	else
 		*buf = mem_strdup("monthly");
@@ -427,7 +445,7 @@ static int config_serialize_calendar_view(char **buf, void *dummy)
 
 static int config_serialize_todo_view(char **buf, void *dummy)
 {
-	if (ui_todo_get_view() == TODO_SHOW_COMPLETED_VIEW)
+	if (conf.todo_view == TODO_SHOW_COMPLETED_VIEW)
 		*buf = mem_strdup("show-completed");
 	else
 		*buf = mem_strdup("hide-completed");
@@ -592,6 +610,12 @@ config_file_walk(config_fn_walk_cb_t fn_cb,
 			WARN_MSG(_("Pre-3.0.0 configuration file format detected, "
 				  "please upgrade running `calcurse-upgrade`."));
 		}
+		/*
+		 * Backwards compatibility for removed configuration options:
+		 * ignored on load, omitted on save.
+		 */
+		if (strcmp(key, "general.progressbar") == 0)
+			continue;
 
 		if (value && (*value == '\0' || *value == '\n')) {
 			/* Backward compatibility mode. */
@@ -614,12 +638,9 @@ static int config_load_cb(const char *key, const char *value, void *dummy)
 	int result = config_set_conf(key, value);
 
 	if (result < 0) {
-		EXIT(_("configuration variable unknown: \"%s\""), key);
-		/* NOTREACHED */
+		WARN_MSG(_("unknown user option: \"%s\" (ignored)"), key);
 	} else if (result == 0) {
-		EXIT(_("wrong configuration variable format for \"%s\""),
-		     key);
-		/* NOTREACHED */
+		WARN_MSG(_("invalid option format: \"%s\" (ignored)"), key);
 	}
 
 	return 1;
@@ -639,12 +660,13 @@ static int config_save_cb(const char *key, const char *value, void *status)
 				  (struct config_save_status *)status);
 
 	if (result < 0) {
-		EXIT(_("configuration variable unknown: \"%s\""), key);
-		/* NOTREACHED */
+		WARN_MSG(_("unknown user option: \"%s\" (disabled)"), key);
 	} else if (result == 0) {
-		EXIT(_("wrong configuration variable format for \"%s\""),
-		     key);
-		/* NOTREACHED */
+		WARN_MSG(_("invalid option format: \"%s\" (disabled)"), key);
+	}
+	if (result <= 0) {
+		fputc('#', ((struct config_save_status *)status)->fp);
+		buf = mem_strdup(value);
 	}
 
 	fputs(key, ((struct config_save_status *)status)->fp);
