@@ -52,13 +52,19 @@ static struct attribute attr;
  * Define window attributes (for both color and non-color terminals):
  * ATTR_HIGHEST are for window titles
  * ATTR_HIGH are for month and days names
- * ATTR_MIDDLE are for the selected day inside calendar panel
+ * ATTR_MIDDLE are for the selected day inside calendar and appointments panel
  * ATTR_LOW are for days inside calendar panel which contains an event
  * ATTR_LOWEST are for current day inside calendar panel
  */
 void custom_init_attr(void)
 {
-	attr.color[ATTR_HIGHEST] = COLOR_PAIR(COLR_CUSTOM);
+	short col_fg, dummy;
+	pair_content(COLR_CUSTOM, &col_fg, &dummy);
+
+	attr.color[ATTR_HIGHEST] =
+		(col_fg == -1 || col_fg == 255) ?
+		COLOR_PAIR(COLR_CUSTOM) | A_BOLD :
+		COLOR_PAIR(COLR_CUSTOM);
 	attr.color[ATTR_HIGH] = COLOR_PAIR(COLR_HIGH);
 	attr.color[ATTR_MIDDLE] = COLOR_PAIR(COLR_RED) | A_BOLD;
 	attr.color[ATTR_LOW] = COLOR_PAIR(COLR_CYAN);
@@ -93,6 +99,41 @@ void custom_remove_attr(WINDOW * win, int attr_num)
 		wattroff(win, attr.nocolor[attr_num]);
 }
 
+static void set_confwin_attr(struct window *cwin)
+{
+	cwin->h = (notify_bar())? row - 3 : row - 2;
+	cwin->w = col;
+	cwin->x = cwin->y = 0;
+}
+
+/*
+ * Create a configuration window and initialize status and notification bar
+ * (useful in case of window resize).
+ */
+static void confwin_init(struct window *confwin, const char *label)
+{
+	if (confwin->p) {
+		erase_window_part(confwin->p, confwin->x, confwin->y,
+				  confwin->x + confwin->w,
+				  confwin->y + confwin->h);
+		delwin(confwin->p);
+	}
+
+	wins_get_config();
+	set_confwin_attr(confwin);
+	confwin->p = newwin(confwin->h, col, 0, 0);
+	box(confwin->p, 0, 0);
+	wins_show(confwin->p, label);
+	delwin(win[STA].p);
+	win[STA].p =
+	    newwin(win[STA].h, win[STA].w, win[STA].y, win[STA].x);
+	keypad(win[STA].p, TRUE);
+	if (notify_bar()) {
+		notify_reinit_bar();
+		notify_update_bar();
+	}
+}
+
 static void layout_selection_bar(void)
 {
 	static int bindings[] = {
@@ -112,7 +153,6 @@ static void layout_selection_bar(void)
 static void display_layout_config(struct window *lwin, int mark,
 				  int cursor)
 {
-#define CURSOR			(32 | A_REVERSE)
 #define MARK			88
 #define LAYOUTH                  5
 #define LAYOUTW                  9
@@ -158,8 +198,9 @@ static void display_layout_config(struct window *lwin, int mark,
 			custom_remove_attr(lwin->p, ATTR_HIGHEST);
 	}
 	mvwaddch(lwin->p, pos[mark][YPOS] + 2, pos[mark][XPOS] + 1, MARK);
-	mvwaddch(lwin->p, pos[cursor][YPOS] + 2, pos[cursor][XPOS] + 1,
-		 CURSOR);
+	/* print cursor */
+	wmove(lwin->p, pos[cursor][YPOS] + 2, pos[cursor][XPOS] + 1);
+	wchgat(lwin->p, 1, A_REVERSE, (colorize ? COLR_CUSTOM : 0), NULL);
 
 	layout_selection_bar();
 	wnoutrefresh(win[STA].p);
@@ -177,7 +218,7 @@ void custom_layout_config(void)
 	const char *label = _("layout configuration");
 
 	conf_win.p = NULL;
-	custom_confwin_init(&conf_win, label);
+	confwin_init(&conf_win, label);
 	cursor = mark = wins_layout() - 1;
 	display_layout_config(&conf_win, mark, cursor);
 	clear();
@@ -218,7 +259,7 @@ void custom_layout_config(void)
 		}
 
 		if (need_reset)
-			custom_confwin_init(&conf_win, label);
+			confwin_init(&conf_win, label);
 
 		display_layout_config(&conf_win, mark, cursor);
 	}
@@ -270,41 +311,6 @@ void custom_sidebar_config(void)
 	}
 }
 
-static void set_confwin_attr(struct window *cwin)
-{
-	cwin->h = (notify_bar())? row - 3 : row - 2;
-	cwin->w = col;
-	cwin->x = cwin->y = 0;
-}
-
-/*
- * Create a configuration window and initialize status and notification bar
- * (useful in case of window resize).
- */
-void custom_confwin_init(struct window *confwin, const char *label)
-{
-	if (confwin->p) {
-		erase_window_part(confwin->p, confwin->x, confwin->y,
-				  confwin->x + confwin->w,
-				  confwin->y + confwin->h);
-		delwin(confwin->p);
-	}
-
-	wins_get_config();
-	set_confwin_attr(confwin);
-	confwin->p = newwin(confwin->h, col, 0, 0);
-	box(confwin->p, 0, 0);
-	wins_show(confwin->p, label);
-	delwin(win[STA].p);
-	win[STA].p =
-	    newwin(win[STA].h, win[STA].w, win[STA].y, win[STA].x);
-	keypad(win[STA].p, TRUE);
-	if (notify_bar()) {
-		notify_reinit_bar();
-		notify_update_bar();
-	}
-}
-
 static void color_selection_bar(void)
 {
 	static int bindings[] = {
@@ -328,7 +334,6 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 #define	SIZE 			(2 * (NBUSERCOLORS + 1))
 #define DEFAULTCOLOR		255
 #define DEFAULTCOLOR_EXT	-1
-#define CURSOR			(32 | A_REVERSE)
 #define MARK			88
 
 	const char *fore_txt = _("Foreground");
@@ -371,6 +376,7 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 			if (colr_back == 255)
 				colr_back = -1;
 			init_pair(COLR_CUSTOM, colr_fore, colr_back);
+			custom_init_attr();
 		} else {
 			/* Retrieve the actual color theme. */
 			pair_content(COLR_CUSTOM, &colr_fore, &colr_back);
@@ -379,7 +385,8 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 			    || (colr_fore == DEFAULTCOLOR_EXT)) {
 				*mark_fore = NBUSERCOLORS;
 			} else {
-				for (i = 0; i < NBUSERCOLORS + 1; i++)
+				for (i = 0; i < NBUSERCOLORS; i++)
+				/* WARNING. Colour pair number used as colour number. */
 					if (colr_fore == colr[i])
 						*mark_fore = i;
 			}
@@ -388,7 +395,8 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 			    || (colr_back == DEFAULTCOLOR_EXT)) {
 				*mark_back = SIZE - 1;
 			} else {
-				for (i = 0; i < NBUSERCOLORS + 1; i++)
+				for (i = 0; i < NBUSERCOLORS; i++)
+				/* WARNING. Colour pair number used as colour number. */
 					if (colr_back ==
 					    colr[NBUSERCOLORS + 1 + i])
 						*mark_back =
@@ -429,8 +437,8 @@ display_color_config(struct window *cwin, int *mark_fore, int *mark_back,
 			 pos[*mark_back][XPOS] + 1, MARK);
 	}
 
-	mvwaddch(cwin->p, pos[cursor][YPOS], pos[cursor][XPOS] + 1,
-		 CURSOR);
+	wmove(cwin->p, pos[cursor][YPOS], pos[cursor][XPOS] + 1);
+	wchgat(cwin->p, 1, A_REVERSE, (colorize ? COLR_CUSTOM : 0), NULL);
 	color_selection_bar();
 	wnoutrefresh(win[STA].p);
 	wnoutrefresh(cwin->p);
@@ -448,7 +456,7 @@ void custom_color_config(void)
 	const char *label = _("color theme");
 
 	conf_win.p = 0;
-	custom_confwin_init(&conf_win, label);
+	confwin_init(&conf_win, label);
 	mark_fore = NBUSERCOLORS;
 	mark_back = SIZE - 1;
 	cursor = 0;
@@ -507,7 +515,7 @@ void custom_color_config(void)
 		}
 
 		if (need_reset)
-			custom_confwin_init(&conf_win, label);
+			confwin_init(&conf_win, label);
 
 		display_color_config(&conf_win, &mark_fore, &mark_back,
 				     cursor, theme_changed);
@@ -518,13 +526,20 @@ void custom_color_config(void)
 enum {
 	COMPACT_PANELS,
 	DEFAULT_PANEL,
+	CAL_VIEW,
+	TODO_VIEW,
+	MULTIPLE_DAYS,
+	HEADER_LINE,
+	EVENT_SEPARATOR,
+	DAY_SEPARATOR,
+	EMPTY_APPT_LINE,
 	AUTO_SAVE,
 	AUTO_GC,
 	PERIODIC_SAVE,
+	SYSTEM_EVENTS,
 	CONFIRM_QUIT,
 	CONFIRM_DELETE,
 	SYSTEM_DIAGS,
-	PROGRESS_BAR,
 	FIRST_DAY_OF_WEEK,
 	OUTPUT_DATE_FMT,
 	INPUT_DATE_FMT,
@@ -540,13 +555,20 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 	char *opt[NB_OPTIONS] = {
 		"appearance.compactpanels = ",
 		"appearance.defaultpanel = ",
+		"appearance.calendarview = ",
+		"appearance.todoview = ",
+		"general.multipledays = ",
+		"appearance.headerline = ",
+		"appearance.eventseparator = ",
+		"appearance.dayseparator = ",
+		"appearance.emptyline = ",
 		"general.autosave = ",
 		"general.autogc = ",
 		"general.periodicsave = ",
+		"general.systemevents = ",
 		"general.confirmquit = ",
 		"general.confirmdelete = ",
 		"general.systemdialogs = ",
-		"general.progressbar = ",
 		"general.firstdayofweek = ",
 		"format.outputdate = ",
 		"format.inputdate = ",
@@ -580,6 +602,52 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(specifies the panel that is selected by default)"));
 		break;
+	case CAL_VIEW:
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[CAL_VIEW]),
+			  conf.cal_view == CAL_MONTH_VIEW ?
+			  _("monthly") : _("weekly"));
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS, _("(preferred calendar display)"));
+		break;
+	case TODO_VIEW:
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[TODO_VIEW]),
+			  conf.todo_view == TODO_SHOW_COMPLETED_VIEW ?
+			  _("show completed") : _("hide completed"));
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS, _("(preferred todo display)"));
+		break;
+	case HEADER_LINE:
+		print_bool_option_incolor(win, conf.header_line, y,
+					  XPOS + strlen(opt[HEADER_LINE]));
+		mvwaddstr(win, y + XPOS, 1,
+			  _("(horizontal line above the day heading)"));
+		break;
+	case EVENT_SEPARATOR:
+		print_bool_option_incolor(win, conf.event_separator, y,
+					  XPOS + strlen(opt[EVENT_SEPARATOR]));
+		mvwaddstr(win, y + XPOS, 1,
+			  _("(empty line between events and appointments)"));
+		break;
+	case DAY_SEPARATOR:
+		print_bool_option_incolor(win, conf.day_separator, y,
+					  XPOS + strlen(opt[DAY_SEPARATOR]));
+		mvwaddstr(win, y + XPOS, 1,
+			  _("(each day ends with an empty line)"));
+		break;
+	case EMPTY_APPT_LINE:
+		print_bool_option_incolor(win, conf.empty_appt_line, y,
+					  XPOS + strlen(opt[EMPTY_APPT_LINE]));
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(insert an empty line after each appointment)"));
+		break;
+	case MULTIPLE_DAYS:
+		print_bool_option_incolor(win, conf.multiple_days, y,
+					  XPOS + strlen(opt[MULTIPLE_DAYS]));
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(display more than one day in the appointments panel)"));
+		break;
 	case AUTO_SAVE:
 		print_bool_option_incolor(win, conf.auto_save, y,
 					  XPOS + strlen(opt[AUTO_SAVE]));
@@ -598,8 +666,15 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 			  conf.periodic_save);
 		custom_remove_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y + 1, XPOS,
-			  _("(if not null, automatically save data every 'periodic_save' "
-			   "minutes)"));
+			  _("(if not null, automatically save data every "
+			  "'periodic_save' minutes)"));
+		break;
+	case SYSTEM_EVENTS:
+		print_bool_option_incolor(win, conf.systemevents, y,
+					  XPOS + strlen(opt[SYSTEM_EVENTS]));
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(if YES, system events are turned into "
+			  "appointments (or else deleted))"));
 		break;
 	case CONFIRM_QUIT:
 		print_bool_option_incolor(win, conf.confirm_quit, y,
@@ -621,13 +696,6 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 			  _("(if set to YES, messages about loaded "
 			    "and saved data will be displayed)"));
 		break;
-	case PROGRESS_BAR:
-		print_bool_option_incolor(win, conf.progress_bar, y,
-					  XPOS + strlen(opt[PROGRESS_BAR]));
-		mvwaddstr(win, y + 1, XPOS,
-			  _("(if set to YES, progress bar will be displayed "
-			    "when saving data)"));
-		break;
 	case FIRST_DAY_OF_WEEK:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y, XPOS + strlen(opt[FIRST_DAY_OF_WEEK]),
@@ -720,6 +788,36 @@ static void general_option_edit(int i)
 		else
 			conf.default_panel++;
 		break;
+	case CAL_VIEW:
+		if (conf.cal_view == CAL_WEEK_VIEW)
+			conf.cal_view = CAL_MONTH_VIEW;
+		else
+			conf.cal_view++;
+		ui_calendar_set_view(conf.cal_view);
+		break;
+	case TODO_VIEW:
+		if (conf.todo_view == TODO_HIDE_COMPLETED_VIEW)
+			conf.todo_view = TODO_SHOW_COMPLETED_VIEW;
+		else
+			conf.todo_view++;
+		ui_todo_set_view(conf.todo_view);
+		ui_todo_load_items();
+		break;
+	case MULTIPLE_DAYS:
+		conf.multiple_days = !conf.multiple_days;
+		break;
+	case HEADER_LINE:
+		conf.header_line = !conf.header_line;
+		break;
+	case EVENT_SEPARATOR:
+		conf.event_separator = !conf.event_separator;
+		break;
+	case DAY_SEPARATOR:
+		conf.day_separator = !conf.day_separator;
+		break;
+	case EMPTY_APPT_LINE:
+		conf.empty_appt_line = !conf.empty_appt_line;
+		break;
 	case HEADING_POS:
 		if (conf.heading_pos == RIGHT)
 			conf.heading_pos = LEFT;
@@ -734,16 +832,20 @@ static void general_option_edit(int i)
 		break;
 	case PERIODIC_SAVE:
 		status_mesg(periodic_save_str, "");
+		snprintf(buf, BUFSIZ, "%d", conf.periodic_save);
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			val = atoi(buf);
-			if (val >= 0)
+			if (val >= 0) {
 				conf.periodic_save = val;
-			if (conf.periodic_save > 0)
-				io_start_psave_thread();
-			else if (conf.periodic_save == 0)
 				io_stop_psave_thread();
+				if (conf.periodic_save > 0)
+					io_start_psave_thread();
+			}
 		}
 		break;
+	case SYSTEM_EVENTS:
+		conf.systemevents = !conf.systemevents;
+		break;
 	case CONFIRM_QUIT:
 		conf.confirm_quit = !conf.confirm_quit;
 		break;
@@ -753,11 +855,9 @@ static void general_option_edit(int i)
 	case SYSTEM_DIAGS:
 		conf.system_dialogs = !conf.system_dialogs;
 		break;
-	case PROGRESS_BAR:
-		conf.progress_bar = !conf.progress_bar;
-		break;
 	case FIRST_DAY_OF_WEEK:
 		ui_calendar_change_first_day_of_week();
+		ui_calendar_monthly_view_cache_set_invalid();
 		break;
 	case OUTPUT_DATE_FMT:
 		status_mesg(output_datefmt_str, "");
@@ -781,7 +881,7 @@ static void general_option_edit(int i)
 		buf[BUFSIZ - 1] = '\0';
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			strncpy(conf.day_heading, buf, BUFSIZ);
-			conf.output_datefmt[BUFSIZ - 1] = '\0';
+			conf.day_heading[BUFSIZ - 1] = '\0';
 		}
 		break;
 	}
@@ -804,7 +904,7 @@ void custom_general_config(void)
 		     general_option_height, print_general_option);
 	listbox_load_items(&lb, NB_OPTIONS);
 	listbox_draw_deco(&lb, 0);
-	listbox_display(&lb);
+	listbox_display(&lb, NOHILT);
 	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
 	wins_status_bar();
 	wnoutrefresh(win[STA].p);
@@ -838,7 +938,7 @@ void custom_general_config(void)
 			}
 		}
 
-		listbox_display(&lb);
+		listbox_display(&lb, NOHILT);
 		wins_status_bar();
 		wnoutrefresh(win[STA].p);
 		wmove(win[STA].p, 0, 0);
@@ -900,7 +1000,7 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 						mvwprintw(win, y, pos,
 							  "%s ", key);
 					noelm++;
-					pos += strlen(key) + 1;
+					pos += utf8_strwidth((char *)key) + 1;
 				}
 			} else {
 				mvwaddstr(win, y, KEYPOS,
@@ -938,12 +1038,12 @@ void custom_keys_config(void)
 	clear();
 	nbdisplayed = ((notify_bar() ? row - 3 : row - 2) - LABELLINES) / LINESPERKEY;
 	wins_scrollwin_init(&kwin, 0, 0, notify_bar() ? row - 3 : row - 2, col, _("keys configuration"));
-	wins_scrollwin_set_linecount(&kwin, NBKEYS * LINESPERKEY);
+	wins_scrollwin_set_pad(&kwin, NBKEYS * LINESPERKEY);
 	wins_scrollwin_draw_deco(&kwin, 0);
 	custom_keys_config_bar();
 	selrow = selelm = 0;
 	nbrowelm = print_keys_bindings(kwin.inner, selrow, selelm, LINESPERKEY);
-	wins_scrollwin_display(&kwin);
+	wins_scrollwin_display(&kwin, NOHILT);
 	firstrow = 0;
 	lastrow = firstrow + nbdisplayed - 1;
 	for (;;) {
@@ -1012,7 +1112,7 @@ void custom_keys_config(void)
 								selrow,
 								selelm,
 								LINESPERKEY);
-					wins_scrollwin_display(&kwin);
+					wins_scrollwin_display(&kwin, NOHILT);
 					wins_redrawwin(grabwin);
 					continue;
 				}
@@ -1065,7 +1165,7 @@ void custom_keys_config(void)
 		nbrowelm =
 		    print_keys_bindings(kwin.inner, selrow, selelm,
 					LINESPERKEY);
-		wins_scrollwin_display(&kwin);
+		wins_scrollwin_display(&kwin, NOHILT);
 	}
 }
 
@@ -1146,4 +1246,8 @@ void custom_config_main(void)
 		wmove(win[STA].p, 0, 0);
 		wins_doupdate();
 	}
+	if (!config_save())
+		EXIT(_("Could not save %s."), path_conf);
+	if (!io_save_keys())
+		EXIT(_("Could not save %s."), path_keys);
 }
