@@ -43,6 +43,8 @@
 #include "sha1.h"
 
 llist_t eventlist;
+/* Dummy event for the APP panel for an otherwise empty day. */
+struct event dummy = { DUMMY, 0, "", NULL };
 
 void event_free(struct event *ev)
 {
@@ -89,7 +91,7 @@ static int event_cmp(struct event *a, struct event *b)
 }
 
 /* Create a new event */
-struct event *event_new(char *mesg, char *note, long day, int id)
+struct event *event_new(char *mesg, char *note, time_t day, int id)
 {
 	struct event *ev;
 
@@ -105,7 +107,7 @@ struct event *event_new(char *mesg, char *note, long day, int id)
 }
 
 /* Check if the event belongs to the selected day */
-unsigned event_inday(struct event *i, long *start)
+unsigned event_inday(struct event *i, time_t *start)
 {
 	return (date_cmp_day(i->day, *start) == 0);
 }
@@ -152,7 +154,8 @@ struct event *event_scan(FILE * f, struct tm start, int id, char *note,
 {
 	char buf[BUFSIZ], *nl;
 	time_t tstart, tend;
-	struct event *ev;
+	struct event *ev = NULL;
+	int cond;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_time(start.tm_hour, start.tm_min),
@@ -179,31 +182,29 @@ struct event *event_scan(FILE * f, struct tm start, int id, char *note,
 
 	/* Filter item. */
 	if (filter) {
-		if (!(filter->type_mask & TYPE_MASK_EVNT))
-			return NULL;
-		if (filter->regex && regexec(filter->regex, buf, 0, 0, 0))
-			return NULL;
-		if (filter->start_from >= 0 && tstart < filter->start_from)
-			return NULL;
-		if (filter->start_to >= 0 && tstart > filter->start_to)
+		cond = (
+		    !(filter->type_mask & TYPE_MASK_EVNT) ||
+		    (filter->regex && regexec(filter->regex, buf, 0, 0, 0)) ||
+		    (filter->start_from != -1 && tstart < filter->start_from) ||
+		    (filter->start_to != -1 && tstart > filter->start_to) ||
+		    (filter->end_from != -1 && tend < filter->end_from) ||
+		    (filter->end_to != -1 && tend > filter->end_to)
+		);
+		if (filter->hash) {
+			ev = event_new(buf, note, tstart, id);
+			char *hash = event_hash(ev);
+			cond = cond || !hash_matches(filter->hash, hash);
+			mem_free(hash);
+		}
+
+		if ((!filter->invert && cond) || (filter->invert && !cond)) {
+			if (filter->hash)
+				event_delete(ev);
 			return NULL;
-		if (filter->end_from >= 0 && tend < filter->end_from)
-			return NULL;
-		if (filter->end_to >= 0 && tend > filter->end_to)
-			return NULL;
-	}
-
-	ev = event_new(buf, note, tstart, id);
-
-	/* Filter by hash. */
-	if (filter && filter->hash) {
-		char *hash = event_hash(ev);
-		if (!hash_matches(filter->hash, hash)) {
-			event_delete(ev);
-			ev = NULL;
 		}
-		mem_free(hash);
 	}
+	if (!ev)
+		ev = event_new(buf, note, tstart, id);
 
 	return ev;
 }
@@ -219,8 +220,14 @@ void event_delete(struct event *ev)
 	LLIST_REMOVE(&eventlist, i);
 }
 
-void event_paste_item(struct event *ev, long date)
+void event_paste_item(struct event *ev, time_t date)
 {
 	ev->day = date;
 	LLIST_ADD_SORTED(&eventlist, ev, event_cmp);
 }
+
+/* Return true if the day_item is the dummy event. */
+int event_dummy(struct day_item *item)
+{
+	return item->item.ev == &dummy;
+}
