@@ -70,7 +70,7 @@ static void getstr_print(WINDOW * win, int x, int y,
 
 	/* print cursor */
 	wmove(win, y, st->ci[st->pos].dpyoff - st->ci[st->scrpos].dpyoff);
-	wchgat(win, 1, A_REVERSE, COLR_CUSTOM, NULL);
+	wchgat(win, 1, A_REVERSE, (colorize ? COLR_CUSTOM : 0), NULL);
 }
 
 /* Delete a character at the given position in string. */
@@ -191,7 +191,8 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 		getstr_print(win, x, y, &st);
 		wins_doupdate();
 
-		if ((ch = wgetch(win)) == '\n')
+		ch = wgetch(win);
+		if ((ch == '\n') || (ch == KEY_ENTER))
 			break;
 		switch (ch) {
 		case KEY_BACKSPACE:	/* delete one character */
@@ -259,6 +260,9 @@ enum getstr getstring(WINDOW * win, char *str, int l, int x, int y)
 		case CTRL('G'):
 			return GETSTRING_ESC;
 			break;
+		case ERR:
+		case KEY_RESIZE:
+			continue;
 		default:	/* insert one character */
 			c[0] = ch;
 			for (k = 1;
