@@ -123,7 +123,7 @@ static void ical_export_recur_events(FILE * stream, int export_uid)
 			fputs("EXDATE:", stream);
 			LLIST_FOREACH(&rev->exc, j) {
 				struct excp *exc = LLIST_GET_DATA(j);
-				date_sec2date_fmt(exc->st, ICALDATEFMT,
+				date_sec2date_fmt(exc->st, ICALDATETIMEFMT,
 						  ical_date);
 				fprintf(stream, "%s", ical_date);
 				fputc(LLIST_NEXT(j) ? ',' : '\n', stream);
@@ -152,7 +152,7 @@ static void ical_export_events(FILE * stream, int export_uid)
 		struct event *ev = LLIST_TS_GET_DATA(i);
 		date_sec2date_fmt(ev->day, ICALDATEFMT, ical_date);
 		fputs("BEGIN:VEVENT\n", stream);
-		fprintf(stream, "DTSTART:%s\n", ical_date);
+		fprintf(stream, "DTSTART;VALUE=DATE:%s\n", ical_date);
 		fprintf(stream, "SUMMARY:%s\n", ev->mesg);
 
 		if (export_uid) {
@@ -202,7 +202,7 @@ static void ical_export_recur_apoints(FILE * stream, int export_uid)
 			fputs("EXDATE:", stream);
 			LLIST_FOREACH(&rapt->exc, j) {
 				struct excp *exc = LLIST_GET_DATA(j);
-				date_sec2date_fmt(exc->st, ICALDATEFMT,
+				date_sec2date_fmt(exc->st, ICALDATETIMEFMT,
 						  ical_date);
 				fprintf(stream, "%s", ical_date);
 				fputc(LLIST_NEXT(j) ? ',' : '\n', stream);
@@ -1087,7 +1087,7 @@ ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos, unsigned *noskipped,
 		}
 
 		if (starts_with_ci(buf, "PRIORITY:")) {
-			sscanf(buf, "%d", &vtodo.priority);
+			sscanf(buf, "PRIORITY:%d\n", &vtodo.priority);
 			if (vtodo.priority < 0 || vtodo.priority > 9) {
 				ical_log(log, ICAL_VTODO, ITEMLINE,
 					 _("item priority is invalid "
