@@ -48,25 +48,6 @@
 #include "calcurse.h"
 #include "sha1.h"
 
-typedef enum {
-	PROGRESS_BAR_SAVE,
-	PROGRESS_BAR_LOAD,
-	PROGRESS_BAR_EXPORT
-} progress_bar_t;
-
-enum {
-	PROGRESS_BAR_CONF,
-	PROGRESS_BAR_TODO,
-	PROGRESS_BAR_APTS,
-	PROGRESS_BAR_KEYS
-};
-
-enum {
-	PROGRESS_BAR_EXPORT_EVENTS,
-	PROGRESS_BAR_EXPORT_APOINTS,
-	PROGRESS_BAR_EXPORT_TODO
-};
-
 struct ht_keybindings_s {
 	const char *label;
 	enum key key;
@@ -88,74 +69,6 @@ static int modified = 0;
 static char apts_sha1[SHA1_DIGESTLEN * 2 + 1];
 static char todo_sha1[SHA1_DIGESTLEN * 2 + 1];
 
-/* Draw a progress bar while saving, loading or exporting data. */
-static void progress_bar(progress_bar_t type, int progress)
-{
-#define NBFILES		4
-#define NBEXPORTED      3
-#define LABELENGTH      15
-	int i, step, steps;
-	const char *mesg_sav = _("Saving...");
-	const char *mesg_load = _("Loading...");
-	const char *mesg_export = _("Exporting...");
-	const char *error_msg =
-	    _("Internal error while displaying progress bar");
-	const char *barchar = "|";
-	const char *file[NBFILES] = {
-		"[    conf    ]",
-		"[    todo    ]",
-		"[    apts    ]",
-		"[    keys    ]"
-	};
-	const char *data[NBEXPORTED] = {
-		"[   events   ]",
-		"[appointments]",
-		"[    todo    ]"
-	};
-	int ipos = LABELENGTH + 2;
-	int epos[NBFILES];
-
-	/* progress bar length init. */
-	ipos = LABELENGTH + 2;
-	steps = (type == PROGRESS_BAR_EXPORT) ? NBEXPORTED : NBFILES;
-	step = floor(col / (steps + 1));
-	for (i = 0; i < steps - 1; i++)
-		epos[i] = (i + 2) * step;
-	epos[steps - 1] = col - 2;
-
-	switch (type) {
-	case PROGRESS_BAR_SAVE:
-		EXIT_IF(progress < 0
-			|| progress > PROGRESS_BAR_KEYS, "%s", error_msg);
-		status_mesg(mesg_sav, file[progress]);
-		break;
-	case PROGRESS_BAR_LOAD:
-		EXIT_IF(progress < 0
-			|| progress > PROGRESS_BAR_KEYS, "%s", error_msg);
-		status_mesg(mesg_load, file[progress]);
-		break;
-	case PROGRESS_BAR_EXPORT:
-		EXIT_IF(progress < 0
-			|| progress > PROGRESS_BAR_EXPORT_TODO, "%s",
-			error_msg);
-		status_mesg(mesg_export, data[progress]);
-		break;
-	}
-
-	/* Draw the progress bar. */
-	mvwaddstr(win[STA].p, 1, ipos, barchar);
-	mvwaddstr(win[STA].p, 1, epos[steps - 1], barchar);
-	custom_apply_attr(win[STA].p, ATTR_HIGHEST);
-	for (i = ipos + 1; i < epos[progress]; i++)
-		mvwaddch(win[STA].p, 1, i, ' ' | A_REVERSE);
-	custom_remove_attr(win[STA].p, ATTR_HIGHEST);
-	wmove(win[STA].p, 0, 0);
-	wins_wrefresh(win[STA].p);
-#undef NBFILES
-#undef NBEXPORTED
-#undef LABELENGTH
-}
-
 /* Ask user for a file name to export data to. */
 static FILE *get_export_stream(enum export_type type)
 {
@@ -225,52 +138,41 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
  * which contains the calendar file. If none is given, then the default
  * one (~/.calcurse/apts) is taken. If the one given does not exist, it
  * is created.
- * The datadir argument can be use to specify an alternative data root dir.
+ * The datadir argument can be used to specify an alternative data root dir.
+ * The confdir argument can be used to specify an alternative configuration dir.
  */
-void io_init(const char *cfile, const char *datadir)
+void io_init(const char *cfile, const char *datadir, const char *confdir)
 {
-	const char *home;
+	if (!datadir) {
+		if (!(datadir = getenv("HOME")))
+			datadir = ".";
+		asprintf(&path_ddir, "%s%s", datadir, "/" DIR_NAME);
+	} else
+		asprintf(&path_ddir, "%s%s", datadir, "/");
 
-	if (datadir != NULL) {
-		home = datadir;
-		snprintf(path_dir, BUFSIZ, "%s", home);
-		snprintf(path_todo, BUFSIZ, "%s/" TODO_PATH_NAME, home);
-		snprintf(path_conf, BUFSIZ, "%s/" CONF_PATH_NAME, home);
-		snprintf(path_notes, BUFSIZ, "%s/" NOTES_DIR_NAME, home);
-		snprintf(path_keys, BUFSIZ, "%s/" KEYS_PATH_NAME, home);
-		snprintf(path_cpid, BUFSIZ, "%s/" CPID_PATH_NAME, home);
-		snprintf(path_dpid, BUFSIZ, "%s/" DPID_PATH_NAME, home);
-		snprintf(path_dmon_log, BUFSIZ, "%s/" DLOG_PATH_NAME,
-			 home);
-		snprintf(path_hooks, BUFSIZ, "%s/" HOOKS_DIR_NAME, home);
-	} else {
-		home = getenv("HOME");
-		if (home == NULL) {
-			home = ".";
-		}
-		snprintf(path_dir, BUFSIZ, "%s/" DIR_NAME, home);
-		snprintf(path_todo, BUFSIZ, "%s/" TODO_PATH, home);
-		snprintf(path_conf, BUFSIZ, "%s/" CONF_PATH, home);
-		snprintf(path_keys, BUFSIZ, "%s/" KEYS_PATH, home);
-		snprintf(path_cpid, BUFSIZ, "%s/" CPID_PATH, home);
-		snprintf(path_dpid, BUFSIZ, "%s/" DPID_PATH, home);
-		snprintf(path_dmon_log, BUFSIZ, "%s/" DLOG_PATH, home);
-		snprintf(path_notes, BUFSIZ, "%s/" NOTES_DIR, home);
-		snprintf(path_hooks, BUFSIZ, "%s/" HOOKS_DIR, home);
-	}
+	if (!confdir)
+		asprintf(&path_cdir, "%s%s", path_ddir, "/");
+	else
+		asprintf(&path_cdir, "%s%s", confdir, "/");
 
-	if (cfile == NULL) {
-		if (datadir != NULL) {
-			snprintf(path_apts, BUFSIZ, "%s/" APTS_PATH_NAME,
-				 home);
-		} else {
-			snprintf(path_apts, BUFSIZ, "%s/" APTS_PATH, home);
-		}
-	} else {
-		snprintf(path_apts, BUFSIZ, "%s", cfile);
+	/* Data files */
+	if (cfile) {
+		path_apts = mem_strdup(cfile);
 		EXIT_IF(!io_file_exists(path_apts), _("%s does not exist"),
 			path_apts);
+	} else {
+		asprintf(&path_apts, "%s%s", path_ddir, APTS_PATH_NAME);
 	}
+	asprintf(&path_todo, "%s%s", path_ddir, TODO_PATH_NAME);
+	asprintf(&path_cpid, "%s%s", path_ddir, CPID_PATH_NAME);
+	asprintf(&path_dpid, "%s%s", path_ddir, DPID_PATH_NAME);
+	asprintf(&path_notes, "%s%s", path_ddir, NOTES_DIR_NAME);
+	asprintf(&path_dmon_log, "%s%s", path_ddir, DLOG_PATH_NAME);
+
+	/* Configuration files */
+	asprintf(&path_conf, "%s%s", path_cdir, CONF_PATH_NAME);
+	asprintf(&path_keys, "%s%s", path_cdir, KEYS_PATH_NAME);
+	asprintf(&path_hooks, "%s%s", path_cdir, HOOKS_DIR_NAME);
 }
 
 void io_extract_data(char *dst_data, const char *org, int len)
@@ -287,6 +189,7 @@ void io_extract_data(char *dst_data, const char *org, int len)
 }
 
 static pthread_mutex_t io_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t io_periodic_save_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static void io_mutex_lock(void)
 {
@@ -438,6 +341,7 @@ static int io_compute_hash(const char *path, char *buf)
 	return 1;
 }
 
+/* A merge implies a save operation and must be followed by reload of data. */
 static void io_merge_data(void)
 {
 	char *path_apts_new, *path_todo_new;
@@ -446,10 +350,8 @@ static void io_merge_data(void)
 	asprintf(&path_apts_new, "%s%s", path_apts, new_ext);
 	asprintf(&path_todo_new, "%s%s", path_todo, new_ext);
 
-	io_mutex_lock();
 	io_save_apts(path_apts_new);
 	io_save_todo(path_todo_new);
-	io_mutex_unlock();
 
 	/*
 	 * We do not directly write to the data files here; however, the
@@ -473,139 +375,140 @@ static void io_merge_data(void)
 	mem_free(path_apts_new);
 	mem_free(path_todo_new);
 
-	io_unset_modified();
-
 	/*
 	 * We do not directly write to the data files here; however, the
 	 * external merge tool will likely have incorporated changes from the
 	 * new file into the main data files at this point.
 	 */
 	run_hook("post-save");
+
+	/*
+	 * The user has merged, so override the modified flag
+	 * (and follow up with reload of the data files).
+	 */
+	io_unset_modified();
 }
 
+/* For the return values, see io_save_cal() below. */
 static int resolve_save_conflict(void)
 {
 	char *msg_um_asktype = NULL;
 	const char *msg_um_prefix =
-			_("Data files were changed since reading:");
-	const char *msg_um_overwrite = _("(o)verwrite");
+			_("Data files have changed and will be overwritten:");
+	const char *msg_um_overwrite = _("(c)ontinue");
 	const char *msg_um_merge = _("(m)erge");
-	const char *msg_um_keep = _("(k)eep and cancel");
-	const char *msg_um_choice = _("[omk]");
-	int ret = 1;
+	const char *msg_um_keep = _("c(a)ncel");
+	const char *msg_um_choice = _("[cma]");
+	int ret = IO_SAVE_CANCEL;
 
 	asprintf(&msg_um_asktype, "%s %s, %s, %s", msg_um_prefix,
 		 msg_um_overwrite, msg_um_merge, msg_um_keep);
 
 	switch (status_ask_choice(msg_um_asktype, msg_um_choice, 3)) {
 	case 1:
-		ret = 0;
+		ret = IO_SAVE_CTINUE;
 		break;
 	case 2:
 		io_merge_data();
+		io_load_data(NULL, FORCE);
+		ret = IO_SAVE_RELOAD;
 		break;
 	case 3:
 		/* FALLTHROUGH */
 	default:
-		wins_update(FLAG_STA);
+		ret = IO_SAVE_CANCEL;
 	}
 
 	mem_free(msg_um_asktype);
 	return ret;
 }
 
-static int io_check_data_files_modified()
+/*
+ * Return codes for new_data() and io_load_data().
+ * Note that they are file internal.
+ */
+#define NONEW		0
+#define APTS		(1 << 0)
+#define TODO		(1 << 1)
+#define APTS_TODO	APTS | TODO
+#define NOKNOW		-1
+static int new_data()
 {
 	char sha1_new[SHA1_DIGESTLEN * 2 + 1];
-	int ret = 1;
-
-	io_mutex_lock();
+	int ret = NONEW;
 
 	if (io_compute_hash(path_apts, sha1_new)) {
-		if (strncmp(sha1_new, apts_sha1, SHA1_DIGESTLEN * 2) != 0)
-			goto cleanup;
+		if (strncmp(sha1_new, apts_sha1, SHA1_DIGESTLEN * 2) != 0) {
+			ret |= APTS;
+		}
+	} else {
+		ret = NOKNOW;
+		goto exit;
 	}
 
 	if (io_compute_hash(path_todo, sha1_new)) {
-		if (strncmp(sha1_new, todo_sha1, SHA1_DIGESTLEN * 2) != 0)
-			goto cleanup;
+		if (strncmp(sha1_new, todo_sha1, SHA1_DIGESTLEN * 2) != 0) {
+			ret |= TODO;
+		}
+	} else {
+		ret = NOKNOW;
+		goto exit;
 	}
+   exit:
+	return ret;
+}
 
-	ret = 0;
+/*
+ * Save the calendar data.
+ * The return value tells how a possible save conflict should be/was resolved:
+ * IO_SAVE_CTINUE: continue save operation and overwrite the data files
+ * IO_SAVE_RELOAD: cancel save operation (data files changed and reloaded)
+ * IO_SAVE_CANCEL: cancel save operation (user's decision, keep data files, no reload)
+ * IO_SAVE_NOOP: cancel save operation (nothing has changed)
+ * IO_SAVE_ERROR: cannot access data
+ */
+int io_save_cal(enum save_type s_t)
+{
+	int ret, new;
+
+	if (read_only)
+		return IO_SAVE_CANCEL;
+
+	io_mutex_lock();
+	if ((new = new_data()) == NOKNOW) {
+		ret = IO_SAVE_ERROR;
+		goto cleanup;
+	}
+	if (new) { /* New data */
+		if (s_t == periodic) {
+			ret = IO_SAVE_CANCEL;
+			goto cleanup;
+		}
+		/* Interactively decide what to do. */
+		if ((ret = resolve_save_conflict()))
+			goto cleanup;
+	} else /* No new data */
+		if (!io_get_modified()) {
+			ret = IO_SAVE_NOOP;
+			goto cleanup;
+		}
+
+	ret = IO_SAVE_CTINUE;
+	run_hook("pre-save");
+	if (io_save_todo(path_todo) &&
+	    io_save_apts(path_apts)) {
+		io_compute_hash(path_apts, apts_sha1);
+		io_compute_hash(path_todo, todo_sha1);
+		io_unset_modified();
+	} else
+		ret = IO_SAVE_ERROR;
+	run_hook("post-save");
 
 cleanup:
 	io_mutex_unlock();
 	return ret;
 }
 
-/* Save the calendar data */
-void io_save_cal(enum save_display display)
-{
-	const char *access_pb = _("Problems accessing data file ...");
-	const char *save_success =
-	    _("The data files were successfully saved");
-	const char *enter = _("Press [ENTER] to continue");
-	int show_bar;
-
-	if (read_only)
-		return;
-
-	if (io_check_data_files_modified() && resolve_save_conflict()) {
-		if (io_reload_data()) {
-			day_process_storage(ui_calendar_get_slctd_day(), 1);
-			ui_day_load_items();
-			ui_day_sel_reset();
-			notify_check_next_app(1);
-			ui_calendar_monthly_view_cache_set_invalid();
-			wins_update(FLAG_ALL);
-		}
-		return;
-	}
-
-	run_hook("pre-save");
-	io_mutex_lock();
-
-	show_bar = 0;
-	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR
-	    && conf.progress_bar)
-		show_bar = 1;
-
-	if (show_bar)
-		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_CONF);
-	if (!config_save())
-		ERROR_MSG("%s", access_pb);
-
-	if (show_bar)
-		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_TODO);
-	if (!io_save_todo(path_todo))
-		ERROR_MSG("%s", access_pb);
-
-	if (show_bar)
-		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_APTS);
-	if (!io_save_apts(path_apts))
-		ERROR_MSG("%s", access_pb);
-
-	if (show_bar)
-		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_KEYS);
-	if (!io_save_keys())
-		ERROR_MSG("%s", access_pb);
-
-	io_unset_modified();
-
-	/* Print a message telling data were saved */
-	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR &&
-	    show_dialogs()) {
-		status_mesg(save_success, enter);
-		keys_wait_for_any_key(win[KEY].p);
-	}
-
-	io_compute_hash(path_apts, apts_sha1);
-	io_compute_hash(path_todo, todo_sha1);
-
-	io_mutex_unlock();
-	run_hook("post-save");
-}
-
 static void io_load_error(const char *filename, unsigned line,
 			  const char *mesg)
 {
@@ -804,8 +707,7 @@ void io_load_todo(struct item_filter *filter)
 {
 	FILE *data_file;
 	char *newline;
-	int nb_tod = 0;
-	int c, id, completed;
+	int c, id, completed, cond;
 	char buf[BUFSIZ], e_todo[BUFSIZ], note[MAX_NOTESIZ + 1];
 	unsigned line = 0;
 
@@ -857,125 +759,121 @@ void io_load_todo(struct item_filter *filter)
 		io_extract_data(e_todo, buf, sizeof buf);
 
 		/* Filter item. */
+		struct todo *todo = NULL;
 		if (filter) {
-			if (!(filter->type_mask & TYPE_MASK_TODO))
-				continue;
-			if (filter->regex &&
-			    regexec(filter->regex, e_todo, 0, 0, 0))
-				continue;
-			if (filter->priority && id != filter->priority)
-				continue;
-			if (filter->completed && !completed)
+			cond = (
+				!(filter->type_mask & TYPE_MASK_TODO) ||
+				(filter->regex && regexec(filter->regex, e_todo, 0, 0, 0)) ||
+				(filter->priority && id != filter->priority) ||
+				(filter->completed && !completed) ||
+				(filter->uncompleted && completed)
+			);
+			if (filter->hash) {
+				todo = todo_add(e_todo, id, completed, note);
+				char *hash = todo_hash(todo);
+				cond = cond || !hash_matches(filter->hash, hash);
+				mem_free(hash);
+			}
+
+			if ((!filter->invert && cond) || (filter->invert && !cond)) {
+				if (filter->hash)
+					todo_delete(todo);
 				continue;
-			if (filter->uncompleted && completed)
-				continue;
-		}
-
-		struct todo *todo = todo_add(e_todo, id, completed, note);
-
-		/* Filter by hash. */
-		if (filter && filter->hash) {
-			char *hash = todo_hash(todo);
-			if (!hash_matches(filter->hash, hash)) {
-				todo_delete(todo);
-				todo = NULL;
 			}
-			mem_free(hash);
 		}
 
-		if (todo)
-			++nb_tod;
+		if (!todo)
+			todo = todo_add(e_todo, id, completed, note);
 	}
 	file_close(data_file, __FILE_POS__);
 }
 
-/* Load appointments and todo items */
-void io_load_data(struct item_filter *filter)
+/*
+ * Load appointments and todo items.
+ * Unless told otherwise, the function will only load a file that has changed
+ * since last saved or loaded. The new_data() return code is passed on when
+ * force is false. When force is true (FORCE), the return code is of no use.
+ */
+int io_load_data(struct item_filter *filter, int force)
 {
 	run_hook("pre-load");
-	io_mutex_lock();
-	io_load_app(filter);
-	io_load_todo(filter);
-	io_mutex_unlock();
+	if (force)
+		force = APTS_TODO;
+	else
+		force = new_data();
+
+	if (force == NOKNOW)
+		goto exit;
+
+	if (force & APTS) {
+		apoint_llist_free();
+		event_llist_free();
+		recur_apoint_llist_free();
+		recur_event_llist_free();
+		apoint_llist_init();
+		event_llist_init();
+		recur_apoint_llist_init();
+		recur_event_llist_init();
+		io_load_app(filter);
+	}
+	if (force & TODO) {
+		todo_free_list();
+		todo_init_list();
+		io_load_todo(filter);
+	}
+
+	io_unset_modified();
+   exit:
 	run_hook("post-load");
+	return force;
 }
 
+/*
+ * The return codes reflect the user choice in case of unsaved in-memory changes.
+ */
 int io_reload_data(void)
 {
 	char *msg_um_asktype = NULL;
-	const char *reload_success =
-		_("The data files were reloaded successfully");
-	const char *enter = _("Press [ENTER] to continue");
-	int ret = 0;
+	int load = NOFORCE;
+	int ret = IO_RELOAD_LOAD;
 
+	io_mutex_lock();
 	if (io_get_modified()) {
 		const char *msg_um_prefix =
-				_("There are unsaved modifications:");
-		const char *msg_um_discard = _("(d)iscard");
+				_("Screen data have changed and will be lost:");
+		const char *msg_um_discard = _("(c)ontinue");
 		const char *msg_um_merge = _("(m)erge");
-		const char *msg_um_keep = _("(k)eep and cancel");
-		const char *msg_um_choice = _("[dmk]");
+		const char *msg_um_keep = _("c(a)ncel");
+		const char *msg_um_choice = _("[cma]");
 
 		asprintf(&msg_um_asktype, "%s %s, %s, %s", msg_um_prefix,
 			 msg_um_discard, msg_um_merge, msg_um_keep);
 
 		switch (status_ask_choice(msg_um_asktype, msg_um_choice, 3)) {
 		case 1:
+			load = FORCE;
+			ret = IO_RELOAD_CTINUE;
 			break;
 		case 2:
 			io_merge_data();
+			load = FORCE;
+			ret = IO_RELOAD_MERGE;
 			break;
 		case 3:
+			ret = IO_RELOAD_CANCEL;
 			/* FALLTHROUGH */
 		default:
-			wins_update(FLAG_STA);
 			goto cleanup;
 		}
 	}
 
-	if (!io_check_data_files_modified())
-		goto cleanup;
-
-	if (notify_bar())
-		notify_stop_main_thread();
-
-	/* Reinitialize data structures. */
-	apoint_llist_free();
-	event_llist_free();
-	recur_apoint_llist_free();
-	recur_event_llist_free();
-	todo_free_list();
-
-	apoint_llist_init();
-	event_llist_init();
-	recur_apoint_llist_init();
-	recur_event_llist_init();
-	todo_init_list();
-
-	/*
-	 * Temporarily reinitialize the todo list box without any items to make
-	 * sure wins_unprepare_external() does not fail when it is called after
-	 * executing the pre-load hook.
-	 */
-	ui_todo_load_items();
-	ui_todo_sel_reset();
-
-	io_load_data(NULL);
-	io_unset_modified();
-	ui_todo_load_items();
-	ui_todo_sel_reset();
-
-	if (show_dialogs()) {
-		status_mesg(reload_success, enter);
-		keys_wait_for_any_key(win[KEY].p);
-	}
-
-	if (notify_bar())
-		notify_start_main_thread();
-
-	ret = 1;
-
+	load = io_load_data(NULL, load);
+	if (load == NONEW)
+		ret = IO_RELOAD_NOOP;
+	else if (load == NOKNOW)
+		ret = IO_RELOAD_ERROR;
 cleanup:
+	io_mutex_unlock();
 	mem_free(msg_um_asktype);
 	return ret;
 }
@@ -1092,6 +990,7 @@ void io_load_keys(const char *pager)
 			while (*p == ' ')
 				p++;
 			(void)strncpy(tmpbuf, p, BUFSIZ);
+			tmpbuf[BUFSIZ - 1] = '\0';
 			if (sscanf(tmpbuf, "%s", key_ch) == AWAITED) {
 				int ch;
 
@@ -1137,8 +1036,7 @@ void io_load_keys(const char *pager)
 	file_close(log->fd, __FILE_POS__);
 	if (skipped > 0) {
 		const char *view_log =
-		    _("There were some errors when loading keys file, see log file?");
-
+			_("There were some errors when loading keys file.");
 		io_log_display(log, view_log, pager);
 	}
 	io_log_free(log);
@@ -1216,23 +1114,23 @@ int io_check_file(const char *file)
  * Checks if data files exist. If not, create them.
  * The following structure has to be created:
  *
- *	$HOME/.calcurse/
- *                 |
- *                 +--- notes/
- *                 |___ conf
- *                 |___ keys
- *                 |___ apts
- *                 |___ todo
+ *   <datadir>   <configdir> (default for both: $HOME/.calcurse/)
+ *        |             |
+ *        |__ apts      |___ conf
+ *        |__ todo      |___ keys
+ *        |__ notes/    |___ hooks/
  */
 int io_check_data_files(void)
 {
 	int missing = 0;
 
-	missing += io_check_dir(path_dir) ? 0 : 1;
+	missing += io_check_dir(path_ddir) ? 0 : 1;
 	missing += io_check_dir(path_notes) ? 0 : 1;
 	missing += io_check_file(path_todo) ? 0 : 1;
 	missing += io_check_file(path_apts) ? 0 : 1;
+	missing += io_check_dir(path_cdir) ? 0 : 1;
 	missing += io_check_file(path_conf) ? 0 : 1;
+	missing += io_check_dir(path_hooks) ? 0 : 1;
 
 	if (!io_check_file(path_keys)) {
 		missing++;
@@ -1288,6 +1186,7 @@ void io_export_data(enum export_type type, int export_uid)
 		pcal_export_data(stream);
 
 	if (show_dialogs() && ui_mode == UI_CURSES) {
+		fclose(stream);
 		status_mesg(success, enter);
 		keys_wait_for_any_key(win[KEY].p);
 	}
@@ -1414,9 +1313,7 @@ void io_import_data(enum import_type type, const char *stream_name,
 	 */
 	file_close(log->fd, __FILE_POS__);
 	if (stats.skipped > 0) {
-		const char *view_log =
-		    _("Some items could not be imported, see log file?");
-
+		const char *view_log = _("Some items could not be imported.");
 		io_log_display(log, view_log, conf.pager);
 	}
 
@@ -1465,25 +1362,22 @@ void io_log_print(struct io_file *log, int line, const char *msg)
 		fprintf(log->fd, "line %d: %s\n", line, msg);
 }
 
-void io_log_display(struct io_file *log, const char *msg,
-		    const char *pager)
+void io_log_display(struct io_file *log, const char *msg, const char *pager)
 {
+	char *msgq;
+
 	RETURN_IF(log == NULL, _("No log file to display!"));
 	if (ui_mode == UI_CMDLINE) {
-		printf("\n%s [y/n] ", msg);
-		if (fgetc(stdin) != 'y')
-			return;
-
-		const char *arg[] = { pager, log->name, NULL };
-		int pid;
-
-		if ((pid = fork_exec(NULL, NULL, pager, arg)))
-			child_wait(NULL, NULL, pid);
+		fprintf(stderr, "\n%s\n", msg);
+		fprintf(stderr, _("See %s for details."), log->name);
+		fputc('\n', stderr);
 	} else {
-		if (status_ask_bool(msg) == 1) {
+		asprintf(&msgq, "%s %s", msg, _("Display log file?"));
+		if (status_ask_bool(msgq) == 1) {
 			const char *arg[] = { pager, log->name, NULL };
 			wins_launch_external(arg);
 		}
+		mem_free(msgq);
 		wins_erase_status_bar();
 	}
 }
@@ -1498,17 +1392,20 @@ void io_log_free(struct io_file *log)
 	mem_free(log);
 }
 
-static pthread_t io_t_psave;
-
 /* Thread used to periodically save data. */
 static void *io_psave_thread(void *arg)
 {
 	int delay = conf.periodic_save;
 	EXIT_IF(delay < 0, _("Invalid delay"));
+	char *mesg = _("Periodic save: data files have changed. Save cancelled.");
 
+	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 	for (;;) {
 		sleep(delay * MININSEC);
-		io_save_cal(IO_SAVE_DISPLAY_NONE);
+		pthread_mutex_lock(&io_periodic_save_mutex);
+		if (io_save_cal(periodic) == IO_SAVE_CANCEL)
+			que_ins(mesg, now(), 2);
+		pthread_mutex_unlock(&io_periodic_save_mutex);
 	}
 }
 
@@ -1521,14 +1418,16 @@ void io_start_psave_thread(void)
 /* Stop periodic data saves. */
 void io_stop_psave_thread(void)
 {
-	if (!io_t_psave)
+	/* Is the thread running? */
+	if (pthread_equal(io_t_psave, pthread_self()))
 		return;
 
 	/* Lock the mutex to avoid cancelling the thread during saving. */
-	io_mutex_lock();
+	pthread_mutex_lock(&io_periodic_save_mutex);
 	pthread_cancel(io_t_psave);
 	pthread_join(io_t_psave, NULL);
-	io_mutex_unlock();
+	pthread_mutex_unlock(&io_periodic_save_mutex);
+	io_t_psave = pthread_self();
 }
 
 /*
