@@ -80,7 +80,7 @@ static struct keydef_s keydef[NBKEYS] = {
 	{ "generic-prev-day", "T ^H", gettext_noop("-1 Day") },
 	{ "generic-next-day", "t ^L", gettext_noop("+1 Day") },
 	{ "generic-prev-week", "W ^K", gettext_noop("-1 Week") },
-	{ "generic-next-week", "w ^J", gettext_noop("+1 Week") },
+	{ "generic-next-week", "w", gettext_noop("+1 Week") },
 	{ "generic-prev-month", "M", gettext_noop("-1 Month") },
 	{ "generic-next-month", "m", gettext_noop("+1 Month") },
 	{ "generic-prev-year", "Y", gettext_noop("-1 Year") },
@@ -99,7 +99,7 @@ static struct keydef_s keydef[NBKEYS] = {
 	{ "add-item", "a A", gettext_noop("Add Item") },
 	{ "del-item", "d D", gettext_noop("Del Item") },
 	{ "edit-item", "e E", gettext_noop("Edit Itm") },
-	{ "view-item", "v V", gettext_noop("View") },
+	{ "view-item", "v V RET", gettext_noop("View") },
 	{ "pipe-item", "|", gettext_noop("Pipe") },
 	{ "flag-item", "!", gettext_noop("Flag Itm") },
 	{ "repeat", "r", gettext_noop("Repeat") },
@@ -266,7 +266,7 @@ int keys_wgetch(WINDOW *win)
 	buf[0] = ch;
 	for (i = 1; i < UTF8_LENGTH(buf[0]); i++)
 		buf[i] = wgetch(win);
-	return utf8_ord(buf) + KEY_MAX;
+	return utf8_decode(buf) + KEY_MAX;
 }
 
 void keys_wait_for_any_key(WINDOW *win)
@@ -395,9 +395,6 @@ int keys_str2int(const char *key)
 	else if (strcmp(key, "KEY_END") == 0)
 		return KEY_END;
 
-	/* UTF-8 multibyte keys. */
-	if (starts_with(key, "U+"))
-		return strtol(&key[strlen("U+")], NULL, 16) + KEY_MAX;
 
 	/* Lookup in the keynames table. */
 	for (int i = 1; i < 128; i++)
@@ -407,8 +404,8 @@ int keys_str2int(const char *key)
 		if (strcmp(key, keynames[i]) == 0)
 			return i;
 
-
-	return -1;
+	/* UTF-8 multibyte keys. */
+	return utf8_decode(key) + KEY_MAX;
 }
 
 char *keys_int2str(int key)
@@ -421,7 +418,7 @@ char *keys_int2str(int key)
 		else
 			return mem_strdup(keynames[key]);
 	} else {
-		asprintf(&res, "U+%04X", key - KEY_MAX);
+		asprintf(&res, "%s", utf8_encode(key - KEY_MAX));
 		return res;
 	}
 }
@@ -452,45 +449,32 @@ char *keys_action_allkeys(enum key action)
 {
 	llist_item_t *i;
 	static char keystr[BUFSIZ];
-	const char *CHAR_SPACE = " ";
+	int keystrlen = 0;
+	int entrylen;
 
 	if (!LLIST_FIRST(&keys[action]))
 		return NULL;
 
 	keystr[0] = '\0';
 	LLIST_FOREACH(&keys[action], i) {
-		const int MAXLEN = sizeof(keystr) - 1 - strlen(keystr);
-		strncat(keystr, LLIST_GET_DATA(i), MAXLEN - 1);
-		strncat(keystr, CHAR_SPACE, 1);
+		entrylen = strlen(LLIST_GET_DATA(i)) + 1;
+		if (keystrlen + entrylen >= BUFSIZ)
+			break;
+		memcpy(keystr + keystrlen, LLIST_GET_DATA(i), entrylen - 1);
+		keystr[keystrlen + entrylen - 1] = ' ';
+		keystrlen += entrylen;
 	}
 
+	keystr[keystrlen] = '\0';
 	return keystr;
 }
 
 /* Need this to display keys properly inside status bar. */
-static char *keys_format_label(char *key, int keylen)
+static unsigned keys_format_label(char **s, char *key, int width)
 {
-	static char fmtkey[BUFSIZ];
-	const int len = strlen(key);
-	const char dot = '.';
-	int i;
-
-	if (keylen > BUFSIZ)
-		return NULL;
-
-	memset(fmtkey, 0, sizeof(fmtkey));
-	if (len == 0) {
-		strncpy(fmtkey, "?", sizeof(fmtkey));
-	} else if (len <= keylen) {
-		for (i = 0; i < keylen - len; i++)
-			fmtkey[i] = ' ';
-		strncat(fmtkey, key, keylen);
-	} else {
-		for (i = 0; i < keylen - 1; i++)
-			fmtkey[i] = key[i];
-		fmtkey[keylen - 1] = dot;
-	}
-	return fmtkey;
+	*s = mem_strdup(key);
+	utf8_chop(*s, width);
+	return utf8_strwidth(*s);
 }
 
 void
@@ -515,7 +499,8 @@ keys_display_bindings_bar(WINDOW * win, int *bindings, int count,
 		const int label_pos_x = key_pos_x + KEYS_KEYLEN + 1;
 		const int label_pos_y = key_pos_y;
 
-		char key[KEYS_KEYLEN + 1], *fmtkey;
+		char key[UTF8_MAXLEN + 1], *fmtkey;
+		unsigned dpywidth, shift_x;
 
 		int binding_key;
 
@@ -527,9 +512,8 @@ keys_display_bindings_bar(WINDOW * win, int *bindings, int count,
 		const char *label;
 
 		if (binding_key < NBKEYS) {
-			strncpy(key, keys_action_firstkey(binding_key),
-				KEYS_KEYLEN);
-			key[KEYS_KEYLEN] = '\0';
+			strncpy(key, keys_action_firstkey(binding_key), UTF8_MAXLEN);
+			key[UTF8_MAXLEN] = '\0';
 			label = gettext(keydef[binding_key].sb_label);
 		} else {
 			switch (binding_key) {
@@ -565,8 +549,10 @@ keys_display_bindings_bar(WINDOW * win, int *bindings, int count,
 		}
 
 		custom_apply_attr(win, ATTR_HIGHEST);
-		fmtkey = keys_format_label(key, KEYS_KEYLEN);
-		mvwaddstr(win, key_pos_y, key_pos_x, fmtkey);
+		dpywidth = keys_format_label(&fmtkey, key, KEYS_KEYLEN);
+		shift_x = KEYS_KEYLEN - dpywidth;
+		mvwaddstr(win, key_pos_y, key_pos_x + shift_x, fmtkey);
+		mem_free(fmtkey);
 		custom_remove_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, label_pos_y, label_pos_x, label);
 	}
@@ -715,6 +701,7 @@ void keys_fill_missing(void)
 			char *p, tmpbuf[BUFSIZ];
 
 			strncpy(tmpbuf, keydef[i].binding, BUFSIZ);
+			tmpbuf[BUFSIZ - 1] = '\0';
 			p = tmpbuf;
 			for (;;) {
 				char key_ch[BUFSIZ];
@@ -731,7 +718,7 @@ void keys_fill_missing(void)
 							  "\"%s\" was already assigned!"),
 							 keydef[i].label,
 							 key_ch);
-					p += strlen(key_ch) + 1;
+					p += strlen(key_ch);
 				} else {
 					break;
 				}
