@@ -54,7 +54,6 @@ struct notify_vars {
 static struct notify_vars notify;
 static struct notify_app notify_app;
 static pthread_attr_t detached_thread_attr;
-static pthread_t notify_t_main;
 
 /*
  * Return the number of seconds before next appointment
@@ -99,7 +98,7 @@ unsigned notify_needs_reminder(void)
  * Note: the mutex associated with this structure must be locked by the
  * caller!
  */
-void notify_update_app(long start, char state, char *msg)
+void notify_update_app(time_t start, char state, char *msg)
 {
 	notify_free_app();
 	notify_app.got_app = 1;
@@ -130,9 +129,12 @@ void notify_init_vars(void)
 	pthread_mutex_init(&nbar.mutex, NULL);
 	nbar.show = 1;
 	nbar.cntdwn = 300;
-	strncpy(nbar.datefmt, date_format, strlen(date_format) + 1);
-	strncpy(nbar.timefmt, time_format, strlen(time_format) + 1);
-	strncpy(nbar.cmd, cmd, strlen(cmd) + 1);
+	strncpy(nbar.datefmt, date_format, BUFSIZ);
+	nbar.datefmt[BUFSIZ - 1] = '\0';
+	strncpy(nbar.timefmt, time_format, BUFSIZ);
+	nbar.timefmt[BUFSIZ - 1] = '\0';
+	strncpy(nbar.cmd, cmd, BUFSIZ);
+	nbar.cmd[BUFSIZ - 1] = '\0';
 
 	if ((nbar.shell = getenv("SHELL")) == NULL)
 		nbar.shell = "/bin/sh";
@@ -190,10 +192,13 @@ void notify_free_app(void)
 /* Stop the notify-bar main thread. */
 void notify_stop_main_thread(void)
 {
-	if (notify_t_main) {
-		pthread_cancel(notify_t_main);
-		pthread_join(notify_t_main, NULL);
-	}
+	/* Is the thread running? */
+	if (pthread_equal(notify_t_main, pthread_self()))
+		return;
+
+	pthread_cancel(notify_t_main);
+	pthread_join(notify_t_main, NULL);
+	notify_t_main = pthread_self();
 }
 
 /*
@@ -274,13 +279,15 @@ void notify_update_bar(void)
 		if (time_left > 0) {
 			int hours_left, minutes_left;
 
-			hours_left = (time_left / HOURINSEC);
-			minutes_left =
-			    (time_left -
-			     hours_left * HOURINSEC) / MININSEC;
-			pthread_mutex_lock(&nbar.mutex);
+			/* In minutes rounded up. */
+			minutes_left = time_left / MININSEC +
+				       (time_left % MININSEC ?  1 : 0);
 
-			blinking = time_left < nbar.cntdwn && notify_trigger();
+			hours_left = minutes_left / HOURINMIN;
+			minutes_left = minutes_left % HOURINMIN;
+
+			pthread_mutex_lock(&nbar.mutex);
+			blinking = time_left <= nbar.cntdwn && notify_trigger();
 
 			WINS_NBAR_LOCK;
 			if (blinking)
@@ -464,7 +471,7 @@ void notify_check_next_app(int force)
 }
 
 /* Check if the newly created appointment is to be notified. */
-void notify_check_added(char *mesg, long start, char state)
+void notify_check_added(char *mesg, time_t start, char state)
 {
 	time_t current_time;
 	int update_notify = 0;
@@ -518,7 +525,7 @@ void notify_check_repeated(struct recur_apoint *i)
 	notify_update_bar();
 }
 
-int notify_same_item(long time)
+int notify_same_item(time_t time)
 {
 	int same = 0;
 
@@ -549,7 +556,7 @@ int notify_same_recur_item(struct recur_apoint *i)
 /* Launch the notify-bar main thread. */
 void notify_start_main_thread(void)
 {
-	/* Avoid starting the notification bar thread twice. */
+        /* Avoid starting the notification bar thread twice. */
 	notify_stop_main_thread();
 
 	pthread_create(&notify_t_main, NULL, notify_main_thread, NULL);
@@ -707,26 +714,26 @@ static void config_option_edit(int i)
 	case 1:
 		status_mesg(date_str, "");
 		pthread_mutex_lock(&nbar.mutex);
-		strncpy(buf, nbar.datefmt,
-			strlen(nbar.datefmt) + 1);
+		strncpy(buf, nbar.datefmt, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
 		pthread_mutex_unlock(&nbar.mutex);
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			pthread_mutex_lock(&nbar.mutex);
-			strncpy(nbar.datefmt, buf,
-				strlen(buf) + 1);
+			strncpy(nbar.datefmt, buf, BUFSIZ);
+			nbar.datefmt[BUFSIZ - 1] = '\0';
 			pthread_mutex_unlock(&nbar.mutex);
 		}
 		break;
 	case 2:
 		status_mesg(time_str, "");
 		pthread_mutex_lock(&nbar.mutex);
-		strncpy(buf, nbar.timefmt,
-			strlen(nbar.timefmt) + 1);
+		strncpy(buf, nbar.timefmt, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
 		pthread_mutex_unlock(&nbar.mutex);
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			pthread_mutex_lock(&nbar.mutex);
-			strncpy(nbar.timefmt, buf,
-				strlen(buf) + 1);
+			strncpy(nbar.timefmt, buf, BUFSIZ);
+			nbar.timefmt[BUFSIZ - 1] = '\0';
 			pthread_mutex_unlock(&nbar.mutex);
 		}
 		break;
@@ -746,11 +753,13 @@ static void config_option_edit(int i)
 	case 4:
 		status_mesg(cmd_str, "");
 		pthread_mutex_lock(&nbar.mutex);
-		strncpy(buf, nbar.cmd, strlen(nbar.cmd) + 1);
+		strncpy(buf, nbar.cmd, BUFSIZ);
+		buf[BUFSIZ - 1] = '\0';
 		pthread_mutex_unlock(&nbar.mutex);
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			pthread_mutex_lock(&nbar.mutex);
-			strncpy(nbar.cmd, buf, strlen(buf) + 1);
+			strncpy(nbar.cmd, buf, BUFSIZ);
+			nbar.cmd[BUFSIZ - 1] = '\0';
 			pthread_mutex_unlock(&nbar.mutex);
 		}
 		break;
@@ -786,7 +795,7 @@ void notify_config_bar(void)
 		     config_option_height, print_config_option);
 	listbox_load_items(&lb, 8);
 	listbox_draw_deco(&lb, 0);
-	listbox_display(&lb);
+	listbox_display(&lb, NOHILT);
 	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
 	wins_status_bar();
 	wnoutrefresh(win[STA].p);
@@ -824,7 +833,7 @@ void notify_config_bar(void)
 			clearok(curscr, TRUE);
 		}
 
-		listbox_display(&lb);
+		listbox_display(&lb, NOHILT);
 		wins_status_bar();
 		wnoutrefresh(win[STA].p);
 		wmove(win[STA].p, 0, 0);
