@@ -62,7 +62,7 @@ static int exc_cmp_day(struct excp *a, struct excp *b)
 	return a->st < b->st ? -1 : (a->st == b->st ? 0 : 1);
 }
 
-static void recur_add_exc(llist_t * exc, long day)
+static void recur_add_exc(llist_t * exc, time_t day)
 {
 	struct excp *o = mem_malloc(sizeof(struct excp));
 	o->st = day;
@@ -84,6 +84,60 @@ static void exc_dup(llist_t * in, llist_t * exc)
 	}
 }
 
+/* Return a string containing the exception days. */
+char *recur_exc2str(llist_t *exc)
+{
+	llist_item_t *i;
+	struct excp *p;
+	struct string s;
+	struct tm tm;
+
+	string_init(&s);
+	LLIST_FOREACH(exc, i) {
+		p = LLIST_GET_DATA(i);
+		localtime_r(&p->st, &tm);
+		string_catftime(&s, DATEFMT(conf.input_datefmt), &tm);
+		string_catf(&s, "%c", ' ');
+	}
+	return string_buf(&s);
+}
+
+/*
+ * Update the list of exceptions from a string of days. Any positive number of
+ * spaces are allowed before, between and after the days.
+ */
+int recur_update_exc(llist_t *exc, char *days)
+{
+	int updated = 0;
+	char *d;
+	time_t t = get_today();
+	llist_t nexc;
+	LLIST_INIT(&nexc);
+
+	while (1) {
+		while (*days == ' ')
+			days++;
+		if ((d = strchr(days, ' ')))
+			*d = '\0';
+		else if (!strlen(days))
+			break;
+		if (parse_datetime(days, &t, 0))
+			recur_add_exc(&nexc, t);
+		else
+			goto cleanup;
+		if (d)
+			days = d + 1;
+		else
+			break;
+	}
+	free_exc_list(exc);
+	exc_dup(exc, &nexc);
+	updated = 1;
+cleanup:
+	free_exc_list(&nexc);
+	return updated;
+}
+
 struct recur_event *recur_event_dup(struct recur_event *in)
 {
 	EXIT_IF(!in, _("null pointer"));
@@ -206,9 +260,9 @@ static int recur_event_cmp(struct recur_event *a, struct recur_event *b)
 }
 
 /* Insert a new recursive appointment in the general linked list */
-struct recur_apoint *recur_apoint_new(char *mesg, char *note, long start,
+struct recur_apoint *recur_apoint_new(char *mesg, char *note, time_t start,
 				      long dur, char state, int type,
-				      int freq, long until,
+				      int freq, time_t until,
 				      llist_t * except)
 {
 	struct recur_apoint *rapt =
@@ -238,8 +292,8 @@ struct recur_apoint *recur_apoint_new(char *mesg, char *note, long start,
 }
 
 /* Insert a new recursive event in the general linked list */
-struct recur_event *recur_event_new(char *mesg, char *note, long day,
-				    int id, int type, int freq, long until,
+struct recur_event *recur_event_new(char *mesg, char *note, time_t day,
+				    int id, int type, int freq, time_t until,
 				    llist_t * except)
 {
 	struct recur_event *rev = mem_malloc(sizeof(struct recur_event));
@@ -349,7 +403,8 @@ struct recur_apoint *recur_apoint_scan(FILE * f, struct tm start,
 {
 	char buf[BUFSIZ], *nl;
 	time_t tstart, tend, tuntil;
-	struct recur_apoint *rapt;
+	struct recur_apoint *rapt = NULL;
+	int cond;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_date(end.tm_year, end.tm_mon, end.tm_mday) ||
@@ -393,32 +448,34 @@ struct recur_apoint *recur_apoint_scan(FILE * f, struct tm start,
 
 	/* Filter item. */
 	if (filter) {
-		if (!(filter->type_mask & TYPE_MASK_RECUR_APPT))
-			return NULL;
-		if (filter->regex && regexec(filter->regex, buf, 0, 0, 0))
-			return NULL;
-		if (filter->start_from >= 0 && tstart < filter->start_from)
-			return NULL;
-		if (filter->start_to >= 0 && tstart > filter->start_to)
+		cond = (
+		    !(filter->type_mask & TYPE_MASK_RECUR_APPT) ||
+		    (filter->regex && regexec(filter->regex, buf, 0, 0, 0)) ||
+		    (filter->start_from != -1 && tstart < filter->start_from) ||
+		    (filter->start_to != -1 && tstart > filter->start_to) ||
+		    (filter->end_from != -1 && tend < filter->end_from) ||
+		    (filter->end_to != -1 && tend > filter->end_to)
+		);
+		if (filter->hash) {
+			rapt = recur_apoint_new(buf, note, tstart,
+						tend - tstart, state,
+						recur_char2def(type),
+						freq, tuntil, exc);
+			char *hash = recur_apoint_hash(rapt);
+			cond = cond || !hash_matches(filter->hash, hash);
+			mem_free(hash);
+		}
+
+		if ((!filter->invert && cond) || (filter->invert && !cond)) {
+			if (filter->hash)
+				recur_apoint_erase(rapt);
 			return NULL;
-		if (filter->end_from >= 0 && tend < filter->end_from)
-			return NULL;
-		if (filter->end_to >= 0 && tend > filter->end_to)
-			return NULL;
-	}
-
-	rapt = recur_apoint_new(buf, note, tstart, tend - tstart, state,
-				recur_char2def(type), freq, tuntil, exc);
-
-	/* Filter by hash. */
-	if (filter && filter->hash) {
-		char *hash = recur_apoint_hash(rapt);
-		if (!hash_matches(filter->hash, hash)) {
-			recur_apoint_erase(rapt);
-			rapt = NULL;
 		}
-		mem_free(hash);
 	}
+	if (!rapt)
+		rapt = recur_apoint_new(buf, note, tstart, tend - tstart,
+					state, recur_char2def(type), freq,
+					tuntil, exc);
 
 	return rapt;
 }
@@ -431,7 +488,8 @@ struct recur_event *recur_event_scan(FILE * f, struct tm start, int id,
 {
 	char buf[BUFSIZ], *nl;
 	time_t tstart, tend, tuntil;
-	struct recur_event *rev;
+	struct recur_event *rev = NULL;
+	int cond;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_time(start.tm_hour, start.tm_min) ||
@@ -466,32 +524,33 @@ struct recur_event *recur_event_scan(FILE * f, struct tm start, int id,
 
 	/* Filter item. */
 	if (filter) {
-		if (!(filter->type_mask & TYPE_MASK_RECUR_EVNT))
-			return NULL;
-		if (filter->regex && regexec(filter->regex, buf, 0, 0, 0))
-			return NULL;
-		if (filter->start_from >= 0 && tstart < filter->start_from)
-			return NULL;
-		if (filter->start_to >= 0 && tstart > filter->start_to)
+		cond = (
+		    !(filter->type_mask & TYPE_MASK_RECUR_EVNT) ||
+		    (filter->regex && regexec(filter->regex, buf, 0, 0, 0)) ||
+		    (filter->start_from != -1 && tstart < filter->start_from) ||
+		    (filter->start_to != -1 && tstart > filter->start_to) ||
+		    (filter->end_from != -1 && tend < filter->end_from) ||
+		    (filter->end_to != -1 && tend > filter->end_to)
+		);
+		if (filter->hash) {
+			rev = recur_event_new(buf, note, tstart, id,
+					      recur_char2def(type),
+					      freq, tuntil, exc);
+			char *hash = recur_event_hash(rev);
+			cond = cond || !hash_matches(filter->hash, hash);
+			mem_free(hash);
+		}
+
+		if ((!filter->invert && cond) || (filter->invert && !cond)) {
+			if (filter->hash)
+				recur_event_erase(rev);
 			return NULL;
-		if (filter->end_from >= 0 && tend < filter->end_from)
-			return NULL;
-		if (filter->end_to >= 0 && tend > filter->end_to)
-			return NULL;
-	}
-
-	rev = recur_event_new(buf, note, tstart, id, recur_char2def(type),
-			      freq, tuntil, exc);
-
-	/* Filter by hash. */
-	if (filter && filter->hash) {
-		char *hash = recur_event_hash(rev);
-		if (!hash_matches(filter->hash, hash)) {
-			recur_event_erase(rev);
-			rev = NULL;
 		}
-		mem_free(hash);
 	}
+	if (!rev)
+		rev = recur_event_new(buf, note, tstart, id,
+				      recur_char2def(type),
+				      freq, tuntil, exc);
 
 	return rev;
 }
@@ -681,7 +740,7 @@ static long diff_years(struct tm lt_start, struct tm lt_end)
 	return lt_end.tm_year - lt_start.tm_year;
 }
 
-static int exc_inday(struct excp *exc, long *day_start)
+static int exc_inday(struct excp *exc, time_t *day_start)
 {
 	return (date_cmp_day(exc->st, *day_start) == 0);
 }
@@ -696,9 +755,9 @@ static int exc_inday(struct excp *exc, long *day_start)
  * calculation of recurrent dates after a turn of years.
  */
 unsigned
-recur_item_find_occurrence(long item_start, long item_dur,
+recur_item_find_occurrence(time_t item_start, long item_dur,
 			   llist_t * item_exc, int rpt_type, int rpt_freq,
-			   long rpt_until, long day_start,
+			   time_t rpt_until, time_t day_start,
 			   time_t *occurrence)
 {
 	struct date start_date;
@@ -759,6 +818,18 @@ recur_item_find_occurrence(long item_start, long item_dur,
 	lt_item_day.tm_isdst = lt_day.tm_isdst;
 	t = mktime(&lt_item_day);
 
+	/*
+	 * Impossible dates must be ignored (according to RFC 5545). Changing
+	 * only the year or the month may lead to dates like 29 February in
+	 * non-leap years or 31 November.
+	 */
+	if (rpt_type == RECUR_MONTHLY || rpt_type == RECUR_YEARLY) {
+		localtime_r(&t, &lt_item_day);
+		if (lt_item_day.tm_mday != lt_item.tm_mday)
+			return 0;
+	}
+
+	/* Exception day? */
 	if (LLIST_FIND_FIRST(item_exc, &t, exc_inday))
 		return 0;
 
@@ -784,7 +855,7 @@ recur_item_find_occurrence(long item_start, long item_dur,
 }
 
 unsigned
-recur_apoint_find_occurrence(struct recur_apoint *rapt, long day_start,
+recur_apoint_find_occurrence(struct recur_apoint *rapt, time_t day_start,
 			     time_t *occurrence)
 {
 	return recur_item_find_occurrence(rapt->start, rapt->dur,
@@ -795,7 +866,7 @@ recur_apoint_find_occurrence(struct recur_apoint *rapt, long day_start,
 }
 
 unsigned
-recur_event_find_occurrence(struct recur_event *rev, long day_start,
+recur_event_find_occurrence(struct recur_event *rev, time_t day_start,
 			    time_t *occurrence)
 {
 	return recur_item_find_occurrence(rev->day, DAYINSEC, &rev->exc,
@@ -806,9 +877,9 @@ recur_event_find_occurrence(struct recur_event *rev, long day_start,
 
 /* Check if a recurrent item belongs to the selected day. */
 unsigned
-recur_item_inday(long item_start, long item_dur, llist_t * item_exc,
-		 int rpt_type, int rpt_freq, long rpt_until,
-		 long day_start)
+recur_item_inday(time_t item_start, long item_dur, llist_t * item_exc,
+		 int rpt_type, int rpt_freq, time_t rpt_until,
+		 time_t day_start)
 {
 	/* We do not need the (real) start time of the occurrence here, so just
 	 * ignore the buffer. */
@@ -817,14 +888,14 @@ recur_item_inday(long item_start, long item_dur, llist_t * item_exc,
 					  day_start, NULL);
 }
 
-unsigned recur_apoint_inday(struct recur_apoint *rapt, long *day_start)
+unsigned recur_apoint_inday(struct recur_apoint *rapt, time_t *day_start)
 {
 	return recur_item_inday(rapt->start, rapt->dur, &rapt->exc,
 				rapt->rpt->type, rapt->rpt->freq,
 				rapt->rpt->until, *day_start);
 }
 
-unsigned recur_event_inday(struct recur_event *rev, long *day_start)
+unsigned recur_event_inday(struct recur_event *rev, time_t *day_start)
 {
 	return recur_item_inday(rev->day, DAYINSEC, &rev->exc,
 				rev->rpt->type, rev->rpt->freq,
@@ -832,13 +903,13 @@ unsigned recur_event_inday(struct recur_event *rev, long *day_start)
 }
 
 /* Add an exception to a recurrent event. */
-void recur_event_add_exc(struct recur_event *rev, long date)
+void recur_event_add_exc(struct recur_event *rev, time_t date)
 {
 	recur_add_exc(&rev->exc, date);
 }
 
 /* Add an exception to a recurrent appointment. */
-void recur_apoint_add_exc(struct recur_apoint *rapt, long date)
+void recur_apoint_add_exc(struct recur_apoint *rapt, time_t date)
 {
 	int need_check_notify = 0;
 
@@ -917,36 +988,31 @@ void recur_exc_scan(llist_t * lexc, FILE * data_file)
 	}
 }
 
-static int recur_apoint_starts_before(struct recur_apoint *rapt, long *time)
-{
-	return rapt->start < *time;
-}
-
 /*
- * Look in the appointment list if we have an item which starts before the item
- * stored in the notify_app structure (which is the next item to be notified).
+ * Look in the appointment list if we have an item which starts after start and
+ * before the item stored in the notify_app structure (which is the next item
+ * to be notified). Note, the search may change the notify_app structure.
  */
-struct notify_app *recur_apoint_check_next(struct notify_app *app,
-					   long start, long day)
+void recur_apoint_check_next(struct notify_app *app, time_t start, time_t day)
 {
 	llist_item_t *i;
 	time_t real_recur_start_time;
 
 	LLIST_TS_LOCK(&recur_alist_p);
-	/*
-	 * Iterate over all recurrent items starting before the current "next"
-	 * appointment. We cannot filter by start time because a recurrent item
-	 * can actually start (several days) before the current "next" item and
-	 * still have an occurrence which is the next item to be notified.
-	 */
-	LLIST_TS_FIND_FOREACH(&recur_alist_p, &app->time,
-			      recur_apoint_starts_before, i) {
+	LLIST_TS_FOREACH(&recur_alist_p, i) {
 		struct recur_apoint *rapt = LLIST_TS_GET_DATA(i);
 
-		/*
-		 * Check whether the recurrent appointment contains an
-		 * occurrence which is the next item to be notified.
-		 */
+		/* Tomorrow? */
+		if (recur_apoint_find_occurrence
+		    (rapt, day + DAYINSEC, &real_recur_start_time)
+		    && real_recur_start_time > start
+		    && real_recur_start_time < app->time) {
+			app->time = real_recur_start_time;
+			app->txt = mem_strdup(rapt->mesg);
+			app->state = rapt->state;
+			app->got_app = 1;
+		}
+		/* Today? */
 		if (recur_apoint_find_occurrence
 		    (rapt, day, &real_recur_start_time)
 		    && real_recur_start_time > start
@@ -958,8 +1024,6 @@ struct notify_app *recur_apoint_check_next(struct notify_app *app,
 		}
 	}
 	LLIST_TS_UNLOCK(&recur_alist_p);
-
-	return app;
 }
 
 /* Switch recurrent item notification state. */
@@ -974,7 +1038,7 @@ void recur_apoint_switch_notify(struct recur_apoint *rapt)
 	LLIST_TS_UNLOCK(&recur_alist_p);
 }
 
-void recur_event_paste_item(struct recur_event *rev, long date)
+void recur_event_paste_item(struct recur_event *rev, time_t date)
 {
 	long time_shift;
 	llist_item_t *i;
@@ -993,20 +1057,25 @@ void recur_event_paste_item(struct recur_event *rev, long date)
 	LLIST_ADD_SORTED(&recur_elist, rev, recur_event_cmp);
 }
 
-void recur_apoint_paste_item(struct recur_apoint *rapt, long date)
+void recur_apoint_paste_item(struct recur_apoint *rapt, time_t date)
 {
-	long time_shift;
+	time_t ostart = rapt->start;
+	int days;
 	llist_item_t *i;
+	struct tm t;
 
-	time_shift = (date + get_item_time(rapt->start)) - rapt->start;
-	rapt->start += time_shift;
+	localtime_r((time_t *)&rapt->start, &t);
+	rapt->start = update_time_in_date(date, t.tm_hour, t.tm_min);
+
+	/* The number of days shifted. */
+	days = (rapt->start - ostart) / DAYINSEC;
 
 	if (rapt->rpt->until != 0)
-		rapt->rpt->until += time_shift;
+		rapt->rpt->until = date_sec_change(rapt->rpt->until, 0, days);
 
 	LLIST_FOREACH(&rapt->exc, i) {
 		struct excp *exc = LLIST_GET_DATA(i);
-		exc->st += time_shift;
+		exc->st = date_sec_change(exc->st, 0, days);
 	}
 
 	LLIST_TS_LOCK(&recur_alist_p);
