@@ -36,135 +36,234 @@
 
 #include "calcurse.h"
 
-struct day_item day_cut[38] = { {0, 0, {NULL}} };
+struct day_item day_cut[38] = { {0, 0, 0, {NULL}} };
 
-struct day_item *ui_day_selitem(void)
+/*
+ * Set the selected day in the calendar from the selected item in the APP panel.
+ */
+static void set_slctd_day(void)
+{
+	ui_calendar_set_slctd_day(sec2date(ui_day_get_sel()->order));
+}
+
+/*
+ * Return the selected APP item.
+ * This is a pointer into the day vector and invalid after a day vector rebuild,
+ * but the (order, item) data may be used to refind the object.
+ */
+struct day_item *ui_day_get_sel(void)
 {
 	if (day_item_count(0) <= 0)
-		return NULL;
+		return &empty_day;
 
 	return day_get_item(listbox_get_sel(&lb_apt));
 }
 
-void ui_day_set_selitem_by_aptev_ptr(union aptev_ptr p)
+/*
+ * Set the selected item and day from the saved day_item.
+ */
+void ui_day_find_sel(void)
 {
-	int n = day_get_position_by_aptev_ptr(p);
-	if (n >= 0)
+	int n;
+
+	if ((n = day_sel_index()) != -1)
 		listbox_set_sel(&lb_apt, n);
+	set_slctd_day();
 }
 
-void ui_day_set_selitem(struct day_item *day)
+/*
+ * Return the date (midnight) of the selected item in the APP panel.
+ */
+time_t ui_day_sel_date(void)
 {
-	ui_day_set_selitem_by_aptev_ptr(day->item);
+	return update_time_in_date(ui_day_get_sel()->order, 0, 0);
 }
 
-/* Request the user to enter a new time. */
-static int day_edit_time(int time)
+/*
+ * If possible, move the selection to the beginning
+ * of previous, current or next day.
+ */
+static void daybegin(int dir)
 {
-	char *input = date_sec2date_str(time, "%H:%M");
-	const char *msg_time =
-	    _("Enter start time ([hh:mm] or [hhmm]) or date:");
+	dir = dir > 0 ? 1 : (dir < 0 ? -1 : 0);
+	int sel = listbox_get_sel(&lb_apt);
+
+	switch (dir) {
+	case -1:
+		while (day_get_item(sel)->type != DAY_HEADING)
+			sel--;
+		if (sel == 0)
+			goto leave;
+		sel--;
+		while (day_get_item(sel)->type != DAY_HEADING)
+			sel--;
+		break;
+	case 0:
+		while (day_get_item(sel)->type != DAY_HEADING)
+			sel--;
+		break;
+	case 1:
+		while (day_get_item(sel)->type != END_SEPARATOR)
+			sel++;
+		if (sel == lb_apt.item_count - 1) {
+			while (day_get_item(sel)->type != DAY_HEADING)
+				sel--;
+			goto leave;
+		} else
+			sel++;
+		break;
+	}
+  leave:
+	listbox_set_sel(&lb_apt, sel);
+	listbox_item_in_view(&lb_apt, sel);
+	set_slctd_day();
+}
+
+/*
+ * Request the user to enter a new start time.
+ * Input: start time and duration in seconds.
+ * Output: return value is new start time.
+ * If move = 1, the new start time is for a move, and duration is passed on
+ * for validation of the new end time.
+ * If move = 0, the new end time is calculated by the caller.
+ */
+static time_t day_edit_time(time_t start, long duration, int move)
+{
+	const char *msg_time = _("Enter start date [%s] and/or time ([hh:mm] or [hhmm]):");
 	const char *enter_str = _("Press [Enter] to continue");
-	const char *fmt_msg =
-	    _("You entered an invalid time, should be [hh:mm] or [hhmm]");
-	long ts = time;
+	const char *fmt_msg = _("Invalid date or time.");
+	char *input, *outstr;
+	time_t ts;
+	int ret;
 
+	asprintf(&outstr, "%s %s", DATEFMT(conf.input_datefmt), "%H:%M");
+	input = date_sec2date_str(start, outstr);
+	mem_free(outstr);
 	for (;;) {
-		status_mesg(msg_time, "");
-		if (updatestring(win[STA].p, &input, 0, 1) != GETSTRING_VALID)
-			return 0;
-		if (parse_datetime(input, &ts))
-			return ts;
+		asprintf(&outstr, msg_time, DATEFMT_DESC(conf.input_datefmt));
+		status_mesg(outstr, "");
+		mem_free(outstr);
+		if (updatestring(win[STA].p, &input, 0, 1) != GETSTRING_VALID) {
+			ret = 0;
+			break;
+		}
+		ts = start;
+		if (parse_datetime(input, &ts, move ? duration : 0)) {
+			ret = ts;
+			break;
+		}
 		status_mesg(fmt_msg, enter_str);
 		keys_wait_for_any_key(win[KEY].p);
 	}
+	mem_free(input);
+	return ret;
 }
 
-/* Request the user to enter a new time or duration. */
-static int day_edit_duration(int start, int dur, unsigned *new_duration)
+/*
+ * Change start time or move an item.
+ * Input/output: start and dur.
+ * If move = 0, end time is fixed, and the new duration is calculated
+ * when the new start time is known.
+ * If move = 1, duration is fixed, but passed on for validation of new end time.
+ */
+static void update_start_time(time_t *start, long *dur, int move)
 {
-	char *timestr = date_sec2date_str(start + dur, "%H:%M");
-	const char *msg_time =
-	    _("Enter end time ([hh:mm], [hhmm]) or duration ([+hh:mm], [+xxxdxxhxxm]):");
-	const char *enter_str = _("Press [Enter] to continue");
-	const char *fmt_msg =
-	    _("You entered an invalid time, should be [hh:mm] or [hhmm]");
-	long end;
+	time_t newtime;
+	const char *msg_wrong_time =
+	    _("Invalid time: start time must come before end time!");
+	const char *msg_enter = _("Press [Enter] to continue");
 
 	for (;;) {
-		int ret;
-
-		status_mesg(msg_time, "");
-		ret = updatestring(win[STA].p, &timestr, 0, 1);
-		if (ret == GETSTRING_ESC) {
-			return 0;
-		} else if (ret == GETSTRING_RET) {
-			*new_duration = 0;
+		newtime = day_edit_time(*start, *dur, move);
+		if (!newtime)
 			break;
-		}
-		if (*timestr == '+') {
-			if (parse_duration(timestr + 1, new_duration)) {
-				*new_duration *= MININSEC;
+		if (move) {
+			*start = newtime;
+			break;
+		} else {
+			if (newtime <= *start + *dur) {
+				*dur -= (newtime - *start);
+				*start = newtime;
 				break;
 			}
 		}
-		end = start + dur;
-		ret = parse_datetime(timestr, &end);
-		/*
-		 * If the user enters a end time which is smaller than
-		 * the start time, assume that the time belongs to the
-		 * next day.
-		 */
-		if (!(ret & PARSE_DATETIME_HAS_DATE) && end < start)
-			end = date_sec_change(end, 0, 1);
-		if (ret) {
-			*new_duration = end - start;
-			break;
-		}
-		status_mesg(fmt_msg, enter_str);
-		keys_wait_for_any_key(win[KEY].p);
+		status_mesg(msg_wrong_time, msg_enter);
+		keys_wgetch(win[KEY].p);
 	}
-
-	mem_free(timestr);
-	return 1;
+	return;
 }
 
 /* Request the user to enter a new end time or duration. */
-static void update_start_time(long *start, long *dur, int update_dur)
+static void update_duration(time_t *start, long *dur)
 {
-	long newtime;
-	int valid_date;
-	const char *msg_wrong_time =
-	    _("Invalid time: start time must be before end time!");
-	const char *msg_enter = _("Press [Enter] to continue");
+	const char *msg_time =
+	    _("Enter end date (and/or time) or duration ('?' for input formats):");
+	const char *msg_help_1 =
+		_("Date: %s, year or month may be omitted.");
+	const char *msg_help_2 =
+		_("Time: hh:mm (hh: or :mm) or hhmm. Duration: +mm, +hh:mm, +??d??h??m.");
+	const char *enter_str = _("Press [Enter] to continue");
+	const char *fmt_msg_1 = _("Invalid time or duration.");
+	const char *fmt_msg_2 = _("Invalid date: end time must come after start time.");
+	time_t end;
+	unsigned newdur;
+	char *timestr, *outstr;
 
-	do {
-		newtime = day_edit_time(*start);
-		if (!newtime)
-			break;
-		if (!update_dur) {
-			*start = newtime;
+	end = *start + *dur;
+	asprintf(&outstr, "%s %s", DATEFMT(conf.input_datefmt), "%H:%M");
+	timestr = date_sec2date_str(end, outstr);
+	mem_free(outstr);
+	for (;;) {
+		int ret, early = 0;
+		status_mesg(msg_time, "");
+		ret = updatestring(win[STA].p, &timestr, 0, 1);
+		if (ret == GETSTRING_ESC) {
+			mem_free(timestr);
 			return;
 		}
-		if (newtime < *start + *dur) {
-			*dur -= (newtime - *start);
-			*start = newtime;
-			valid_date = 1;
-		} else {
-			status_mesg(msg_wrong_time, msg_enter);
-			keys_wait_for_any_key(win[KEY].p);
-			valid_date = 0;
+		if (*(timestr + strlen(timestr) - 1) == '?') {
+			asprintf(&outstr, "%s %s", DATEFMT(conf.input_datefmt), "%H:%M");
+			mem_free(timestr);
+			timestr = date_sec2date_str(end, outstr);
+			asprintf(&outstr, msg_help_1, DATEFMT_DESC(conf.input_datefmt));
+			status_mesg(outstr, msg_help_2);
+			mem_free(outstr);
+			keys_wgetch(win[KEY].p);
+			continue;
 		}
+		if (ret == GETSTRING_RET) {
+			newdur = 0;
+			break;
+		}
+		if (*timestr == '+') {
+			if (parse_duration(timestr + 1, &newdur, *start)) {
+				newdur *= MININSEC;
+				break;
+			}
+		} else {
+			int val = 1;
+			ret = parse_datetime(timestr, &end, 0);
+			/*
+			 * If same day and end time is earlier than start time,
+			 * assume that it belongs to the next day.
+			 */
+			if (ret == PARSE_DATETIME_HAS_TIME && end < *start) {
+				end = date_sec_change(end, 0, 1);
+				/* Still valid? */
+				val = check_sec(&end);
+			}
+			if (ret && val && *start <= end) {
+				newdur = end - *start;
+				break;
+			}
+			/* Valid format, but too early? */
+			early = ret && val && end < *start;
+		}
+		status_mesg(early ? fmt_msg_2 : fmt_msg_1 , enter_str);
+		keys_wgetch(win[KEY].p);
 	}
-	while (valid_date == 0);
-}
-
-static void update_duration(long *start, long *dur)
-{
-	unsigned newdur;
-
-	if (day_edit_duration(*start, *dur, &newdur))
-		*dur = newdur;
+	mem_free(timestr);
+	*dur = newdur;
 }
 
 static void update_desc(char **desc)
@@ -173,11 +272,48 @@ static void update_desc(char **desc)
 	updatestring(win[STA].p, desc, 0, 1);
 }
 
-static void update_rept(struct rpt **rpt, const long start)
+/* Edit the list of exception days for a recurrent item. */
+static int update_exc(llist_t *exc)
 {
-	int newtype, newfreq;
-	time_t newuntil;
-	char *freqstr = NULL, *timstr = NULL, *outstr;
+	int updated = 0;
+
+	if (!exc->head)
+		return updated;
+	char *days;
+	enum getstr ret;
+
+	status_mesg(_("Exception days:"), "");
+	days = recur_exc2str(exc);
+	while (1) {
+		ret = updatestring(win[STA].p, &days, 0, 1);
+		if (ret == GETSTRING_VALID || ret == GETSTRING_RET) {
+			if (recur_update_exc(exc, days)) {
+				updated = 1;
+				break;
+			} else {
+				status_mesg(_("Invalid date format - try again:."), "");
+				mem_free(days);
+				days = recur_exc2str(exc);
+			}
+		} else if (ret == GETSTRING_ESC)
+			break;
+	}
+	mem_free(days);
+
+	return updated;
+}
+
+static void update_rept(struct rpt **rpt, const time_t start, llist_t *exc)
+{
+	/* Pointers to dynamically allocated memory. */
+	char *msg_rpt_current = NULL;
+	char *msg_rpt_asktype = NULL;
+	char *freqstr = NULL;
+	char *timstr = NULL;
+	char *outstr = NULL;
+
+	/* Update repetition type. */
+	int newtype;
 	const char *msg_rpt_prefix = _("Enter the new repetition type:");
 	const char *msg_rpt_daily = _("(d)aily");
 	const char *msg_rpt_weekly = _("(w)eekly");
@@ -186,7 +322,6 @@ static void update_rept(struct rpt **rpt, const long start)
 
 	/* Find the current repetition type. */
 	const char *rpt_current;
-	char *msg_rpt_current;
 	switch (recur_def2char((*rpt)->type)) {
 	case 'D':
 		rpt_current = msg_rpt_daily;
@@ -204,23 +339,11 @@ static void update_rept(struct rpt **rpt, const long start)
 		/* NOTREACHED, but makes the compiler happier. */
 		rpt_current = msg_rpt_daily;
 	}
-
 	asprintf(&msg_rpt_current, _("(currently using %s)"), rpt_current);
-
-	char *msg_rpt_asktype;
-	asprintf(&msg_rpt_asktype, "%s %s, %s, %s, %s ? %s", msg_rpt_prefix,
+	asprintf(&msg_rpt_asktype, "%s %s, %s, %s, %s? %s", msg_rpt_prefix,
 		 msg_rpt_daily, msg_rpt_weekly, msg_rpt_monthly,
 		 msg_rpt_yearly, msg_rpt_current);
-
 	const char *msg_rpt_choice = _("[dwmy]");
-	const char *msg_wrong_freq = _("Invalid frequency.");
-	const char *msg_wrong_time =
-	    _("Invalid time: start time must be before end time!");
-	const char *msg_wrong_date = _("The entered date is not valid.");
-	const char *msg_fmts =
-	    _("Possible formats are [%s] or '0' for an endless repetition.");
-	const char *msg_enter = _("Press [Enter] to continue");
-
 	switch (status_ask_choice(msg_rpt_asktype, msg_rpt_choice, 4)) {
 	case 1:
 		newtype = 'D';
@@ -238,8 +361,12 @@ static void update_rept(struct rpt **rpt, const long start)
 		goto cleanup;
 	}
 
+	/* Update frequency. */
+	int newfreq;
+	const char *msg_wrong_freq = _("Invalid frequency.");
+	const char *msg_enter = _("Press [Enter] to continue");
 	do {
-		status_mesg(_("Enter the new repetition frequency:"), "");
+		status_mesg(_("Enter the repetition frequency:"), "");
 		mem_free(freqstr);
 		asprintf(&freqstr, "%d", (*rpt)->freq);
 		if (updatestring(win[STA].p, &freqstr, 0, 1) !=
@@ -254,67 +381,72 @@ static void update_rept(struct rpt **rpt, const long start)
 	}
 	while (newfreq == 0);
 
-	for (;;) {
-		struct tm lt;
-		time_t t;
-		struct date new_date;
-		int newmonth, newday, newyear;
-		unsigned days;
+	/* Update end date. */
+	time_t newuntil;
+	const char *msg_until_1 =
+		_("Enter end date or duration ('?' for input formats):");
+	const char *msg_help_1 =
+		_("Date: %s (year or month may be omitted). Endless duration: 0.");
+	const char *msg_help_2 =
+		_("Duration in days: +dd. Duration in weeks and days: +??w??d.");
+	const char *msg_wrong_time =
+		_("Invalid date: end date must come after start date (%s).");
+	const char *msg_wrong_date = _("Invalid date.");
 
-		asprintf(&outstr,
-			 _("Enter end date ([%s]), duration ([+xxwxxd]) or '0':"),
-			 DATEFMT_DESC(conf.input_datefmt));
-		status_mesg(outstr, "");
-		mem_free(outstr);
-		timstr =
-		    date_sec2date_str((*rpt)->until,
-				      DATEFMT(conf.input_datefmt));
-		if (updatestring(win[STA].p, &timstr, 0, 1) !=
-		    GETSTRING_VALID) {
+	for (;;) {
+		mem_free(timstr);
+		if ((*rpt)->until)
+			timstr = date_sec2date_str((*rpt)->until, DATEFMT(conf.input_datefmt));
+		else
+			timstr = mem_strdup("");
+		status_mesg(msg_until_1, "");
+		if (updatestring(win[STA].p, &timstr, 0, 1) == GETSTRING_ESC)
 			goto cleanup;
-		}
-		if (strcmp(timstr, "0") == 0) {
+		if (strcmp(timstr, "") == 0 || strcmp(timstr, "0") == 0) {
 			newuntil = 0;
 			break;
 		}
+		if (*(timstr + strlen(timstr) - 1) == '?') {
+			mem_free(outstr);
+			asprintf(&outstr, msg_help_1, DATEFMT_DESC(conf.input_datefmt));
+			status_mesg(outstr, msg_help_2);
+			keys_wgetch(win[KEY].p);
+			continue;
+		}
 		if (*timstr == '+') {
-			if (!parse_date_duration(timstr + 1, &days)) {
-				asprintf(&outstr, msg_fmts,
-					 DATEFMT_DESC(conf.input_datefmt));
-				status_mesg(msg_wrong_date, outstr);
-				mem_free(outstr);
-				keys_wait_for_any_key(win[KEY].p);
+			unsigned days;
+			if (!parse_date_duration(timstr + 1, &days, start)) {
+				status_mesg(msg_wrong_date, msg_enter);
+				keys_wgetch(win[KEY].p);
 				continue;
 			}
-			t = start;
-			localtime_r(&t, &lt);
-			date_change(&lt, 0, days);
-			new_date.dd = lt.tm_mday;
-			new_date.mm = lt.tm_mon + 1;
-			new_date.yyyy = lt.tm_year + 1900;
+			newuntil = date_sec_change(start, 0, days);
 		} else {
-			if (!parse_date_interactive(timstr, &newyear,
-						    &newmonth, &newday)) {
-				asprintf(&outstr, msg_fmts,
-					 DATEFMT_DESC(conf.input_datefmt));
-				status_mesg(msg_wrong_date, outstr);
-				mem_free(outstr);
-				keys_wait_for_any_key(win[KEY].p);
+			int year, month, day;
+			if (!parse_date(timstr, conf.input_datefmt, &year,
+			    &month, &day, ui_calendar_get_slctd_day())) {
+				status_mesg(msg_wrong_date, msg_enter);
+				keys_wgetch(win[KEY].p);
 				continue;
 			}
-			t = start;
-			localtime_r(&t, &lt);
-			new_date.dd = newday;
-			new_date.mm = newmonth;
-			new_date.yyyy = newyear;
+			struct date d = { day, month, year };
+			newuntil = date2sec(d, 0, 0);
 		}
-		newuntil = date2sec(new_date, lt.tm_hour, lt.tm_min);
 		if (newuntil >= start)
 			break;
-		status_mesg(msg_wrong_time, msg_enter);
-		keys_wait_for_any_key(win[KEY].p);
+
+		mem_free(timstr);
+		mem_free(outstr);
+		timstr = date_sec2date_str(start, DATEFMT(conf.input_datefmt));
+		asprintf(&outstr, msg_wrong_time, timstr);
+		status_mesg(outstr, msg_enter);
+		keys_wgetch(win[KEY].p);
 	}
 
+	/* Update exception list. */
+	if (!update_exc(exc))
+		goto cleanup;
+
 	(*rpt)->type = recur_char2def(newtype);
 	(*rpt)->freq = newfreq;
 	(*rpt)->until = newuntil;
@@ -324,6 +456,7 @@ cleanup:
 	mem_free(msg_rpt_asktype);
 	mem_free(freqstr);
 	mem_free(timstr);
+	mem_free(outstr);
 }
 
 /* Edit an already existing item. */
@@ -338,14 +471,14 @@ void ui_day_item_edit(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = ui_day_selitem();
+	struct day_item *p = ui_day_get_sel();
 
 	switch (p->type) {
 	case RECUR_EVNT:
 		re = p->item.rev;
 		const char *choice_recur_evnt[2] = {
 			_("Description"),
-			_("Repetition"),
+			_("Repetition")
 		};
 		switch (status_ask_simplechoice
 			(_("Edit: "), choice_recur_evnt, 2)) {
@@ -354,7 +487,7 @@ void ui_day_item_edit(void)
 			io_set_modified();
 			break;
 		case 2:
-			update_rept(&re->rpt, re->day);
+			update_rept(&re->rpt, re->day, &re->exc);
 			io_set_modified();
 			break;
 		default:
@@ -379,7 +512,7 @@ void ui_day_item_edit(void)
 			(_("Edit: "), choice_recur_appt, 5)) {
 		case 1:
 			need_check_notify = 1;
-			update_start_time(&ra->start, &ra->dur, 1);
+			update_start_time(&ra->start, &ra->dur, ra->dur == 0);
 			io_set_modified();
 			break;
 		case 2:
@@ -395,12 +528,12 @@ void ui_day_item_edit(void)
 			break;
 		case 4:
 			need_check_notify = 1;
-			update_rept(&ra->rpt, ra->start);
+			update_rept(&ra->rpt, ra->start, &ra->exc);
 			io_set_modified();
 			break;
 		case 5:
 			need_check_notify = 1;
-			update_start_time(&ra->start, &ra->dur, 0);
+			update_start_time(&ra->start, &ra->dur, 1);
 			io_set_modified();
 			break;
 		default:
@@ -419,7 +552,7 @@ void ui_day_item_edit(void)
 			(_("Edit: "), choice_appt, 4)) {
 		case 1:
 			need_check_notify = 1;
-			update_start_time(&a->start, &a->dur, 1);
+			update_start_time(&a->start, &a->dur, a->dur == 0);
 			io_set_modified();
 			break;
 		case 2:
@@ -435,7 +568,7 @@ void ui_day_item_edit(void)
 			break;
 		case 4:
 			need_check_notify = 1;
-			update_start_time(&a->start, &a->dur, 0);
+			update_start_time(&a->start, &a->dur, 1);
 			io_set_modified();
 			break;
 		default:
@@ -464,7 +597,7 @@ void ui_day_item_pipe(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = ui_day_selitem();
+	struct day_item *p = ui_day_get_sel();
 
 	status_mesg(_("Pipe item to external command:"), "");
 	if (getstring(win[STA].p, cmd, BUFSIZ, 0, 1) != GETSTRING_VALID)
@@ -504,24 +637,25 @@ void ui_day_item_pipe(void)
  */
 void ui_day_item_add(void)
 {
-#define LTIME 16
-#define LDUR 12
+#define LTIME 17
 	const char *mesg_1 =
 	    _("Enter start time ([hh:mm] or [hhmm]), leave blank for an all-day event:");
 	const char *mesg_2 =
-	    _("Enter end time ([hh:mm], [hhmm]) or duration ([+hh:mm], [+xxxdxxhxxm]):");
+	    _("Enter end time as date (and/or time) or duration ('?' for input formats):");
 	const char *mesg_3 = _("Enter description:");
-	const char *format_message_1 =
-	    _("You entered an invalid start time, should be [hh:mm] or [hhmm]");
-	const char *format_message_2 =
-	    _("Invalid end time/duration, should be [hh:mm], [hhmm], [+hh:mm], [+xxxdxxhxxm] or [+mm]");
+	const char *mesg_help_1 =
+		_("Date: %s (and/or time), year or month may be omitted.");
+	const char *mesg_help_2 =
+		_("Time: hh:mm (hh: or :mm) or hhmm. Duration: +mm, +hh:mm, +??d??h??m.");
+	const char *format_message_1 = _("Invalid start time.");
+	const char *format_message_2 = _("Invalid time or duration.");
+	const char *format_message_3 = _("Invalid date: end time must come after start time.");
 	const char *enter_str = _("Press [Enter] to continue");
-	char item_time[LDUR] = "";
+	char item_time[LTIME] = "";
 	char item_mesg[BUFSIZ] = "";
-	time_t start = date2sec(*ui_calendar_get_slctd_day(), 0, 0), end;
+	time_t start = ui_day_sel_date(), end, saved = start;
 	unsigned dur;
 	int is_appointment = 1;
-	int ret;
 	union aptev_ptr item;
 
 	/* Get the starting time */
@@ -534,11 +668,12 @@ void ui_day_item_add(void)
 			is_appointment = 0;
 			break;
 		}
-		ret = parse_datetime(item_time, &start);
-		if (!(ret & PARSE_DATETIME_HAS_TIME))
-			is_appointment = 0;
-		if (ret)
+		start = saved;
+		/* Only time, no date, allowed. */
+		if (parse_datetime(item_time, &start, 0) ==
+		    PARSE_DATETIME_HAS_TIME) {
 			break;
+		}
 		status_mesg(format_message_1, enter_str);
 		keys_wait_for_any_key(win[KEY].p);
 	}
@@ -551,41 +686,58 @@ void ui_day_item_add(void)
 	if (is_appointment) {	/* Get the appointment duration */
 		item_time[0] = '\0';
 		for (;;) {
+			int early = 0;
 			status_mesg(mesg_2, "");
-			if (getstring(win[STA].p, item_time, LDUR, 0, 1) ==
+			if (getstring(win[STA].p, item_time, LTIME, 0, 1) ==
 			    GETSTRING_ESC)
 				return;
+			if (*item_time == '?') {
+				char *outstr;
+				item_time[0] = '\0';
+				asprintf(&outstr, mesg_help_1, DATEFMT_DESC(conf.input_datefmt));
+				status_mesg(outstr, mesg_help_2);
+				mem_free(outstr);
+				wgetch(win[KEY].p);
+				continue;
+			}
 			if (strlen(item_time) == 0) {
 				dur = 0;
 				break;
 			}
 			if (*item_time == '+') {
-				if (parse_duration(item_time + 1, &dur)) {
+				if (parse_duration(item_time + 1, &dur, start)) {
 					dur *= MININSEC;
 					break;
 				}
+			} else {
+				int ret, val = 1;
+				/* Same day? */
+				end = start;
+				ret = parse_datetime(item_time, &end, 0);
+				/*
+				 * If same day and end time is earlier than start time,
+				 * assume that it belongs to the next day.
+				 */
+				if (ret == PARSE_DATETIME_HAS_TIME && end < start) {
+					end = date_sec_change(end, 0, 1);
+					/* Still valid? */
+					val = check_sec(&end);
+				}
+				if (ret && val && start <= end) {
+					dur = end - start;
+					break;
+				}
+				/* Valid format, but too early? */
+				early = ret && val && end < start;
+					
 			}
-			end = start;
-			ret = parse_datetime(item_time, &end);
-			/*
-			 * If the user enters a end time which is smaller than
-			 * the start time, assume that the time belongs to the
-			 * next day.
-			 */
-			if (!(ret & PARSE_DATETIME_HAS_DATE) && end < start)
-				end = date_sec_change(end, 0, 1);
-			if (ret) {
-				dur = end - start;
-				break;
-			}
-			status_mesg(format_message_2, enter_str);
-			keys_wait_for_any_key(win[KEY].p);
+			status_mesg(early ? format_message_3 : format_message_2 , enter_str);
+			keys_wgetch(win[KEY].p);
 		}
 	}
 
 	status_mesg(mesg_3, "");
-	if (getstring(win[STA].p, item_mesg, BUFSIZ, 0, 1) ==
-	    GETSTRING_VALID) {
+	if (getstring(win[STA].p, item_mesg, BUFSIZ, 0, 1) == GETSTRING_VALID) {
 		if (is_appointment) {
 			item.apt = apoint_new(item_mesg, 0L, start, dur, 0L);
 			if (notify_bar())
@@ -594,9 +746,11 @@ void ui_day_item_add(void)
 			item.ev = event_new(item_mesg, 0L, start, 1);
 		}
 		io_set_modified();
-		day_process_storage(ui_calendar_get_slctd_day(), 0);
-		ui_day_load_items();
-		ui_day_set_selitem_by_aptev_ptr(item);
+		/* Set the selected APP item. */
+		struct day_item d = empty_day;
+		d.order = start;
+		d.item = item;
+		day_set_sel_data(&d);
 	}
 
 	ui_calendar_monthly_view_cache_set_invalid();
@@ -621,12 +775,12 @@ void ui_day_item_delete(unsigned reg)
 	      "Delete (i)tem or just its (n)ote?");
 	const char *note_choices = _("[in]");
 	const int nb_note_choices = 2;
-	long date = ui_calendar_get_slctd_day_sec();
+	time_t occurrence;
 
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = ui_day_selitem();
+	struct day_item *p = ui_day_get_sel();
 
 	if (conf.confirm_delete) {
 		if (status_ask_bool(del_app_str) != 1) {
@@ -655,8 +809,21 @@ void ui_day_item_delete(unsigned reg)
 		case 1:
 			break;
 		case 2:
-			day_item_add_exc(p, date);
+			if (p->type == RECUR_EVNT) {
+				day_item_add_exc(p, ui_day_sel_date());
+			} else {
+				recur_apoint_find_occurrence(p->item.rapt,
+							     ui_day_sel_date(),
+							     &occurrence);
+				day_item_add_exc(p, occurrence);
+			}
+
 			io_set_modified();
+			ui_calendar_monthly_view_cache_set_invalid();
+			/* Keep the selection on the same day. */
+			day_set_sel_data(
+				day_get_item(listbox_get_sel(&lb_apt) - 1)
+			);
 			return;
 		default:
 			return;
@@ -664,11 +831,12 @@ void ui_day_item_delete(unsigned reg)
 	}
 
 	ui_day_item_cut_free(reg);
-	p = day_cut_item(date, listbox_get_sel(&lb_apt));
+	p = day_cut_item(listbox_get_sel(&lb_apt));
 	day_cut[reg].type = p->type;
 	day_cut[reg].item = p->item;
+	/* Keep the selection on the same day. */
+	day_set_sel_data(day_get_item(listbox_get_sel(&lb_apt) - 1));
 	io_set_modified();
-
 	ui_calendar_monthly_view_cache_set_invalid();
 }
 
@@ -681,11 +849,6 @@ void ui_day_item_delete(unsigned reg)
  */
 void ui_day_item_repeat(void)
 {
-	struct tm lt;
-	time_t t;
-	int year = 0, month = 0, day = 0;
-	struct date until_date;
-	char *outstr;
 	char user_input[BUFSIZ] = "";
 	const char *msg_rpt_prefix = _("Enter the repetition type:");
 	const char *msg_rpt_daily = _("(d)aily");
@@ -695,16 +858,14 @@ void ui_day_item_repeat(void)
 	const char *msg_type_choice = _("[dwmy]");
 	const char *mesg_freq_1 = _("Enter the repetition frequency:");
 	const char *mesg_wrong_freq = _("Invalid frequency.");
-	const char *mesg_until_1 =
-	    _("Enter end date ([%s]), duration ([+xxwxxd]) or '0' for endless repetition:");
-	const char *mesg_wrong_1 = _("The entered date is not valid.");
-	const char *mesg_wrong_2 =
-	    _("Possible formats are [%s], [+xxwxxd] or '0'.");
-	const char *wrong_type_1 =
-	    _("This item is already a repeated one.");
+	const char *mesg_until_1 = _("Enter end date or duration ('?' for input formats):");
+	const char *mesg_help_1 = _("Date: %s (year or month may be omitted). Endless duration: '0'.");
+	const char *mesg_help_2 = _("Duration in days: +dd. Duration in weeks and days: +??w??d.");
+	const char *mesg_wrong_1 = _("Invalid date.");
+	const char *mesg_wrong_2 = _("Press [ENTER] to continue.");
+	const char *wrong_type_1 = _("This item is already a repeated one.");
 	const char *wrong_type_2 = _("Press [ENTER] to continue.");
-	const char *mesg_older =
-	    _("Sorry, the date you entered is older than the item start time.");
+	const char *mesg_older = _("Invalid date: end date must come after start date (%s).");
 
 	char *msg_asktype;
 	asprintf(&msg_asktype, "%s %s, %s, %s, %s", msg_rpt_prefix,
@@ -715,7 +876,7 @@ void ui_day_item_repeat(void)
 	int item_nb;
 	struct day_item *p;
 	struct recur_apoint *ra;
-	time_t until, date;
+	time_t until;
 	unsigned days;
 
 	if (day_item_count(0) <= 0)
@@ -759,75 +920,74 @@ void ui_day_item_repeat(void)
 		user_input[0] = '\0';
 	}
 
+	char *outstr, *datestr;
 	for (;;) {
-		asprintf(&outstr, mesg_until_1,
-			 DATEFMT_DESC(conf.input_datefmt));
-		status_mesg(outstr, "");
-		mem_free(outstr);
-		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) !=
-		    GETSTRING_VALID)
+		status_mesg(mesg_until_1, "");
+		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) == GETSTRING_ESC)
 			goto cleanup;
-		if (strlen(user_input) == 1 && strcmp(user_input, "0") == 0) {
+		if (strcmp(user_input, "") == 0 || strcmp(user_input, "0") == 0) {
 			until = 0;
 			break;
 		}
-		if (*user_input == '+') {
-			if (!parse_date_duration(user_input + 1, &days)) {
-				asprintf(&outstr, mesg_wrong_2,
-					 DATEFMT_DESC(conf.input_datefmt));
-				status_mesg(mesg_wrong_1, outstr);
-				mem_free(outstr);
-				keys_wait_for_any_key(win[KEY].p);
-				continue;
-			}
-			t = p->start;
-			localtime_r(&t, &lt);
-			date_change(&lt, 0, days);
-			until_date.dd = lt.tm_mday;
-			until_date.mm = lt.tm_mon + 1;
-			until_date.yyyy = lt.tm_year + 1900;
-		} else if (parse_date_interactive(user_input, &year, &month,
-						  &day)) {
-			t = p->start;
-			localtime_r(&t, &lt);
-			until_date.dd = day;
-			until_date.mm = month;
-			until_date.yyyy = year;
-		} else {
-			asprintf(&outstr, mesg_wrong_2,
-				 DATEFMT_DESC(conf.input_datefmt));
-			status_mesg(mesg_wrong_1, outstr);
+		if (*user_input == '?') {
+			user_input[0] = '\0';
+			asprintf(&outstr, mesg_help_1, DATEFMT_DESC(conf.input_datefmt));
+			status_mesg(outstr, mesg_help_2);
 			mem_free(outstr);
-			keys_wait_for_any_key(win[KEY].p);
+			wgetch(win[KEY].p);
 			continue;
 		}
-
-		until = date2sec(until_date, lt.tm_hour, lt.tm_min);
+		if (*user_input == '+') {
+			if (!parse_date_duration(user_input + 1, &days, p->start)) {
+				status_mesg(mesg_wrong_1, mesg_wrong_2);
+				keys_wgetch(win[KEY].p);
+				continue;
+			}
+			until = date_sec_change(p->start, 0, days);
+		} else {
+			int year, month, day;
+			if (!parse_date(user_input, conf.input_datefmt,
+			    &year, &month, &day, ui_calendar_get_slctd_day())) {
+				status_mesg(mesg_wrong_1, mesg_wrong_2);
+				keys_wgetch(win[KEY].p);
+				continue;
+			}
+			struct date d = { day, month, year };
+			until = date2sec(d, 0, 0);
+		}
 		if (until >= p->start)
 			break;
-		status_mesg(mesg_older, wrong_type_2);
-		keys_wait_for_any_key(win[KEY].p);
+
+		datestr = date_sec2date_str(p->start, DATEFMT(conf.input_datefmt));
+		asprintf(&outstr, mesg_older, datestr);
+		status_mesg(outstr, wrong_type_2);
+		mem_free(datestr);
+		mem_free(outstr);
+		keys_wgetch(win[KEY].p);
 	}
 
-	date = ui_calendar_get_slctd_day_sec();
+	/* Set the selected APP item. */
+	struct day_item d = empty_day;
 	if (p->type == EVNT) {
 		struct event *ev = p->item.ev;
-		recur_event_new(ev->mesg, ev->note, ev->day, ev->id, type,
-				freq, until, NULL);
+		d.item.rev = recur_event_new(ev->mesg, ev->note, ev->day,
+					     ev->id, type, freq, until, NULL);
 	} else if (p->type == APPT) {
 		struct apoint *apt = p->item.apt;
-		ra = recur_apoint_new(apt->mesg, apt->note, apt->start,
-				      apt->dur, apt->state, type, freq,
-				      until, NULL);
+		d.item.rapt = ra = recur_apoint_new(apt->mesg, apt->note,
+						    apt->start, apt->dur,
+						    apt->state, type, freq,
+						    until, NULL);
 		if (notify_bar())
 			notify_check_repeated(ra);
 	} else {
 		EXIT(_("wrong item type"));
 		/* NOTREACHED */
 	}
+	day_set_sel_data(&d);
 
 	ui_day_item_cut_free(REG_BLACK_HOLE);
-	p = day_cut_item(date, item_nb);
+	p = day_cut_item(item_nb);
 	day_cut[REG_BLACK_HOLE].type = p->type;
 	day_cut[REG_BLACK_HOLE].item = p->item;
 	io_set_modified();
@@ -870,7 +1030,7 @@ void ui_day_item_copy(unsigned reg)
 	if (day_item_count(0) <= 0 || reg == REG_BLACK_HOLE)
 		return;
 
-	struct day_item *item = ui_day_selitem();
+	struct day_item *item = ui_day_get_sel();
 	ui_day_item_cut_free(reg);
 	day_item_fork(item, &day_cut[reg]);
 }
@@ -878,15 +1038,15 @@ void ui_day_item_copy(unsigned reg)
 /* Paste a previously cut item. */
 void ui_day_item_paste(unsigned reg)
 {
-	struct day_item day;
+	struct day_item day = empty_day;
 
 	if (reg == REG_BLACK_HOLE || !day_cut[reg].type)
 		return;
 
 	day_item_fork(&day_cut[reg], &day);
-	day_paste_item(&day, ui_calendar_get_slctd_day_sec());
+	day_paste_item(&day, ui_day_sel_date());
+	day_set_sel_data(&day);
 	io_set_modified();
-
 	ui_calendar_monthly_view_cache_set_invalid();
 }
 
@@ -898,11 +1058,53 @@ void ui_day_load_items(void)
 void ui_day_sel_reset(void)
 {
 	listbox_set_sel(&lb_apt, 0);
+	/* Make the day visible. */
+	if (lb_apt.item_sel)
+		listbox_item_in_view(&lb_apt, lb_apt.item_sel - 1);
+	set_slctd_day();
 }
 
-void ui_day_sel_move(int delta)
+int ui_day_sel_move(int delta)
 {
-	listbox_sel_move(&lb_apt, delta);
+	int ret;
+
+	ret = listbox_sel_move(&lb_apt, delta);
+	/* When moving up, make the line above visible. */
+	if (delta < 0 && ret && lb_apt.item_sel)
+		listbox_item_in_view(&lb_apt, lb_apt.item_sel - 1);
+	set_slctd_day();
+	return ret;
+}
+
+/*
+ * Move the selection to the beginning of the current day or
+ * the day n days before or after.
+ */
+void ui_day_sel_daybegin(int n)
+{
+	if (n == 0) {
+		daybegin(0);
+		return;
+	}
+	int dir = n > 0 ? 1 : -1;
+	n = dir * n;
+	for (int i = 0; i < n; i++)
+		daybegin(dir);
+}
+
+/*
+ * Move the selection to the end of the current day.
+ */
+void ui_day_sel_dayend(void)
+{
+	int sel = listbox_get_sel(&lb_apt);
+
+	while (day_get_item(sel)->type != END_SEPARATOR)
+		sel++;
+	while (lb_apt.type[sel] != LISTBOX_ROW_TEXT)
+		sel--;
+	listbox_set_sel(&lb_apt, sel);
+	listbox_item_in_view(&lb_apt, sel);
 }
 
 static char *fmt_day_heading(time_t date)
@@ -919,10 +1121,10 @@ static char *fmt_day_heading(time_t date)
 /* Display appointments in the corresponding panel. */
 void ui_day_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 {
-	struct date slctd_date = *ui_calendar_get_slctd_day();
-	time_t date = date2sec(slctd_date, 0, 0);
 	struct day_item *item = day_get_item(n);
-	int width = lb_apt.sw.w - 2;
+	/* The item order always indicates the date. */
+	time_t date = update_time_in_date(item->order, 0, 0);
+	int width = lb_apt.sw.w - 2, is_slctd;
 
 	hilt = hilt && (wins_slctd() == APP);
 	if (item->type == EVNT || item->type == RECUR_EVNT) {
@@ -931,18 +1133,20 @@ void ui_day_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 		day_display_item_date(item, win, !hilt, date, y, 1);
 		day_display_item(item, win, !hilt, width - 1, y + 1, 1);
 	} else if (item->type == DAY_HEADING) {
+		is_slctd = conf.multiple_days && (date == get_slctd_day());
+		if (conf.header_line && n) {
+			wmove(win, y, 0);
+			whline(win, ACS_HLINE, width);
+		}
 		char *buf = fmt_day_heading(date);
 		utf8_chop(buf, width);
-		custom_apply_attr(win, ATTR_HIGHEST);
-		mvwprintw(win, y,
+		custom_apply_attr(win, is_slctd ? ATTR_MIDDLE : ATTR_HIGHEST);
+		mvwprintw(win, y + (conf.header_line && n),
 			conf.heading_pos == RIGHT ? width - utf8_strwidth(buf) - 1 :
 			conf.heading_pos == LEFT ? 1 :
 			(width - utf8_strwidth(buf)) / 2, "%s", buf);
-		custom_remove_attr(win, ATTR_HIGHEST);
+		custom_remove_attr(win, is_slctd ? ATTR_MIDDLE : ATTR_HIGHEST);
 		mem_free(buf);
-	} else if (item->type == DAY_SEPARATOR) {
-		wmove(win, y, 0);
-		whline(win, 0, width);
 	}
 }
 
@@ -950,7 +1154,10 @@ enum listbox_row_type ui_day_row_type(int n, void *cb_data)
 {
 	struct day_item *item = day_get_item(n);
 
-	if (item->type == DAY_HEADING || item->type == DAY_SEPARATOR)
+	if (item->type == DAY_HEADING ||
+	    item->type == EVNT_SEPARATOR ||
+	    item->type == EMPTY_SEPARATOR ||
+	    item->type == END_SEPARATOR)
 		return LISTBOX_ROW_CAPTION;
 	else
 		return LISTBOX_ROW_TEXT;
@@ -960,16 +1167,22 @@ int ui_day_height(int n, void *cb_data)
 {
 	struct day_item *item = day_get_item(n);
 
-	if (item->type == APPT || item->type == RECUR_APPT)
-		return 3;
+	if (item->type == DAY_HEADING)
+		return 1 + (conf.header_line && n);
+	else if (item->type == APPT || item->type == RECUR_APPT)
+		return conf.empty_appt_line ? 3 : 2;
+	else if (item->type == EVNT_SEPARATOR)
+		return conf.event_separator;
+	else if (item->type == END_SEPARATOR)
+		return conf.day_separator;
 	else
 		return 1;
 }
 
 /* Updates the Appointment panel */
-void ui_day_update_panel(int which_pan)
+void ui_day_update_panel(int hilt)
 {
-	listbox_display(&lb_apt);
+	listbox_display(&lb_apt, hilt);
 }
 
 void ui_day_popup_item(void)
@@ -977,7 +1190,7 @@ void ui_day_popup_item(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = ui_day_selitem();
+	struct day_item *item = ui_day_get_sel();
 	day_popup_item(item);
 }
 
@@ -986,7 +1199,7 @@ void ui_day_flag(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = ui_day_selitem();
+	struct day_item *item = ui_day_get_sel();
 	day_item_switch_notify(item);
 	io_set_modified();
 }
@@ -996,7 +1209,7 @@ void ui_day_view_note(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = ui_day_selitem();
+	struct day_item *item = ui_day_get_sel();
 	day_view_note(item, conf.pager);
 }
 
@@ -1005,7 +1218,7 @@ void ui_day_edit_note(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = ui_day_selitem();
+	struct day_item *item = ui_day_get_sel();
 	day_edit_note(item, conf.editor);
 	io_set_modified();
 }
