@@ -269,11 +269,11 @@ static const struct utf8_range utf8_widthtab[] = {
 	{0xe0100, 0xe01ef, 0}
 };
 
-/* Decode a UTF-8 code point. */
-int utf8_ord(const char *s)
+/* Decode a UTF-8 encoded character. Return the Unicode code point. */
+int utf8_decode(const char *s)
 {
 	if (UTF8_ISCONT(*s))
-		return 0;
+		return -1;
 
 	switch (UTF8_LENGTH(*s)) {
 	case 1:
@@ -285,26 +285,57 @@ int utf8_ord(const char *s)
 			(s[0] & 0x0f) << 12;
 	case 4:
 		return (((s[3] & 0x3f) | (s[2] & 0x3f) << 6) |
-			(s[1] & 0x3f) << 12) | (s[0] & 0x3f) << 18;
-	case 5:
-		return ((((s[4] & 0x3f) | (s[3] & 0x3f) << 6) |
-			(s[2] & 0x3f) << 12) | (s[1] & 0x3f) << 18) |
-			(s[0] & 0x3f) << 24;
-	case 6:
-		return (((((s[5] & 0x3f) | (s[4] & 0x3f) << 6) |
-			(s[3] & 0x3f) << 12) | (s[2] & 0x3f) << 18) |
-			(s[1] & 0x3f) << 24) | (s[0] & 0x3f) << 30;
+			(s[1] & 0x3f) << 12) | (s[0] & 0x7) << 18;
 	default:
-		return 0;
+		return -1;
 	}
 }
 
-/* Get the width of a UTF-8 character. */
+/*
+ * Encode a Unicode code point.
+ * Return a pointer to the resulting UTF-8 encoded character.
+ */
+char *utf8_encode(int u)
+{
+	static char c[5]; /* 4 bytes + string termination */
+
+	/* 0x0000 - 0x007F: 0xxxxxxx */
+	if (u < 0x80) {
+		*(c + 1) = '\0';
+		*c = u;
+	/* 0x0080 - 0x07FF: 110xxxxx 10xxxxxx */
+	} else if (u < 0x800) {
+		*(c + 2) = '\0';
+		*(c + 1) = (u       & 0x3F) | 0x80;
+		*c       = (u >> 6)         | 0xC0;
+	/* 0x0800 - 0xFFFF: 1110xxxx 10xxxxxx 10xxxxxx */
+	} else if (u < 0x10000) {
+		*(c + 3) = '\0';
+		*(c + 2) = (u       & 0x3F) | 0x80;
+		*(c + 1) = (u >> 6  & 0x3F) | 0x80;
+		*c       = (u >> 12)        | 0xE0;
+	} else if (u < 0x110000) {
+	/* 0x10000 - 0x10FFFF: 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx */
+		*(c + 4) = '\0';
+		*(c + 3) = (u       & 0x3F) | 0x80;
+		*(c + 2) = (u >> 6  & 0x3F) | 0x80;
+		*(c + 1) = (u >> 12 & 0x3F) | 0x80;
+		*c       = (u >> 18)        | 0xF0;
+	} else {
+		return NULL;
+	}
+
+	return c;
+}
+
+/* Get the display width of a UTF-8 character. */
 int utf8_width(char *s)
 {
 	int val, low, high, cur;
 
-	val = utf8_ord(s);
+	if (UTF8_ISCONT(*s))
+		return 0;
+	val = utf8_decode(s);
 	low = 0;
 	high = ARRAY_SIZE(utf8_widthtab);
 	do {
@@ -328,11 +359,8 @@ int utf8_strwidth(char *s)
 {
 	int width = 0;
 
-	for (; s && *s; s++) {
-		if (!UTF8_ISCONT(*s))
-			width += utf8_width(s);
-	}
-
+	for (; *s; s++)
+		width += utf8_width(s);
 	return width;
 }
 
