@@ -74,11 +74,11 @@ void exit_calcurse(int status)
 {
 	int was_interactive;
 
-	ui_calendar_stop_date_thread();
-	io_stop_psave_thread();
-
 	if (ui_mode == UI_CURSES) {
 		notify_stop_main_thread();
+		ui_calendar_stop_date_thread();
+		io_stop_psave_thread();
+
 		clear();
 		wins_refresh();
 		endwin();
@@ -192,7 +192,7 @@ void status_mesg(const char *msg1, const char *msg2)
 /*
  * Prompts the user to make a choice between named alternatives.
  *
- * The available choices are described by a string of the form
+ * The available choices are described (in po-files) by a string of the form
  * "[ynp]". The first and last char are ignored (they are only here to
  * make the translators' life easier), and every other char indicates
  * a key the user is allowed to press.
@@ -203,25 +203,36 @@ void status_mesg(const char *msg1, const char *msg2)
 int status_ask_choice(const char *message, const char choice[],
 		      int nb_choice)
 {
-	/* "[4/2/f/t/w/.../Z] " */
-	char avail_choice[2 * nb_choice + 3];
-	int i, ch;
+	/* Turn "[42w...Z]" into * "[4/2/w/.../Z]". */
+	char avail_choice[nb_choice * UTF8_MAXLEN + nb_choice + 1];
+	int ichoice[nb_choice];
+	int i, j, k, n, ch;
 
 	avail_choice[0] = '[';
-	for (i = 1; i <= nb_choice; i++) {
-		avail_choice[i * 2 - 1] = choice[i];
-		avail_choice[i * 2] = '/';
+	for (n = 0, i = 1, j = 1; n < nb_choice; n++, i += k) {
+		for (k = 0; k < UTF8_LENGTH(choice[i]); k++) {
+			avail_choice[j] = choice[i + k];
+			j++;
+		}
+		avail_choice[j] = '/';
+		j++;
 	}
-	avail_choice[nb_choice * 2] = ']';
-	avail_choice[nb_choice * 2 + 1] = '\0';
+	avail_choice[j - 1] = ']';
+	avail_choice[j] = '\0';
 
 	status_mesg(message, avail_choice);
 
+	/* Convert the character choices to internal integer codes. */
+	for (n = 0, i = 1; n < nb_choice; n++, i += j) {
+		j = UTF8_LENGTH(choice[i]);
+		ichoice[n] = utf8_decode(choice + i) + (j > 1 ? KEY_MAX : 0);
+	}
+
 	for (;;) {
 		ch = keys_wgetch(win[KEY].p);
-		for (i = 1; i <= nb_choice; i++)
-			if (ch == choice[i])
-				return i;
+		for (i = 0; i < nb_choice; i++)
+			if (ch == ichoice[i])
+				return i + 1;
 		if (ch == ESCAPE)
 			return (-1);
 		if (resize) {
@@ -347,25 +358,25 @@ int is_all_digit(const char *string)
 }
 
 /* Given an item date expressed in seconds, return its start time in seconds. */
-long get_item_time(long date)
+long get_item_time(time_t date)
 {
 	return (long)(get_item_hour(date) * HOURINSEC +
 		      get_item_min(date) * MININSEC);
 }
 
-int get_item_hour(long date)
+int get_item_hour(time_t date)
 {
 	struct tm lt;
 
-	localtime_r((time_t *) & date, &lt);
+	localtime_r(&date, &lt);
 	return lt.tm_hour;
 }
 
-int get_item_min(long date)
+int get_item_min(time_t date)
 {
 	struct tm lt;
 
-	localtime_r((time_t *) & date, &lt);
+	localtime_r(&date, &lt);
 	return lt.tm_min;
 }
 
@@ -397,6 +408,19 @@ time_t date2sec(struct date day, unsigned hour, unsigned min)
 	return t;
 }
 
+/* Return the (calcurse) date of a (Unix) time in seconds. */
+struct date sec2date(time_t t)
+{
+	struct tm tm;
+	struct date d;
+
+	localtime_r(&t, &tm);
+	d.dd = tm.tm_mday;
+	d.mm = tm.tm_mon + 1;
+	d.yyyy = tm.tm_year + 1900;
+	return d;
+}
+
 time_t utcdate2sec(struct date day, unsigned hour, unsigned min)
 {
 	char *tz;
@@ -421,6 +445,24 @@ time_t utcdate2sec(struct date day, unsigned hour, unsigned min)
 	return t;
 }
 
+/* Compare two calcurse dates. */
+int date_cmp(struct date *d1, struct date *d2)
+{
+	if (d1->yyyy < d2->yyyy)
+		return -1;
+	if (d1->yyyy > d2->yyyy)
+		return 1;
+	if (d1->mm < d2->mm)
+		return -1;
+	if (d1->mm > d2->mm)
+		return 1;
+	if (d1->dd < d2->dd)
+		return -1;
+	if (d1->dd > d2->dd)
+		return 1;
+	return 0;
+}
+
 /* Compare two dates (without comparing times). */
 int date_cmp_day(time_t d1, time_t d2)
 {
@@ -445,24 +487,8 @@ int date_cmp_day(time_t d1, time_t d2)
 	return 0;
 }
 
-/* Return a string containing the date, given a date in seconds. */
-char *date_sec2date_str(long sec, const char *datefmt)
-{
-	struct tm lt;
-	char *datestr = (char *)mem_calloc(BUFSIZ, sizeof(char));
-
-	if (sec == 0) {
-		strncpy(datestr, "0", BUFSIZ);
-	} else {
-		localtime_r((time_t *) & sec, &lt);
-		strftime(datestr, BUFSIZ, datefmt, &lt);
-	}
-
-	return datestr;
-}
-
 /* Generic function to format date. */
-void date_sec2date_fmt(long sec, const char *fmt, char *datef)
+void date_sec2date_fmt(time_t sec, const char *fmt, char *datef)
 {
 #if ENABLE_NLS
 	/* TODO: Find a better way to deal with localization and strftime(). */
@@ -471,7 +497,7 @@ void date_sec2date_fmt(long sec, const char *fmt, char *datef)
 #endif
 
 	struct tm lt;
-	localtime_r((time_t *) & sec, &lt);
+	localtime_r(&sec, &lt);
 	strftime(datef, BUFSIZ, fmt, &lt);
 
 #if ENABLE_NLS
@@ -480,8 +506,16 @@ void date_sec2date_fmt(long sec, const char *fmt, char *datef)
 #endif
 }
 
+/* Return a string containing the date, given a date in seconds. */
+char *date_sec2date_str(time_t sec, const char *datefmt)
+{
+	char *datestr = (char *)mem_calloc(BUFSIZ, sizeof(char));
+	date_sec2date_fmt(sec, datefmt, datestr);
+	return datestr;
+}
+
 /*
- * Used to change date by adding a certain amount of days or weeks.
+ * Used to change date by adding a certain amount of days or months.
  * Returns 0 on success, 1 otherwise.
  */
 int date_change(struct tm *date, int delta_month, int delta_day)
@@ -501,9 +535,9 @@ int date_change(struct tm *date, int delta_month, int delta_day)
 }
 
 /*
- * Used to change date by adding a certain amount of days or weeks.
+ * Used to change date by adding a certain amount of days or months.
  */
-long date_sec_change(long date, int delta_month, int delta_day)
+time_t date_sec_change(time_t date, int delta_month, int delta_day)
 {
 	struct tm lt;
 	time_t t;
@@ -520,27 +554,43 @@ long date_sec_change(long date, int delta_month, int delta_day)
 }
 
 /*
- * Return a long containing the date which is updated taking into account
- * the new time and date entered by the user.
+ * A date in seconds is updated with new day, month and year and returned.
  */
-long update_time_in_date(long date, unsigned hr, unsigned mn)
+static time_t update_date_in_date(time_t date, int day, int month, int year)
 {
 	struct tm lt;
-	time_t t, new_date;
 
-	t = date;
-	localtime_r(&t, &lt);
-	lt.tm_hour = hr;
-	lt.tm_min = mn;
-	lt.tm_sec = 0;
-	new_date = mktime(&lt);
-	EXIT_IF(new_date == -1, _("error in mktime"));
+	localtime_r(&date, &lt);
+	lt.tm_mday = day;
+	lt.tm_mon = month - 1;
+	lt.tm_year = year - 1900;
+	lt.tm_isdst = -1;
+	date = mktime(&lt);
+	EXIT_IF(date == -1, _("error in mktime"));
 
-	return new_date;
+	return date;
 }
 
 /*
- * Returns the date in seconds from year 1900.
+ * A date in seconds is updated with new hour and minutes and returned.
+ */
+time_t update_time_in_date(time_t date, unsigned hr, unsigned mn)
+{
+	struct tm lt;
+
+	localtime_r(&date, &lt);
+	lt.tm_hour = hr;
+	lt.tm_min = mn;
+	lt.tm_sec = 0;
+	lt.tm_isdst = -1;
+	date = mktime(&lt);
+	EXIT_IF(date == -1, _("error in mktime"));
+
+	return date;
+}
+
+/*
+ * Returns the date in seconds from year 1970.
  * If no date is entered, current date is chosen.
  */
 time_t get_sec_date(struct date date)
@@ -573,19 +623,35 @@ long min2sec(unsigned minutes)
 /*
  * Display a scroll bar when there are so many items that they
  * can not be displayed inside the corresponding panel.
+ * Leave it out in the appointments panel in when multiple days mode.
  */
-void
-draw_scrollbar(WINDOW * win, int y, int x, int length,
-	       int bar_top, int bar_bottom, unsigned hilt)
+void draw_scrollbar(struct scrollwin *sw, int hilt)
 {
-	mvwvline(win, bar_top, x, ACS_VLINE, bar_bottom - bar_top + 1);
+	if (sw == &lb_apt.sw && conf.multiple_days)
+		return;
+
+	int y = (conf.compact_panels ? 1 : 3);
+	int h = sw->h - (conf.compact_panels ? 2 : 4);
+
+	int sbar_h = MAX(h * h / sw->line_num, 1);
+	int sbar_y = y + sw->line_off * (h - sbar_h) / (sw->line_num - h);
+	int sbar_x = sw->w - 1;
+
+	/* Redraw part of the border. */
 	if (hilt)
-		custom_apply_attr(win, ATTR_HIGHEST);
-	wattron(win, A_REVERSE);
-	mvwvline(win, y, x, ' ', length);
-	wattroff(win, A_REVERSE);
+		custom_apply_attr(sw->win, ATTR_HIGHEST);
+	mvwvline(sw->win, y, sbar_x, ACS_VLINE, h);
 	if (hilt)
-		custom_remove_attr(win, ATTR_HIGHEST);
+		custom_remove_attr(sw->win, ATTR_HIGHEST);
+
+	/* Draw the scrollbar. */
+	if (hilt)
+		custom_apply_attr(sw->win, ATTR_HIGHEST);
+	wattron(sw->win, A_REVERSE);
+	mvwvline(sw->win, sbar_y, sbar_x, ' ', sbar_h);
+	wattroff(sw->win, A_REVERSE);
+	if (hilt)
+		custom_remove_attr(sw->win, ATTR_HIGHEST);
 }
 
 /*
@@ -617,7 +683,7 @@ item_in_popup(const char *a_start, const char *a_end, const char *msg,
 	delwin(popup_win);
 }
 
-/* Returns the beginning of current day in seconds from 1900. */
+/* Returns the beginning of current day in seconds from 1970. */
 time_t get_today(void)
 {
 	struct tm lt;
@@ -633,10 +699,16 @@ time_t get_today(void)
 	return date2sec(day, 0, 0);
 }
 
-/* Returns the current time in seconds. */
-long now(void)
+/* Returns the beginning of the selected day in the calendar. */
+time_t get_slctd_day(void)
 {
-	return (long)time(NULL);
+	return date2sec(*ui_calendar_get_slctd_day(), 0, 0);
+}
+
+/* Returns the current time in seconds. */
+time_t now(void)
+{
+	return time(NULL);
 }
 
 char *nowstr(void)
@@ -669,6 +741,12 @@ print_bool_option_incolor(WINDOW * win, unsigned option, int pos_y,
 		EXIT(_("option not defined"));
 	}
 
+	/*
+	 * Possibly nested custom_apply_attr() calls. Turn
+	 * custom_apply_attr(ATTR_HIGHEST) off explicitly,
+	 * while it may have other attributes besides the colour.
+	 */
+	custom_remove_attr(win, ATTR_HIGHEST);
 	custom_apply_attr(win, color);
 	mvwaddstr(win, pos_y, pos_x, option_value);
 	custom_remove_attr(win, color);
@@ -749,14 +827,26 @@ static void get_weekday_ymd(int *year, int *month, int *day, int weekday)
 }
 
 /*
- * Check if a date is valid.
+ * Check if a calcurse date is valid.
  */
 int check_date(unsigned year, unsigned month, unsigned day)
 {
-	return (year >= 1902 && month >= 1 && month <= 12 && day >= 1 &&
+	return ((YEAR1902_2037 ? year >= 1902 && year <= 2037 : 1) &&
+		month >= 1 && month <= 12 &&
+		day >= 1 &&
 		day <= days[month - 1] + (month == 2 && ISLEAP(year)) ? 1 : 0);
 }
 
+/*
+ * Check that a time in seconds is a valid calcurse date (ignoring hour:min:sec).
+ */
+int check_sec(time_t *time)
+{
+	struct tm tm;
+	localtime_r(time, &tm);
+	return check_date(tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);
+}
+
 /*
  * Convert a string containing a date into three integers containing the year,
  * month and day.
@@ -895,10 +985,18 @@ parse_date_interactive(const char *datestr, int *year, int *month, int *day)
 
 /*
  * Convert a date duration string into a number of days.
+ * If start is non-zero, the final end time is validated.
+ *
+ * Allowed formats in lenient BNF:
+ * <duration> ::= <days> | <period>
+ * <period>   ::= [ <weeks>w ][ <days>d ]
+ * Notes:
+ *            <days> and <weeks> are any integer >= 0.
+ *            <period> must have at least one non-terminal.
  *
  * Returns 1 on success and 0 on failure.
  */
-int parse_date_duration(const char *string, unsigned *days)
+int parse_date_duration(const char *string, unsigned *days, time_t start)
 {
 	enum {
 		STATE_INITIAL,
@@ -954,13 +1052,20 @@ int parse_date_duration(const char *string, unsigned *days)
 			denom = 1;
 		}
 	}
-
 	if (state == STATE_DONE && in > 0)
 		return 0;
-
 	dur += in;
+	if (start) {
+		/* wanted: start = start + dur * DAYINSEC */
+		long p;
+		if (overflow_mul(dur, DAYINSEC, &p))
+			return 0;
+		if (overflow_add(start, p, &start))
+			return 0;
+		if (!check_sec(&start))
+			return 0;
+	}
 	*days = dur;
-
 	return 1;
 }
 
@@ -974,7 +1079,9 @@ int check_time(unsigned hours, unsigned minutes)
 
 /*
  * Converts a time string into hours and minutes. Short forms like "23:"
- * (23:00) or ":45" (0:45) are allowed.
+ * (23:00) or ":45" (0:45) are allowed as well as "2345". Note: the latter
+ * clashes with date formats 0001 .. 0031 and must be picked up before
+ * dates when parsing in parse_datetime.
  *
  * Returns 1 on success and 0 on failure.
  */
@@ -991,14 +1098,18 @@ int parse_time(const char *string, unsigned *hour, unsigned *minute)
 		if (*p == ':') {
 			if ((++n) > 1)
 				return 0;
-		} else if ((*p >= '0') && (*p <= '9')) {
-			if ((n == 0) && (p == (string + 2)) && *(p + 1))
-				n++;
-			in[n] = in[n] * 10 + (int)(*p - '0');
+		} else if (isdigit(*p)) {
+			in[n] = in[n] * 10 + *p - '0';
 		} else {
 			return 0;
 		}
 	}
+	/* 24-hour format without ':' (hhmm)? */
+	if (n == 0 && strlen(string) == 4) {
+		in[1] = in[0] % 100;
+		in[0] = in[0] / 100;
+		n = 1;
+	}
 
 	if (n != 1 || !check_time(in[0], in[1]))
 		return 0;
@@ -1010,18 +1121,21 @@ int parse_time(const char *string, unsigned *hour, unsigned *minute)
 
 /*
  * Converts a duration string into minutes.
+ * If start time is non-zero, the final end time is validated.
  *
- * Allowed formats (noted as regular expressions):
- *
- * - \d*:\d*
- * - (\d*m|\d*h(|\d*m)|\d*d(|\d*m|\d*h(|\d*m)))
- * - \d+
- *
- * "\d" is used as a placeholder for "(0|1|2|3|4|5|6|7|8|9)".
+ * Allowed formats in lenient BNF:
+ * <duration> ::= <minutes> | <time> | <period>
+ * <time>     ::= <hours>:<min>
+ * <period>   ::= [ <days>d ][ <hours>h ][ <minutes>m ]
+ * <min>      ::= integer in the range 0-59
+ * Notes:
+ *            <days>, <hours> and <minutes> are any sequence of
+ *	      the characters 1,2,..,9,0.
+ *            <period> must have at least one non-terminal.
  *
  * Returns 1 on success and 0 on failure.
  */
-int parse_duration(const char *string, unsigned *duration)
+int parse_duration(const char *string, unsigned *duration, time_t start)
 {
 	enum {
 		STATE_INITIAL,
@@ -1033,7 +1147,7 @@ int parse_duration(const char *string, unsigned *duration)
 
 	const char *p;
 	unsigned in = 0, frac = 0, denom = 1;
-	unsigned dur = 0;
+	long dur = 0;
 
 	if (!string || *string == '\0')
 		return 0;
@@ -1090,7 +1204,6 @@ int parse_duration(const char *string, unsigned *duration)
 				break;
 			case STATE_HHMM_MM:
 				return 0;
-				break;
 			default:
 				break;
 			}
@@ -1099,15 +1212,23 @@ int parse_duration(const char *string, unsigned *duration)
 			denom = 1;
 		}
 	}
-
 	if ((state == STATE_HHMM_MM && in >= HOURINMIN) ||
-	    ((state == STATE_DDHHMM_HH || state == STATE_DDHHMM_MM)
-	     && in > 0))
+	    ((state == STATE_DDHHMM_HH || state == STATE_DDHHMM_MM) && in > 0))
 		return 0;
-
 	dur += in;
+	if (start) {
+		/* wanted: end = start + dur * MININSEC */
+		time_t end;
+		long p, s;
+		if (overflow_mul(dur, MININSEC, &p))
+			return 0;
+		if (overflow_add(start, p, &s))
+			return 0;
+		end = s;
+		if (!check_sec(&end) || end < start)
+			return 0;
+	}
 	*duration = dur;
-
 	return 1;
 }
 
@@ -1120,42 +1241,53 @@ int parse_duration(const char *string, unsigned *duration)
  * updated and the date remains the same. If the string contains both a date
  * and a time, the time stamp is updated to match the given string.
  *
+ * The final time is validated. In addition, if a positive duration is given,
+ * time + duration validated (zero duration needs no validation).
+ *
  * Returns a positive value on success and 0 on failure. The least-significant
  * bit is set if the date was updated. Bit 1 is set if the time was updated.
  */
-int parse_datetime(const char *string, long *ts)
+int parse_datetime(const char *string, time_t *ts, time_t dur)
 {
-	char *t = mem_strdup(string);
-	char *p = strchr(t, ' ');
-
-	unsigned int hour, minute;
+	unsigned hour, minute;
 	int year, month, day;
-	struct date new_date;
 	int ret = 0;
 
-	if (p) {
-		*p = '\0';
-		if (parse_date_interactive(t, &year, &month, &day) &&
-		    parse_time(p + 1, &hour, &minute)) {
+	char *d = mem_strdup(string);
+	/* Split into date and time, if possible. */
+	char *t = strchr(d, ' ');
+	if (t)
+		*t++ = '\0';
+	if (t) {
+		if (parse_date_interactive(d, &year, &month, &day) &&
+	            parse_time(t, &hour, &minute)) {
 			ret |= PARSE_DATETIME_HAS_DATE |
 			       PARSE_DATETIME_HAS_TIME;
 		}
-	} else if (parse_time(t, &hour, &minute)) {
+	/* Time before date, see comments in parse_time(). */
+	} else if (parse_time(d, &hour, &minute)) {
 		ret |= PARSE_DATETIME_HAS_TIME;
-	} else if (parse_date_interactive(t, &year, &month, &day)) {
+	} else if (parse_date_interactive(d, &year, &month, &day)) {
 		ret |= PARSE_DATETIME_HAS_DATE;
 	}
-
-	if (ret & PARSE_DATETIME_HAS_DATE) {
-		new_date.dd = day;
-		new_date.mm = month;
-		new_date.yyyy = year;
-		*ts = date2sec(new_date, 0, 0) + get_item_time(*ts);
-	}
+	if (ret & PARSE_DATETIME_HAS_DATE)
+		*ts = update_date_in_date(*ts, day, month, year);
 	if (ret & PARSE_DATETIME_HAS_TIME)
 		*ts = update_time_in_date(*ts, hour, minute);
+	mem_free(d);
 
-	mem_free(t);
+	/* Is the resulting time a valid (start or end) time? */
+	if (!check_sec(ts))
+		return 0;
+	/* Is the resulting time + dur a valid end time? */
+	if (dur) {
+		/* want: sec = *ts + dur */
+		time_t sec;
+		if (overflow_add(*ts, dur, &sec))
+			return 0;
+		if (!check_sec(&sec))
+			return 0;
+	}
 	return ret;
 }
 
@@ -1492,22 +1624,25 @@ static enum format_specifier parse_fs(const char **s, char *extformat)
 	}
 }
 
-/* Print a formatted date to stdout. */
-static void print_date(long date, long day, const char *extformat)
+/*
+ * Print date to stdout, formatted to be displayed for day.
+ * The "day" argument may be any time belonging to that day.
+ */
+static void print_date(time_t date, time_t day, const char *extformat)
 {
 	char buf[BUFSIZ];
 
 	if (!strcmp(extformat, "epoch")) {
-		printf("%ld", date);
+		printf("%ld", (long)date);
 	} else {
-		time_t day_end = date_sec_change(day, 0, 1);
-		time_t t = date;
+		time_t day_start = update_time_in_date(day, 0, 0);
+		time_t day_end = date_sec_change(day_start, 0, 1);
 		struct tm lt;
 
-		localtime_r((time_t *) & t, &lt);
+		localtime_r((time_t *) &date, &lt);
 
 		if (extformat[0] == '\0' || !strcmp(extformat, "default")) {
-			if (date >= day && date <= day_end)
+			if (date >= day_start && date <= day_end)
 				strftime(buf, BUFSIZ, "%H:%M", &lt);
 			else
 				strftime(buf, BUFSIZ, "..:..", &lt);
@@ -1593,7 +1728,7 @@ static void print_datediff(long difference, const char *extformat)
 }
 
 /* Print a formatted appointment to stdout. */
-static void print_apoint_helper(const char *format, long day,
+static void print_apoint_helper(const char *format, time_t day,
 				struct apoint *apt, struct recur_apoint *rapt)
 {
 	const char *p;
@@ -1661,7 +1796,7 @@ static void print_apoint_helper(const char *format, long day,
 }
 
 /* Print a formatted event to stdout. */
-static void print_event_helper(const char *format, long day, struct event *ev,
+static void print_event_helper(const char *format, time_t day, struct event *ev,
 			       struct recur_event *rev)
 {
 	const char *p;
@@ -1711,20 +1846,20 @@ static void print_event_helper(const char *format, long day, struct event *ev,
 }
 
 /* Print a formatted appointment to stdout. */
-void print_apoint(const char *format, long day, struct apoint *apt)
+void print_apoint(const char *format, time_t day, struct apoint *apt)
 {
 	print_apoint_helper(format, day, apt, NULL);
 }
 
 /* Print a formatted event to stdout. */
-void print_event(const char *format, long day, struct event *ev)
+void print_event(const char *format, time_t day, struct event *ev)
 {
 	print_event_helper(format, day, ev, NULL);
 }
 
 /* Print a formatted recurrent appointment to stdout. */
 void
-print_recur_apoint(const char *format, long day, time_t occurrence,
+print_recur_apoint(const char *format, time_t day, time_t occurrence,
 		   struct recur_apoint *rapt)
 {
 	struct apoint apt;
@@ -1738,7 +1873,7 @@ print_recur_apoint(const char *format, long day, time_t occurrence,
 }
 
 /* Print a formatted recurrent event to stdout. */
-void print_recur_event(const char *format, long day,
+void print_recur_event(const char *format, time_t day,
 		       struct recur_event *rev)
 {
 	struct event ev;
@@ -1839,3 +1974,37 @@ int show_dialogs(void)
 {
 	return (!quiet) && conf.system_dialogs;
 }
+
+/*
+ * Overflow check for addition with positive second term.
+ */
+long overflow_add(long x, long y, long *z)
+{
+	if (!YEAR1902_2037)
+		goto exit;
+
+	if (y < 0)
+		return 1;
+	if (INT_MAX - y < x)
+		return 1;
+   exit:
+	*z = x + y;
+	return 0;
+}
+
+/*
+ * Overflow check for multiplication with positive terms.
+ */
+long overflow_mul(long x, long y, long *z)
+{
+	if (!YEAR1902_2037)
+		goto exit;
+
+	if (x < 0 || y <= 0)
+		return 1;
+	if (INT_MAX / y < x)
+		return 1;
+   exit:
+	*z = x * y;
+	return 0;
+}
