@@ -79,16 +79,17 @@ int days[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  * variables to store data path names, which are initialized in
  * io_init()
  */
-char path_dir[] = "";
-char path_todo[] = "";
-char path_apts[] = "";
-char path_conf[] = "";
-char path_notes[] = "";
-char path_keys[] = "";
-char path_cpid[] = "";
-char path_dpid[] = "";
-char path_dmon_log[] = "";
-char path_hooks[] = "";
+char *path_ddir = NULL;
+char *path_cdir = NULL;
+char *path_todo = NULL;
+char *path_apts = NULL;
+char *path_conf = NULL;
+char *path_notes = NULL;
+char *path_keys = NULL;
+char *path_cpid = NULL;
+char *path_dpid = NULL;
+char *path_dmon_log = NULL;
+char *path_hooks = NULL;
 
 /* Variable to store global configuration. */
 struct conf conf;
@@ -102,6 +103,15 @@ struct nbar nbar;
 /* Variable to store daemon configuration. */
 struct dmon_conf dmon;
 
+/*
+ * Thread id variables for threads that never exit.
+ *
+ * Each variable either carries the identifier of the corresponding thread. If
+ * one of the threads is not running, the corresponding variable is assigned
+ * the identifier of the main thread instead.
+ */
+pthread_t notify_t_main, io_t_psave, ui_calendar_t_date;
+
 /*
  * Variables init
  */
@@ -110,15 +120,22 @@ void vars_init(void)
 	const char *ed, *pg, *mt;
 
 	/* Variables for user configuration */
+	conf.cal_view = CAL_MONTH_VIEW;
+	conf.todo_view = TODO_HIDE_COMPLETED_VIEW;
+	conf.multiple_days = 1;
+	conf.header_line = 1;
+	conf.event_separator = 1;
+	conf.day_separator = 1;
+	conf.empty_appt_line = 1;
 	conf.confirm_quit = 1;
 	conf.confirm_delete = 1;
 	conf.auto_save = 1;
 	conf.auto_gc = 0;
 	conf.periodic_save = 0;
+	conf.systemevents = 1;
 	conf.default_panel = CAL;
 	conf.compact_panels = 0;
 	conf.system_dialogs = 1;
-	conf.progress_bar = 1;
 	strncpy(conf.output_datefmt, "%D", 3);
 	conf.input_datefmt = 1;
 	conf.heading_pos = RIGHT;
@@ -166,4 +183,7 @@ void vars_init(void)
 
 	/* Start at the current date */
 	ui_calendar_init_slctd_day();
+
+	/* Threads not yet running. */
+	notify_t_main = io_t_psave = ui_calendar_t_date = pthread_self();
 }
