@@ -315,14 +315,25 @@ void wins_scrollwin_resize(struct scrollwin *sw, int y, int x, int h, int w)
 	delwin(sw->win);
 	sw->win = newwin(h, w, y, x);
 	sw->inner = newpad(BUFSIZ, w);
+	wins_scrollwin_set_pad(sw, sw->line_num);
 }
 
 /*
- * Set the number of lines to be displayed.
+ * Set the number of lines (pad size) and the line offset
+ * (the part of the pad to be displayed in the viewport).
  */
-void wins_scrollwin_set_linecount(struct scrollwin *sw, unsigned lines)
+void wins_scrollwin_set_pad(struct scrollwin *sw, unsigned lines)
 {
+	int inner_h = sw->h - (conf.compact_panels ? 2 : 4);
+
 	sw->line_num = lines;
+	if (lines > inner_h) {
+		/* pad partly displayed in viewport */
+		if (sw->line_off > lines - inner_h)
+			/* offset too big */
+			sw->line_off = lines - inner_h;
+	} else
+		sw->line_off = 0;
 }
 
 /* Free an already created scrollwin. */
@@ -337,7 +348,7 @@ void wins_scrollwin_delete(struct scrollwin *sw)
 void wins_scrollwin_draw_deco(struct scrollwin *sw, int hilt)
 {
 	if (hilt)
-		wattron(sw->win, A_BOLD | COLOR_PAIR(COLR_CUSTOM));
+		custom_apply_attr(sw->win, ATTR_HIGHEST);
 
 	box(sw->win, 0, 0);
 
@@ -348,34 +359,41 @@ void wins_scrollwin_draw_deco(struct scrollwin *sw, int hilt)
 	}
 
 	if (hilt)
-		wattroff(sw->win, A_BOLD | COLOR_PAIR(COLR_CUSTOM));
+		custom_remove_attr(sw->win, ATTR_HIGHEST);
 
 	if (!conf.compact_panels)
 		print_in_middle(sw->win, 1, 0, sw->w, sw->label);
 }
 
 /* Display a scrolling window. */
-void wins_scrollwin_display(struct scrollwin *sw)
+void wins_scrollwin_display(struct scrollwin *sw, int hilt)
 {
-	int inner_y = (conf.compact_panels ? 1 : 3);
-	int inner_x = 1;
+	int inner_y = sw->y + (conf.compact_panels ? 1 : 3);
+	int inner_x = sw->x + 1;
 	int inner_h = sw->h - (conf.compact_panels ? 2 : 4);
 	int inner_w = sw->w - 2;
 
-	if (sw->line_num > inner_h) {
-		int sbar_h = MAX(inner_h * inner_h / sw->line_num, 1);
-		int sbar_y = inner_y + sw->line_off * (inner_h - sbar_h) / (sw->line_num - inner_h);
-		int sbar_x = sw->w - 1;
-
-		draw_scrollbar(sw->win, sbar_y, sbar_x, sbar_h, inner_y,
-			       inner_y + inner_h - 1, 1);
-	}
-
+	if (sw->line_num > inner_h)
+		draw_scrollbar(sw, hilt);
 	wmove(win[STA].p, 0, 0);
 	wnoutrefresh(sw->win);
-	pnoutrefresh(sw->inner, sw->line_off, 0, sw->y + inner_y,
-		     sw->x + inner_x, sw->y + inner_y + inner_h - 1,
-		     sw->x + inner_x + inner_w - 1);
+	/*
+	(sw->line_off, 0): upper left corner in pad to display;
+	scrolling is achieved by in/decreasing sw->line_off
+	between 0 and line_num - inner.h
+
+	(Y,X)=(inner_y,inner_x): upper left corner in screen window
+	(Y,X) + (inner_h - 1,inner_w - 1): lower right corner in screen window
+
+                        |-inner_w-|
+	        (Y,X)-> +---------+     -
+	                |         |     |
+	                |  (pad)  |    inner_h
+	                |         |     |
+	                +---------+ <-  -
+	*/
+	pnoutrefresh(sw->inner, sw->line_off, 0, inner_y, inner_x,
+		     inner_y + inner_h - 1, inner_x + inner_w - 1);
 	wins_doupdate();
 }
 
@@ -397,28 +415,17 @@ void wins_scrollwin_down(struct scrollwin *sw, int amount)
 		sw->line_off = sw->line_num - inner_h;
 }
 
-int wins_scrollwin_is_visible(struct scrollwin *sw, unsigned line)
-{
-	int inner_h = sw->h - (conf.compact_panels ? 2 : 4);
-	return ((line >= sw->line_off) && (line < sw->line_off + inner_h));
-}
-
-void wins_scrollwin_ensure_visible(struct scrollwin *sw, unsigned line)
+/* Keep 'line' in the viewport of a scrollwindow. */
+void wins_scrollwin_in_view(struct scrollwin *sw, unsigned line)
 {
 	int inner_h = sw->h - (conf.compact_panels ? 2 : 4);
 
 	if (line < sw->line_off)
 		sw->line_off = line;
-	else if (line >= sw->line_off + inner_h)
+	else if (line > sw->line_off + inner_h - 1)
 		sw->line_off = line - inner_h + 1;
 }
 
-void wins_scrollwin_set_lower(struct scrollwin *sw, unsigned line)
-{
-	int inner_h = sw->h - (conf.compact_panels ? 2 : 4);
-	sw->line_off = MAX((int)line - inner_h + 1, 0);
-}
-
 void wins_resize_panels(void)
 {
 	wins_get_config();
@@ -540,9 +547,9 @@ void wins_update_border(int flags)
 void wins_update_panels(int flags)
 {
 	if (flags & FLAG_APP)
-		ui_day_update_panel(slctd_win);
+		ui_day_update_panel(slctd_win == APP);
 	if (flags & FLAG_TOD)
-		ui_todo_update_panel(slctd_win);
+		ui_todo_update_panel(slctd_win == TOD);
 	if (flags & FLAG_CAL)
 		ui_calendar_update_panel();
 }
@@ -606,7 +613,6 @@ void wins_unprepare_external(void)
 	ui_mode = UI_CURSES;
 	wins_refresh();
 	wins_resize();
-	wins_update(FLAG_ALL);
 	if (notify_bar())
 		notify_start_main_thread();
 	if (conf.periodic_save > 0)
@@ -730,7 +736,7 @@ void wins_erase_status_bar(void)
 }
 
 /* Update the status bar page number to display other commands. */
-void wins_other_status_page(int panel)
+void wins_other_status_page()
 {
 	/*
 	 * Determine the number of completely filled pages of key bindings.
